#! /bin/bash
## The chunk.sh shell script (from the classpack package v.1.28, 2024-02-21)
## Generated by ClassPack v.1.28 (2024-02-21) on 24-02-26T13:47:16
# Bourne shell script to create chunks of a long document
# splitting on named boundaries in [doubled] comment characters
# or (XML) Processing Instructions. See §6.3.4 of Classpack PDF

# 1.3 added dbincludes to assist post-editing

VERSION=1.3
VDATE="2024-02-19"
DEV=~/texmf/dev
DIR=`pwd`
APP=`basename $DIR`/`basename $DIR`.xml
SCRIPT=~/bin/chunk.awk

if [ -z "$1" ]; then
    cat <<EOF
Synopsis

$ chunk -p name file

  -p  specifies a PI name to use for splitting XML (no default)

  file  name of the XML document from dev to process

Output files are named after the values of the chunk boundaries
eg filename-xxxxxx.ext
EOF
    exit 0
else
    while [[ $# -gt 0 ]]; do
key="$1"
case $key in
    -p)
BOUNDARY="$2"
shift
shift
;;
    *)
if [ ! -s "$DEV/$key" ]; then
    echo No such file as "$DEV/$key"
    exit 1
else
    INPUT="$DEV/$key"
    EXT=`echo $INPUT | awk -F. '{print $NF}'`
    FNAME=`basename $INPUT .$EXT`
    if [ -z "$BOUNDARY" ]; then
if [ "$EXT" = "tex" ]; then
    CHUNKS=`grep '^%% [A-Za-z][A-Za-z\-]* %%' $INPUT |\
wc -l`
    BOUNDARY="%%"
elif [ "$EXT" = "css" ]; then
    CHUNKS=`grep '^/* [A-Za-z][A-Za-z\-]* */' $INPUT |\
wc -l`
    BOUNDARY="/* */"
elif [ "$EXT" = "awk" -o "$EXT" = "make" ]; then
    CHUNKS=`grep '^## [A-Za-z][A-Za-z\-]* ##' $INPUT |\
wc -l`
    BOUNDARY="##"
else # scripts with no filetype
    CHUNKS=`grep '^## [A-Za-z][A-Za-z\-]* ##' $INPUT |\
wc -l`
    BOUNDARY="##"
fi
    else
if [ "$EXT" = "xml" -o "$EXT" = "xsl" -o "$EXT" = "dtd" ]; then
    CHUNKS=`grep "^<?$BOUNDARY [A-Za-z][A-Za-z\-]*[ ]*?>" $INPUT|wc -l`
else
    echo The -p PI boundary cannot be used with non-XML/XSLT files
    exit 3
fi
    fi
    if [ "$CHUNKS" -eq 0 ]; then
echo No $BOUNDARY fragments found in $INPUT
echo Nothing to do
exit 4
    else
echo $CHUNKS $BOUNDARY fragments found in $INPUT
    fi
fi
shift
;;
esac
    done
fi

cat <<EOF
This is fragment chunk, v.$VERSION ($VDATE) by Silmaril for ClassPack
Read from $INPUT, chunking on $CHUNKS $BOUNDARY boundaries
EOF

cat <<EOF >$FNAME.chunks
<chunks source="$INPUT" date="`date --iso-8601=seconds`"
        boundary="$BOUNDARY" xmlns:xlink="http://www.w3.org/1999/xlink">
EOF

cat $INPUT | awk -v boundary="$BOUNDARY" \
 -v fname="$FNAME" \
 -v ext="$EXT" \
 -f $SCRIPT

LINES=`grep -v $BOUNDARY $INPUT | wc -l`
echo $LINES "lines of code and comment"

cat <<EOF >>$FNAME.chunks
</chunks>
EOF

# Summaries

lxprintf -e programlisting[@xlink:href=\"$3\"][@conformance=\"frag\"] \
 "%s\n" @startinglinenumber $DEV/$APP >$FNAME.doclist

lxprintf -e programlisting[@xlink:href=\"$3\"][@conformance=\"frag\"] \
 "%s\n" @startinglinenumber $FNAME.chunks >$FNAME.actual

COMP=`diff $FNAME.doclist $FNAME.actual`

cat <<EOF >$FNAME-dbincludes.xml
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE sect1
  PUBLIC "+//DTD Silmaril//DocBook 5 adapted for ClassPack//EN"
         "../doctexbook-master.dtd">
      <sect1>
        <title></title>
EOF
cat $FNAME.actual | while read CHUNKNAME; do

    cat <<EOF >>$FNAME-dbincludes.xml
        <annotation role="" xreflabel="$FNAME">
          <para></para>
          <programlisting xlink:href="$3"
            language="XML" conformance="frag"
            startinglinenumber="$CHUNKNAME"/>
        </annotation>
EOF

done
cat <<EOF >>$FNAME-dbincludes.xml
      </sect1>
EOF

if [ -n "$COMP" ]; then
    echo $DEV/$APP and $FNAME.xml have different chunk references
    exit 5
fi

exit

