<!-- The db2dtx.xsl script (from the classpack package v.1.28, 2024-02-21) -->
<!-- Generated by ClassPack v.1.28 (2024-02-21) on 24-02-26T13:47:16 -->
<!-- DEPENDENCIES: doctexbook.dtd, docbook5.dtd,
     db2bibtex.xsl, db2md.xsl, figtab2latex.xsl,
     rfc2119.xml, classpack.xml, prepost.xml,
     languages.xml, readme.xml, representation.xml,
     lppl.xml (or other specified licence).
     This program accompanies v1.26 of ClassPack.
-->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:db="http://docbook.org/ns/docbook"
  xmlns:sil="http://xml.silmaril.ie/identity/functions"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  version="3.1"
  exclude-result-prefixes="xsl db xlink sil xs">
  <xsl:output method="text"/>
  <xsl:output method="text" name="textFormat"/>
  <xsl:output method="xml" name="xmlFormat" indent="yes"
    doctype-system="../doctexbook.dtd"/>
  <xsl:output method="xml" name="htmlFormat" indent="yes"
    doctype-system="about:legacy-compat" omit-xml-declaration="yes"/>

  <xsl:strip-space elements="*"/>

  <xsl:include href="db2bibtex.xsl"/>
  <xsl:include href="db2md.xsl"/>
  <xsl:include href="figtab2latex.xsl"/>
  <xsl:param name="armour">
    <xsl:text>% </xsl:text>
  </xsl:param>
  <xsl:variable name="shield">
    <xsl:text>%</xsl:text>
  </xsl:variable>

  <!-- path to the classpack directory -->
  <xsl:param name="cpdir"/>
  <!-- path to the project’s app dev directory -->
  <xsl:param name="appdir"/>
  <xsl:variable name="thisversion"
    select="document(concat($cpdir,
                            '/classpack/classpack.xml'))
            /db:book/db:info/db:revhistory/db:revision
            [not(../db:revision/@version > @version)]
            /@version"/>
  <xsl:variable name="thisverdate"
    select="document(concat($cpdir,
                            '/classpack/classpack.xml'))
            /db:book/db:info/db:revhistory/db:revision
            [not(../db:revision/@version > @version)]
            /db:date/@YYYY-MM-DD"/>
  <xsl:variable name="prepost"
    select="document(concat($cpdir,'/prepost.xml'))
    /db:refsection"/>
  <xsl:variable name="langs"
    select="document(concat($cpdir,'/languages-master.xml'))
            /languages"/>
  <xsl:variable name="readme"
    select="document(concat($cpdir,'/readme-master.xml'))
            /db:chapter"/>
  <xsl:variable name="licence"
    select="document(concat($appdir,'/',
            /db:book/@audience,'.xml'))
            /db:chapter"/>
  <xsl:variable name="thisdoc" select="/"/>
  <xsl:variable name="maxcodelen">
    <xsl:text>40</xsl:text>
  </xsl:variable>

  <xsl:variable name="docname" select="/db:book/@xml:id"/>
  <xsl:variable name="doctitle">
    <xsl:call-template name="detex">
      <xsl:with-param name="string"
        select="/db:book/db:info/db:title/node()"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="firstverdate"
    select="/db:book/db:info/db:revhistory/db:revision
            [not(../db:revision/@version &lt; @version)]
            /db:date/@YYYY-MM-DD"/>
  <xsl:variable name="name">
    <xsl:choose>
      <!-- if there is more than one code appendix,
           use the ID of the first chapter in the code part
           as the 'name' of this bundle -->
      <xsl:when
        test="count(/db:book/db:part[@xml:id='code']
                    //db:appendix
                    [substring-after(@xlink:href,'.')
                    = 'cls'])>1">
        <xsl:value-of
          select="/db:book/db:part[@xml:id='code']
                  /db:chapter[1]/@xml:id"/>
      </xsl:when>
      <!-- otherwise use the ID of this document -->
      <xsl:otherwise>
        <xsl:value-of select="$docname"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="doctype" select="/db:book/@arch"/>
  <xsl:variable name="version" select="/db:book/@version"/>
  <xsl:variable name="revision" select="/db:book/@revision"/>
  <xsl:variable name="filetype" select="/db:book/@userlevel"/>
  <xsl:variable name="date"
    select="/db:book/db:info/db:revhistory/db:revision
            [not(../db:revision/@version > @version)]
            /db:date/@YYYY-MM-DD"/>
  <xsl:variable name="latestrevhist"
    select="/db:book/db:info/db:revhistory/db:revision
            [not(../db:revision/@version > @version)]
            /@version"/>
  <!-- comment characters depend on file type
       so extractable chapters/appendixes MUST have
       an xlink:href ending with one of these filetypes -->
  <xsl:variable name="commentchars">
    <class    filetype="cls">%</class>
    <document filetype="bib">%</document>
    <document filetype="tex">%</document>
    <document filetype="cnf">%</document>
    <document filetype="def">%</document>
    <document filetype="dtd" term="--&gt;">&lt;!--</document>
    <document filetype="xml" term="--&gt;">&lt;!--</document>
    <package  filetype="sty">%</package>
    <script   filetype="cp">#</script>
    <script   filetype="sh">#</script>
    <script   filetype="awk">#</script>
    <script   filetype="xsl" term="--&gt;">&lt;!--</script>
  </xsl:variable>

  <!-- Default shield is double delimiter plus space -->
  <xsl:variable name="sentinel"
    select="concat(
            $commentchars/*[name()=current()/db:book/@arch]
            [@filetype=current()/db:book/@userlevel],
            $commentchars/*[name()=current()/db:book/@arch]
            [@filetype=current()/db:book/@userlevel],' ')"/>
  <xsl:variable name="latexprog">
    <xsl:choose>
      <xsl:when
        test="/db:book/db:info/db:cover/db:constraintdef
        [@xml:id=concat(/db:book/@userlevel,'packages')]
        /@arch!=''">
        <xsl:value-of
          select="/db:book/db:info/db:cover
          /db:constraintdef
          [@xml:id=concat(/db:book/@userlevel,'packages')]
          /@arch"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>pdflatex</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:template match="/">
   <xsl:message>
      <xsl:text>This is DB2DTX, Version </xsl:text>
      <xsl:value-of select="$thisversion"/>
      <xsl:text> (</xsl:text>
      <xsl:value-of select="$thisverdate"/>
      <xsl:text>).</xsl:text>
    </xsl:message>
    <xsl:variable name="fence">
      <xsl:call-template name="makefence"/>
    </xsl:variable>
    <!-- stop if the version numbers don't accord -->
    <xsl:choose>
      <xsl:when test="concat($version,'.',$revision)
                      != $latestrevhist">
        <xsl:message>
          <xsl:text>! Declared version </xsl:text>
          <xsl:value-of
            select="concat($version,'.',$revision)"/>
          <xsl:text> does not match latest </xsl:text>
          <xsl:text>revision history </xsl:text>
          <xsl:value-of select="$latestrevhist"/>
          <xsl:text>&#xa;  I'm sorry, </xsl:text>
          <xsl:text>I can't go on </xsl:text>
          <xsl:text>until you fix this.</xsl:text>
        </xsl:message>
      </xsl:when>
      <xsl:otherwise>
        <!-- output the .ins file first -->
        <xsl:apply-templates
          select="/db:book/db:info" mode="ins"/>
        <!-- then the README.md -->
        <xsl:call-template name="readme"/>
        <!-- and then the MANIFEST -->
        <xsl:call-template name="manifest"/>
        <!-- and then the index.html for the Silmaril site -->
        <xsl:call-template name="webindex"/>
        <!-- start creating the .dtx file -->
        <xsl:message>
          <xsl:text>Making </xsl:text>
          <xsl:value-of select="$doctype"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="$docname"/>
        </xsl:message>
        <xsl:text>% \iffalse meta-comment&#xa;%&#xa;</xsl:text>
        <xsl:call-template
          name="copyright-statement">
          <xsl:with-param name="ftype">
            <xsl:text>dtx</xsl:text>
          </xsl:with-param>
        </xsl:call-template>
        <xsl:text>%
        <!-- SPECIAL EXCEPTION FOR CLASSES:
             needs \ProvidesFile -->
        <xsl:if test="$doctype='class'">
          <xsl:text>%&lt;*driver>
\ProvidesFile{</xsl:text>
          <xsl:value-of select="$docname"/>
          <xsl:text>.dtx}
</xsl:text>
        </xsl:if>
        <xsl:choose>
          <!-- SPECIAL EXCEPTION FOR SHELL SCRIPTS: -->
          <xsl:when test="$doctype='script'">
            <!-- shell call -->
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>></xsl:text>
            <xsl:value-of select="$commentchars"/>
            <xsl:text>! /bin/</xsl:text>
            <xsl:value-of select="/db:book/@userlevel"/>
            <xsl:text>&#xa;</xsl:text>
            <!-- author, affil, date -->
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>></xsl:text>
            <xsl:value-of select="$fence"/>
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>></xsl:text>
            <xsl:value-of select="$sentinel"/>
            <xsl:value-of
              select="/db:book/db:info/db:author[1]
                      /db:personname/db:firstname"/>
            <xsl:text> </xsl:text>
            <xsl:value-of
              select="/db:book/db:info/db:author[1]
                      /db:personname/db:surname"/>
            <xsl:text>, </xsl:text>
            <xsl:value-of
              select="/db:book/db:info/db:author[1]
                      /db:affiliation/db:orgname"/>
            <xsl:text>, </xsl:text>
            <xsl:value-of
              select="format-date(
                      /db:book/db:info
                      /db:revhistory/db:revision
                      [@version=max(parent::db:revhistory
                                    /db:revision/@version)]
                      /db:date/@YYYY-MM-DD,
                      '[D] [MNn] [Y]')"/>
            <xsl:text>&#xa;</xsl:text>
            <!-- history -->
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>></xsl:text>
            <xsl:value-of select="$fence"/>
            <xsl:for-each
              select="/db:book/db:info
                      /db:revhistory/db:revision">
              <xsl:sort select="@version"
                order="ascending"/>
              <xsl:text>%&lt;</xsl:text>
              <xsl:value-of select="$doctype"/>
              <xsl:text>></xsl:text>
              <xsl:value-of select="$sentinel"/>
              <xsl:value-of select="@version"/>
              <xsl:text> </xsl:text>
              <xsl:value-of
                select="db:revdescription/db:itemizedlist
                        /db:title"/>
              <xsl:text>: </xsl:text>
              <xsl:for-each
                select="db:revdescription/db:itemizedlist
                        /db:listitem">
                <xsl:value-of select="normalize-space(.)"/>
              </xsl:for-each>
              <xsl:text> </xsl:text>
              <xsl:value-of select="db:date/@YYYY-MM-DD"/>
              <xsl:text>&#xa;</xsl:text>
            </xsl:for-each>
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>></xsl:text>
            <xsl:value-of select="$fence"/>
            <!-- version and date -->
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>>VERSION=</xsl:text>
            <xsl:value-of
              select="/db:book/db:info
                      /db:revhistory/db:revision
                      [@version=max(parent::db:revhistory
                                    /db:revision/@version)]
                      /@version"/>
            <xsl:text>&#xa;</xsl:text>
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>>VERSDATE=</xsl:text>
            <xsl:value-of
              select="/db:book/db:info
                      /db:revhistory/db:revision
                      [@version=max(parent::db:revhistory
                                    /db:revision/@version)]
                      /db:date/@YYYY-MM-DD"/>
            <xsl:text>&#xa;</xsl:text>
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>>PROGRAM=`echo $0 |</xsl:text>
            <xsl:text>awk -F/ '{print $NF}' |</xsl:text>
            <xsl:text>awk -F. '{print $1}'`</xsl:text>
            <xsl:text>&#xa;</xsl:text>
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>></xsl:text>
            <xsl:value-of select="$fence"/>
          </xsl:when>
          <!-- end of block for shell scripts -->
          <!-- ELSE FOR PACKAGES AND CLASSES -->
          <xsl:otherwise>
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>>\NeedsTeXFormat{</xsl:text>
            <xsl:value-of select="/db:book/@conformance"/>
            <xsl:text>}[</xsl:text>
            <xsl:value-of
              select="translate(/db:book/@condition,
                                '-','/')"/>
            <xsl:text>]&#xa;</xsl:text>
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>>\Provides</xsl:text>
            <xsl:value-of
              select="upper-case(substring($doctype,1,1))"/>
            <xsl:value-of select="substring($doctype,2)"/>
            <xsl:text>{</xsl:text>
            <xsl:value-of select="$docname"/>
            <xsl:text>}[</xsl:text>
            <!-- use latest rev date as distro date -->
            <xsl:value-of
              select="translate($date,'-','/')"/>
            <xsl:text> v</xsl:text>
            <xsl:value-of select="/db:book/@version"/>
            <xsl:text>.</xsl:text>
            <xsl:value-of select="/db:book/@revision"/>
            <xsl:text>&#xa;%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>>  </xsl:text>
            <xsl:variable name="title">
              <xsl:choose>
                <!-- use TeX-parsed content for code title -->
                <xsl:when test="//db:part[@xml:id='code']
                                /db:title">
                  <xsl:value-of
                    select="normalize-space(replace(
                            //db:part[@xml:id='code']
                            /db:title,
                            '\\LaTeX\{\}','LaTeX'))"/>
                </xsl:when>
                <xsl:when test="//db:info/db:subtitle">
                  <xsl:value-of
                    select="normalize-space(
                            //db:info/db:subtitle)"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of
                    select="normalize-space(
                            //db:info/db:title)"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <xsl:call-template name="delogify">
              <xsl:with-param name="string"
                select="$title"/>
            </xsl:call-template>
            <xsl:text>]&#xa;</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
<step condition="doc" remap="makeidx">
  <para>Package for creating indexes.</para>
  <constructorsynopsis condition=""/>
</step>
<step remap="url" condition="doc">
  <para>Handling of URI formatting.</para>
  <constructorsynopsis condition="link"/>
  <constructorsynopsis condition="uri"/>
  <constructorsynopsis condition="email"/>
</step>
        <!-- AUTOPACKAGE STARTS HERE -->
        <!-- 1. CREATE THE LIST OF PACKAGES NEEDED FOR DOCUMENTATION -->
        <xsl:variable name="packages">
          <!-- Create a list of <seglistitem>s -->
          <xsl:call-template name="makepackages"/>
        </xsl:variable>
        <!-- 2. OUTPUT THE LIST FOR REFERENCE, unsorted and unpruned -->
        <xsl:result-document format="xmlFormat"
          href="autopack.xml">
          <db:segmentedlist arch="{$doctype}" remap="{$docname}">
            <xsl:copy-of select="$packages"/>
          </db:segmentedlist>
        </xsl:result-document>
        <!-- 3. NON-DEFERRED PACKAGES (u/s) -->
        <xsl:call-template name="nondeferredpackages"/>
        <!-- unsure about them -->
        <!-- 4. START THE CODE FOR THE DOCUMENTATION (the "driver") -->
        <xsl:text>%&lt;*driver>&#xa;</xsl:text>
        <!-- 5a. do the compulsory preloads first -->
        <xsl:for-each select="$packages/db:seglistitem
                              [db:seg/@annotations='required']">
          <!-- note that the inverse of this condition is needed
               later to PREVENT this package being loaded on a
               second or subsequent time -->
          <xsl:text>\RequirePackage</xsl:text>
          <xsl:if test="db:seg/@role!=''">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="db:seg/@role"/>
            <xsl:text>]</xsl:text>
          </xsl:if>
          <xsl:text>{</xsl:text>
          <xsl:value-of select="db:seg"/>
          <xsl:text>}% </xsl:text>
          <xsl:text>because "</xsl:text>
          <xsl:value-of select="@role"/>
          <xsl:text>" :: "</xsl:text>
          <xsl:value-of select="@arch"/>
          <xsl:text>"&#xa;</xsl:text>
        </xsl:for-each>
        <!-- end of compulsory preloads -->
        <!-- @arch here is the class of autopackage mapping -->
        <!--      |
                  $packages/db:seglistitem
                    [db:seg=$thisdoc//db:constraintdef
                    [@xml:id='stypackages']//db:seg[@role]]
             this was removed to stop it affecting sty packages -->
        <xsl:for-each-group
          select="$packages/db:seglistitem
                    [@arch='dependencies']
                    [db:seg/@annotations='optional']
                  |
                  $packages/db:seglistitem
                    [@arch='requests']
                    [db:seg[@role]=$packages/db:seglistitem
                                     [@arch!='requests']/db:seg]
                  |
                  $packages/db:seglistitem
                    [@arch='defaults']
                    [db:seg=$prepost/db:procedure
                              [@xml:id='prepackage']
                              /db:step[@role and @performance]
                              /@remap]"
          group-by="db:seg">
          <xsl:text>\PassOptionsToPackage{</xsl:text>
          <!-- use the @role on the <seg>, at least one of them has one
               but create them as a variable so we can group them -->
          <xsl:variable name="options">
            <xsl:for-each select="current-group()">
              <xsl:if test="db:seg/@role">
                <xsl:for-each select="tokenize(db:seg/@role,' ')">
                  <opt>
                    <xsl:value-of select="."/>
                  </opt>
                </xsl:for-each>
              </xsl:if>
              <xsl:if test="@arch='dependencies'
                            and @audience
                            and @role">
                <xsl:for-each select="tokenize(@role,' ')">
                  <opt>
                    <xsl:value-of select="."/>
                  </opt>
                </xsl:for-each>
              </xsl:if>
            </xsl:for-each>
            <!-- also use the @role in the <seglistitem> -->
            <xsl:for-each select="tokenize(@role,' ')">
              <opt>
                <xsl:value-of select="."/>
              </opt>
            </xsl:for-each>
          </xsl:variable>
          <xsl:for-each-group select="$options" group-by="opt">
            <xsl:if test="position()>1">
              <xsl:text>,</xsl:text>
            </xsl:if>
            <xsl:value-of select="current-grouping-key()"/>
          </xsl:for-each-group>
          <!-- say what -->
          <xsl:message>
            <xsl:text>Passing options "</xsl:text>
             <xsl:for-each-group select="$options" group-by="opt">
            <xsl:if test="position()>1">
              <xsl:text>,</xsl:text>
            </xsl:if>
            <xsl:value-of select="current-grouping-key()"/>
          </xsl:for-each-group>
          <xsl:text>" to package </xsl:text>
            <xsl:value-of select="db:seg"/>
          </xsl:message>
          <xsl:text>}{</xsl:text>
          <xsl:value-of select="db:seg"/>
          <xsl:text>}% because </xsl:text>
          <xsl:value-of select="@arch"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="@audience | @condition"/>
          <xsl:text> requires </xsl:text>
          <xsl:value-of select="@role"/>
          <xsl:text>&#xa;</xsl:text>
        </xsl:for-each-group>
        <!-- end of 5b. passing options to packages -->
        <!-- 6. ADD COMMANDS to hold the docname and doctype -->
        <xsl:text>\providecommand{\CPKdocname}{</xsl:text>
        <xsl:value-of select="/db:book/@xml:id"/>
        <xsl:text>}&#xa;\providecommand{\CPKdoctype}{</xsl:text>
        <xsl:value-of select="/db:book/@arch"/>
        <xsl:text>}&#xa;</xsl:text>
        <!-- SPECIAL FOR BABEL, other languages preloaded
             This kind of dependency can't be expressed in
             prepost.xml -->
        <xsl:variable name="baselang">
          <xsl:choose>
            <xsl:when test="/db:book/@xml:lang">
              <xsl:value-of select="/db:book/@xml:lang"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>en-IE</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <!-- see what other langs are used: only in the 'doc'
             part and its descendants; if @xml:lang is
             given *and* if it's non-null *and* if it's
             not the $baselang we already snagged above -->
        <xsl:if test="/db:book/db:part[@xml:id='doc']
                      /descendant-or-self::*[@xml:lang]
                      [@xml:lang!='']
                      [@xml:lang!=$baselang]
                      or
                      /db:book/@xml:lang">
          <xsl:text>\PassOptionsToPackage{</xsl:text>
          <xsl:variable name="loadedlangs">
            <xsl:for-each-group group-by="."
              select="/db:book/db:part[@xml:id='doc']
                      /descendant-or-self::*[@xml:lang]
                      [@xml:lang!='']
                      [@xml:lang!=$baselang]/@xml:lang">
              <xsl:value-of
                select="$langs/language
                        [(string-length(current-grouping-key())>2
                          and concat(@Lang,'-',@Country)
                             =current-grouping-key())
                         or
                         (string-length(current-grouping-key())=2
                          and
                          @Lang=current-grouping-key())]
                        [position()=last()]/@BabelName"/>
              <xsl:text>,</xsl:text>
            </xsl:for-each-group>
            <xsl:value-of
              select="$langs/language
                      [(string-length($baselang)>2
                        and concat(@Lang,'-',@Country)
                        =$baselang)
                       or
                       (string-length($baselang)=2
                        and
                        @Lang=$baselang)]
                      [position()=last()]/@BabelName"/>
          </xsl:variable>
          <xsl:value-of select="$loadedlangs"/>
          <xsl:text>}{babel}% use of other (non-</xsl:text>
          <xsl:value-of select="$baselang"/>
          <xsl:text>) languages&#xa;</xsl:text>
        </xsl:if>
        <xsl:text>\documentclass</xsl:text>
        <xsl:if test="/db:book/@remap">
          <xsl:text>[</xsl:text>
          <xsl:value-of select="/db:book/@remap"/>
          <xsl:text>]</xsl:text>
        </xsl:if>
        <xsl:text>{ltxdoc}&#xa;</xsl:text>
        <!-- RATIONALISATION OF ANY PACKAGES DETECTED -->
        <xsl:if test="count($packages/db:seglistitem)>0">
          <!-- They need to be handled in the order in which they occur
               in prepost.xml because that determines the sequence in
               which packages get loaded. Because packages may occur
               more than once in $packages (eg loaded by deduction,
               and loaded by request) we can use grouping to make sure
               multiple entries get handled together. The order from
               prepost.xml is carried in the @userlevel attribute BUT
               this used to be overridden for any packages requested
               by the author, in order to ensure they come last, but
               this is probably A Bad Idea and has been reverted,
               see under APauthorreq -->
          <xsl:text>%%
%% Packages for documentation
%%&#xa;</xsl:text>
          <!-- sort by package userlevel for the order from prepost.xml
               because it's unique, although a package can still get
               specified multiple times, so within it, sort by
               @security, which hold the package name requested by
               the author -->
          <!-- grouping by @userlevel lets us identify duplicates -->
          <xsl:for-each-group select="$packages/db:seglistitem"
            group-by="@userlevel">
            <xsl:sort select="@security"/>
            <!-- maxlevel should no longer be needed -->
            <xsl:variable name="maxlevel"
              select="current-group()
                      [not(current-group()/@userlevel
                           &lt; @userlevel)][1]/@userlevel"/>
            <xsl:choose>
              <!-- OMIT if another entry (not this one)
                   has this package name specified in @remap
                   eg fontspec will mention inputenc and fontenc
                   both of which need omitting if fontspec is used -->
              <xsl:when
                test="$packages/db:seglistitem
                      [not(@userlevel=current-grouping-key())]
                      [contains(@remap,current-group()[1]/db:seg)]">
                <xsl:text>%% </xsl:text>
                <xsl:value-of select="current-group()[1]/db:seg"/>
                <xsl:text> (</xsl:text>
                <xsl:value-of select="$maxlevel"/>
                <xsl:text>) omitted in favour of </xsl:text>
                <xsl:value-of
                  select="$packages/db:seglistitem
                          [not(@userlevel=current-grouping-key())]
                          [contains(@remap,current-group()[1]/db:seg)]
                          /db:seg"/>
                <xsl:text>&#xa;</xsl:text>
                <xsl:message>
                  <xsl:if test="$maxlevel&lt;100">
                    <xsl:text> </xsl:text>
                    <xsl:if test="$maxlevel&lt;10">
                      <xsl:text> </xsl:text>
                    </xsl:if>
                  </xsl:if>
                  <xsl:value-of select="$maxlevel"/>
                  <xsl:text>. Package '</xsl:text>
                  <xsl:value-of
                    select="current-group()"/>
                  <xsl:text>' (</xsl:text>
                  <xsl:value-of
                    select="count(current-group())"/>
                  <xsl:text>×) omitted: transcluded by </xsl:text>
                  <xsl:value-of
                    select="$packages/db:seglistitem
                            [contains(@remap,current-group()[1]/db:seg)]
                            /db:seg"/>
                </xsl:message>
              </xsl:when>
              <!-- OMIT packages flagged in conformance
                   (eg nosc, unclear legacy) -->
              <!--
              <xsl:when
                test="$packages/db:seglistitem
                      [db:seg=current()/@conformance]">
                <xsl:message>
                  <xsl:value-of select="$maxlevel"/>
                  <xsl:text>. Package '</xsl:text>
                  <xsl:value-of
                    select="current-grouping-key()"/>
                  <xsl:text>' (</xsl:text>
                  <xsl:value-of
                    select="count(current-group())"/>
                  <xsl:text>×) omit: </xsl:text>
                  <xsl:text>(conformance="</xsl:text>
                  <xsl:value-of
                    select="$packages/db:seglistitem
                            /db:seg"/>
                  <xsl:text>")</xsl:text>
                </xsl:message>
              </xsl:when>
              -->
              <!-- OTHERWISE USE THE PACKAGE -->
              <xsl:otherwise>
                <xsl:message>
                  <xsl:if test="number($maxlevel)&lt;100">
                    <xsl:text> </xsl:text>
                    <xsl:if test="number($maxlevel)&lt;10">
                      <xsl:text> </xsl:text>
                    </xsl:if>
                  </xsl:if>
                  <xsl:value-of select="$maxlevel"/>
                  <xsl:text>. Package '</xsl:text>
                  <xsl:value-of
                    select="current-group()[1]/db:seg"/>
                  <xsl:text>' (</xsl:text>
                  <xsl:value-of
                    select="count(current-group())"/>
                  <xsl:text>×) being included</xsl:text>
                  <xsl:if
                    test="current-group()[1]/db:seg='inputenc'">
                    <xsl:text>:</xsl:text>
                    <xsl:for-each
                      select="current-group()[@userlevel=$maxlevel][1]">
                      <xsl:text> </xsl:text>
                      <xsl:value-of select="position()"/>
                      <xsl:text>) </xsl:text>
                      <xsl:value-of select="name()"/>
                      <xsl:text>/</xsl:text>
                      <xsl:value-of select="db:seg/@role"/>
                      <xsl:text>:</xsl:text>
                      <xsl:value-of select="$maxlevel"/>
                    </xsl:for-each>
                  </xsl:if>
                </xsl:message>
                <!-- ADD THE PACKAGE -->
                <xsl:call-template name="packages">
                  <!-- for multiply-recurrent packages, ***FIX THIS ***
                       use the one with the highest userlevel -->
                  <xsl:with-param name="pkg"
                    select="current-group()[@userlevel=$maxlevel][1]"/>
                  <xsl:with-param name="dest" select="'doc'"/>
                  <!-- no cmdblocks on doc packages yet -->
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:for-each-group>
        </xsl:if>
        <!-- INCLUDE THE PACKAGE BEING CREATED? -->
        <xsl:if test="$doctype='package'
                      and
                      /db:book/@xlink:role">
          <xsl:message>
            <xsl:text>Adding </xsl:text>
            <xsl:value-of select="$name"/>
            <xsl:text> as specified</xsl:text>
          </xsl:message>
          <xsl:text>\usepackage</xsl:text>
          <xsl:if test="/db:book/@xlink:role!=''">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="/db:book/@xlink:role"/>
            <xsl:text>]</xsl:text>
          </xsl:if>
          <xsl:text>{</xsl:text>
          <xsl:value-of select="$name"/>
          <xsl:text>}[</xsl:text>
          <!-- use latest rev date as the distro date -->
          <xsl:value-of select="translate($date,'-','/')"/>
          <xsl:text>]% added by spec&#xa;</xsl:text>
        </xsl:if>
        <!-- End of autopackage -->
        <xsl:apply-templates
          select="/db:book/db:info/db:cover
                  /db:constraintdef[@xml:id='docpackages']
                  /db:cmdsynopsis[.!='']"/>
        <xsl:if test="//db:bibliography[@arch='biblatex']">
          <xsl:text>\addbibresource{</xsl:text>
          <xsl:choose>
            <xsl:when
              test="//db:bibliography[@arch='biblatex']
                                     [@xlink:href]">
              <xsl:value-of
                select="//db:bibliography[@arch='biblatex']
                        /@xlink:href"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="$docname"/>
              <xsl:text>.bib</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>}&#xa;</xsl:text>
        </xsl:if>
        <xsl:text>\hypersetup{pdfauthor={</xsl:text>
        <xsl:value-of
          select="normalize-space(/db:book/db:info
                  /db:author/db:personname/db:firstname)"/>
        <xsl:text> </xsl:text>
        <xsl:value-of
          select="normalize-space(/db:book/db:info
                  /db:author/db:personname/db:surname)"/>
        <xsl:text>},
        pdftitle={The </xsl:text>
        <xsl:value-of select="$docname"/>
        <xsl:text> LaTeX2e document </xsl:text>
        <xsl:value-of select="$doctype"/>
        <xsl:text>},
        pdfsubject={</xsl:text>
        <xsl:value-of
          select="normalize-space(/db:book/db:info
                  /db:title)"/>
        <xsl:text>},
        pdfkeywords={</xsl:text>
        <xsl:for-each
          select="/db:book/db:info
                  /db:keywordset/db:keyword">
          <xsl:if test="preceding-sibling::db:keyword">
            <xsl:text>,</xsl:text>
          </xsl:if>
          <xsl:value-of select="."/>
        </xsl:for-each>
        <xsl:text>},
        pdfproducer={XeLateX with hyperref},
        pdfcreator={Emacs/psgml, ClassPack/Saxon, LaTeX/TeX Live}}
</xsl:text>
        <xsl:for-each
          select="/db:book/db:info/db:cover/db:constraintdef
                    [@xml:id='atendpreamble']
                  /db:procedure/db:step
                   [normalize-space(.)!='']
                  /db:cmdsynopsis/db:command">
          <xsl:value-of select="normalize-space(.)"/>
          <xsl:text>&#xa;</xsl:text>
        </xsl:for-each>
        <xsl:text>%&#xa;</xsl:text>
        <xsl:text>%%
%% Settings for docstrip and ltxdoc
%%
\EnableCrossrefs
\CodelineIndex
\RecordChanges&#xa;</xsl:text>
        <xsl:text>\begin{document}</xsl:text>
        <xsl:value-of
          select="normalize-space(db:book/@annotations)"/>
        <xsl:text>&#xa;  \DocInput{</xsl:text>
        <xsl:value-of select="$docname"/>
        <xsl:text>.dtx}&#xa;\end{document}&#xa;</xsl:text>
        <xsl:text>%&lt;/driver>
        <xsl:value-of select="/db:book/@security"/>
        <xsl:text>}
        <xsl:for-each
          select="/db:book/db:info/db:revhistory
                  /db:revision">
          <xsl:text>% \changes{v</xsl:text>
          <xsl:value-of select="@version"/>
          <xsl:text>}{</xsl:text>
          <xsl:value-of
            select="translate(db:date/@YYYY-MM-DD,
                              '-','/')"/>
          <xsl:text>}{</xsl:text>
          <xsl:if test="db:revdescription/
                        (db:itemizedlist|db:orderedlist)
                        /db:title">
            <xsl:value-of
              select="normalize-space(
                      db:revdescription/
                      (db:itemizedlist|db:orderedlist)
                      /db:title)"/>
            <xsl:text>: </xsl:text>
          </xsl:if>
          <xsl:choose>
            <xsl:when test="db:revdescription/db:para">
              <xsl:for-each select="db:revdescription/db:para">
                <xsl:value-of
                  select="normalize-space(.)"/>
                <xsl:if test="position()>1">
                  <xsl:text> ¶ </xsl:text>
                </xsl:if>
              </xsl:for-each>
            </xsl:when>
            <xsl:when
              test="count(db:revdescription
                    /db:itemizedlist/db:listitem)=1">
              <xsl:value-of
                select="normalize-space(db:revdescription
                        /db:itemizedlist/db:listitem)"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:for-each
                select="db:revdescription/db:itemizedlist
                        /db:listitem">
                <xsl:number/>
                <xsl:text>) </xsl:text>
                <xsl:value-of
                  select="normalize-space(db:para)"/>
                <xsl:if
                  test="db:itemizedlist|db:orderedlist">
                  <xsl:text>: </xsl:text>
                  <xsl:for-each
                    select="(db:itemizedlist|db:orderedlist)
                            /db:listitem">
                    <xsl:number format="a"/>
                    <xsl:text>] </xsl:text>
                    <xsl:value-of
                      select="normalize-space(db:para)"/>
                    <xsl:if test="position()!=last()">
                      <xsl:text>, </xsl:text>
                    </xsl:if>
                  </xsl:for-each>
                </xsl:if>
                <xsl:if test="position()!=last()">
                  <xsl:text>; </xsl:text>
                </xsl:if>
              </xsl:for-each>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>.}&#xa;</xsl:text>
        </xsl:for-each>
        <xsl:text>%
        <xsl:value-of select="$docname"/>
        <xsl:text>.dtx}&#xa;</xsl:text>
        <xsl:text>%
</xsl:text>
        <!-- add entry for examples unless with spaces -->
        <xsl:for-each-group
          select="//db:command
                  [not(@role)]
                  [not(contains(.,' ') or contains(.,'{'))]"
          group-by="normalize-space(.)">
          <xsl:text>% \DoNotIndex{\</xsl:text>
          <xsl:value-of select="current-grouping-key()"/>
          <xsl:text>}&#xa;</xsl:text>
        </xsl:for-each-group>
        <!-- Documentation start-up commands to execute
             *after* \begin{document} (ie during the
             absorption phase of the .dtx file running
             from \DocInput) -->
        <xsl:for-each
          select="/db:book/db:info/db:cover
                  /db:constraintdef
                    [@xml:id='atbegindoc']
                  /db:procedure/db:step
                   [normalize-space(.)!='']
                  /db:cmdsynopsis/db:command">
          <xsl:if test="contains(.,'@')">
            <xsl:text>% \makeatletter&#xa;</xsl:text>
          </xsl:if>
          <xsl:text>% </xsl:text>
          <xsl:value-of select="normalize-space(.)"/>
          <xsl:text>&#xa;</xsl:text>
          <xsl:if test="contains(.,'@')">
            <xsl:text>% \makeatother&#xa;</xsl:text>
          </xsl:if>
        </xsl:for-each>
        <xsl:text>%&#xa;</xsl:text>
        <!-- now the fun starts -->
        <xsl:apply-templates
          select="db:book/db:info |
                  db:book/db:part[@xml:id='doc'] |
                  db:book/db:part[@xml:id='code'] |
                  db:book/db:part[@xml:id='files']"/>
        <xsl:text>&#xa;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!-- end of root templae -->
  <xsl:template match="db:info/db:abstract[@role='CTAN']"/>

  <xsl:template match="db:legalnotice"/>

  <xsl:template match="db:info/db:abstract[not(@role='CTAN')]">
    <xsl:if test="db:title">
      <xsl:text>% \renewcommand{\abstractname}{</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:text>}\thispagestyle{empty}&#xa;</xsl:text>
    </xsl:if>
    <xsl:text>% \begin{abstract}&#xa;</xsl:text>
    <xsl:if
      test="not(//db:constraintdef[@xml:id='docpackages']//
            db:seglistitem/db:seg[.='parskip']
            [@condition='off'])">
      <xsl:text>% \parskip=0.5\baselineskip
    </xsl:if>
    <xsl:text>% \noindent&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:if test="@xlink:href and not(@xlink:show='none')">
      <xsl:text>% \par\centering
      <xsl:value-of select="@xlink:href"/>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
    <xsl:text>% \end{abstract}
    <xsl:if test="//db:exceptionname">
      <xsl:apply-templates
        select="document('rfc2119.xml')/db:section"/>
    </xsl:if>
    <xsl:apply-templates
      select="document('representation.xml')/db:section"/>
  </xsl:template>
  <xsl:template match="db:cmdsynopsis">
    <xsl:if test="position()=1
                  and
                  ../db:cmdsynopsis[contains(.,'@')]">
      <xsl:text>\makeatletter&#xa;</xsl:text>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="db:command/@role='asis'">
        <xsl:value-of select="db:command"/>
        <xsl:text>&#xa;</xsl:text>
      </xsl:when>
      <!-- new \write files: new: no arg -->
      <xsl:when test="db:command/@remap='write'">
        <xsl:text>\newwrite\</xsl:text>
        <xsl:value-of select="db:command"/>
        <xsl:text>&#xa;</xsl:text>
      </xsl:when>
      <!-- new counters -->
      <xsl:when test="db:command/@remap='counter'">
        <xsl:text>\newcounter{</xsl:text>
        <xsl:value-of select="db:command"/>
        <xsl:text>}</xsl:text>
        <xsl:if test="db:arg">
          <xsl:text>\setcounter{</xsl:text>
          <xsl:value-of select="db:command"/>
          <xsl:text>}{</xsl:text>
          <xsl:value-of select="db:arg"/>
          <xsl:text>}</xsl:text>
        </xsl:if>
        <xsl:text>&#xa;</xsl:text>
      </xsl:when>
      <!-- new lengths -->
      <xsl:when test="db:command/@remap='length'">
        <xsl:text>\newlength{\</xsl:text>
        <xsl:value-of select="db:command"/>
        <xsl:text>}</xsl:text>
        <xsl:if test="db:arg">
          <xsl:text>\setlength{\</xsl:text>
          <xsl:value-of select="db:command"/>
          <xsl:text>}{</xsl:text>
          <xsl:value-of select="db:arg"/>
          <xsl:text>}</xsl:text>
        </xsl:if>
        <xsl:text>&#xa;</xsl:text>
      </xsl:when>
      <!-- plain TeX commands -->
      <xsl:when test="@language='TeX'">
        <xsl:if test="@role='long'">
          <xsl:text>\long</xsl:text>
        </xsl:if>
        <xsl:text>\def\</xsl:text>
        <xsl:value-of select="db:command"/>
        <xsl:if test="db:arg/@wordsize">
          <xsl:call-template name="repeatarg">
            <xsl:with-param name="limit"
              select="db:arg/@wordsize"/>
            <xsl:with-param name="prefix"
              select="substring(db:arg/@annotations,1,
                      (string-length(db:arg/@annotations)
                      div 2))"/>
            <xsl:with-param name="suffix"
              select="substring(db:arg/@annotations,
                      (string-length(db:arg/@annotations)
                      div 2)+1)"/>
          </xsl:call-template>
        </xsl:if>
        <xsl:choose>
          <xsl:when test="db:arg/@remap">
            <xsl:text>{</xsl:text>
            <xsl:value-of
              select="/db:book/@*[name()=current()
                                  /db:arg/@remap]"/>
            <xsl:text>}&#xa;</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>{%&#xa;&#9;</xsl:text>
            <xsl:value-of select="db:arg"/>
            <xsl:text>}&#xa;</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="db:command/@remap='environment'
                      and count(db:command/db:arg)=2">
        <xsl:choose>
          <xsl:when test="db:command/@role='renew'">
            <xsl:text>\renewenvironment{</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>\newenvironment{</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:value-of select="db:command"/>
        <xsl:text>}</xsl:text>
        <!-- number of arguments, if any -->
        <xsl:if test="db:arg[1]/@wordsize">
          <xsl:text>[</xsl:text>
          <xsl:value-of select="db:arg[1]/@wordsize"/>
          <xsl:text>]</xsl:text>
        </xsl:if>
        <xsl:text>{%&#xa;&#9;</xsl:text>
        <xsl:value-of select="db:arg[1]"/>
        <xsl:text>}&#xa;</xsl:text>
        <xsl:text>{%&#xa;&#9;</xsl:text>
        <xsl:value-of select="db:arg[2]"/>
        <xsl:text>}&#xa;</xsl:text>
      </xsl:when>
      <!-- LaTeX commands -->
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="db:command/@role='renew'">
            <xsl:text>\renewcommand{\</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>\newcommand{\</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:value-of select="db:command"/>
        <xsl:text>}</xsl:text>
        <!-- number of arguments, if any -->
        <xsl:if test="db:arg/@wordsize">
          <xsl:text>[</xsl:text>
          <xsl:value-of select="db:arg/@wordsize"/>
          <xsl:text>]</xsl:text>
          <xsl:if test="db:arg/@condition">
          <xsl:text>[</xsl:text>
          <xsl:value-of select="db:arg/@condition"/>
          <xsl:text>]</xsl:text>
          </xsl:if>
        </xsl:if>
        <xsl:choose>
          <!-- remap is a reference to an attribute of the
               root element, eg arch means use @arch -->
          <xsl:when test="db:arg/@remap">
            <xsl:text>{</xsl:text>
            <xsl:value-of
              select="/db:book/@*[name()=
                              current()/db:arg/@remap]"/>
            <xsl:text>}&#xa;</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>{%&#xa;&#9;</xsl:text>
            <xsl:value-of select="db:arg"/>
            <xsl:text>}&#xa;</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="position()=last()
                  and
                  ../db:cmdsynopsis[contains(.,'@')]">
      <xsl:text>\makeatother&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>
   <xsl:template match="db:info">
    <xsl:message>
      <xsl:text>Creating </xsl:text>
      <xsl:value-of select="$doctype"/>
      <xsl:text> '</xsl:text>
      <xsl:value-of select="$docname"/>
      <xsl:text>' (v</xsl:text>
      <xsl:value-of select="$version"/>
      <xsl:text>.</xsl:text>
      <xsl:value-of select="$revision"/>
      <xsl:text>) with checksum </xsl:text>
      <xsl:value-of select="/db:book/@security"/>
      <xsl:text>.</xsl:text>
    </xsl:message>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="db:cover"/>
  <xsl:template match="db:info/db:title">
    <xsl:if test="/db:book/@arch='package' or
                  not(/db:book/@xlink:role)">
      <xsl:text>% \def\fileversion{</xsl:text>
      <xsl:value-of select="/db:book/@version"/>
      <xsl:text>.</xsl:text>
      <xsl:value-of select="/db:book/@revision"/>
      <xsl:text>}&#xa;% \def\filedate{</xsl:text>
      <xsl:value-of
        select="translate(/db:book/db:info
                /db:revhistory/db:revision
                [@version=max(parent::db:revhistory
                              /db:revision/@version)]
                /db:date/@YYYY-MM-DD,'-','/')"/>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
    <xsl:text>% \title{The </xsl:text>
    <xsl:text> \textsf{</xsl:text>
    <xsl:value-of select="$docname"/>
    <xsl:text>} </xsl:text>
    <xsl:choose>
      <xsl:when test="/db:book/@arch='script'">
        <xsl:text></xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\LaTeXe\ </xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:choose>
      <xsl:when test="$doctype='class'">
        <xsl:text>document class</xsl:text>
        <xsl:if
          test="count(//db:part//db:appendix
                [substring-after(@xlink:href,'.')
                 = 'cls'])>1">
          <xsl:text>es</xsl:text>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$doctype"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>\thanks{%
    <xsl:text>\textsf{</xsl:text>
    <xsl:value-of select="$docname"/>
    <xsl:text>}&#xa;% \textit{v.}\ \fileversion </xsl:text>
    <xsl:choose>
      <xsl:when test="/db:book/@status='alpha' or
                      /db:book/@status='beta'">
        <xsl:text>$\</xsl:text>
        <xsl:value-of select="/db:book/@status"/>
        <xsl:text>$</xsl:text>
      </xsl:when>
      <!-- alpha, beta, delta (draft), phi (final),
           or pi (production) -->
      <xsl:when test="/db:book/@status='α' or
                      /db:book/@status='β' or
                      /db:book/@status='δ' or
                      /db:book/@status='φ' or
                      /db:book/@status='π'">
        <xsl:value-of select="/db:book/@status"/>
      </xsl:when>
      <xsl:when test="/db:book/@status">
        <xsl:value-of select="/db:book/@status"/>
      </xsl:when>
    </xsl:choose>
    <xsl:text>, dated \filedate.}</xsl:text>
    <!-- add second note for sponsors -->
    <xsl:if test="ancestor::db:info/descendant::db:author/
                  db:contrib[@role='sponsor']">
      <xsl:text>\enspace\thanks{%
      <xsl:for-each
        select="ancestor::db:info/descendant::db:author/
                db:contrib[@role='sponsor']">
        <xsl:if test="position()>1">
          <xsl:text>, </xsl:text>
          <xsl:if test="position()=last()">
            <xsl:text>and </xsl:text>
          </xsl:if>
        </xsl:if>
        <xsl:apply-templates/>
      </xsl:for-each>
      <xsl:text>.}</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;% \\[1em]\Large &#xa;% </xsl:text>
    <xsl:apply-templates/>
    <xsl:if test="following-sibling::db:subtitle
                  [not(@role='labelonly')]">
      <xsl:text>&#xa;% \\[1ex]\large &#xa;% </xsl:text>
      <xsl:apply-templates
        select="following-sibling::db:subtitle/node()"/>
    </xsl:if>
    <xsl:text>}&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="db:info/db:subtitle"/>
  <xsl:template match="db:info/db:titleabbrev"/>

  <xsl:variable name="shorttitle">
    <xsl:choose>
      <xsl:when test="/db:book/db:info/db:titleabbrev">
        <xsl:value-of select="/db:book/db:info/db:titleabbrev/node()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="/db:book/db:info/db:title/node()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:template match="db:info//db:author">
    <xsl:if test="count(preceding-sibling::db:author)=0">
      <xsl:text>% \author{</xsl:text>
    </xsl:if>
    <xsl:for-each select="db:personname/db:*">
      <xsl:apply-templates/>
      <xsl:if test="position()!=last()">
        <xsl:text> </xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:apply-templates
      select="db:honorific | db:affiliation"/>
    <xsl:if test="db:email">
      <xsl:text>\\\normalsize(\url{</xsl:text>
      <xsl:value-of select="db:email"/>
      <xsl:text>})</xsl:text>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="following-sibling::db:author">
        <xsl:text>&#xa;% \and&#xa;% </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>}&#xa;% \maketitle&#xa;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="db:honorific">
    <xsl:text> \textsc{</xsl:text>
    <xsl:value-of
      select="translate(.,
              'ABCDEFGHIJKLMN0PQRSTUVWXYZ',
              'abcdefghijklmn0pqrstuvwxyz')"/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:affiliation">
    <xsl:text>\\\normalsize </xsl:text>
    <xsl:for-each select="db:*">
      <xsl:value-of select="normalize-space(.)"/>
      <!-- capitalisation for Centre, Hospital, Unit,
           Project, and maybe others -->
      <xsl:if test="@remap">
        <xsl:text> </xsl:text>
        <xsl:value-of
          select="translate(substring(@remap,1,1),
                  'cuhp','CUHP')"/>
        <xsl:value-of select="substring(@remap,2)"/>
      </xsl:if>
      <xsl:if test="position()!=last()">
        <xsl:text>\\[-.25ex]\normalsize </xsl:text>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
  <xsl:template match="db:info/db:releaseinfo |
                       db:info/db:annotation |
                       db:info/db:revhistory |
                       db:info/db:copyright"/>
  <xsl:template match="db:preface">
    <xsl:choose>
      <xsl:when test="db:title">
        <xsl:text>% \clearpage\section*{</xsl:text>
        <xsl:apply-templates select="db:title/node()"/>
        <xsl:text>}\addcontentsline{toc}</xsl:text>
        <xsl:text>{subsection}{</xsl:text>
        <xsl:apply-templates select="db:title/node()"/>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>% \clearpage\null\vfill</xsl:text>
        <xsl:text>\begingroup\centering</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:if test="not(db:title)">
      <xsl:text>% \par\endgroup\vfill&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:part">
    <xsl:choose>
      <xsl:when test="@xml:id='doc'">
        <xsl:text>% \clearpage&#xa;</xsl:text>
        <xsl:text>% \section*{Latest changes}&#xa;</xsl:text>
        <xsl:apply-templates
          select="//db:revhistory/db:revision
                  [@version=$latestrevhist]"/>
        <xsl:for-each
          select="//db:revhistory/db:revision
                  [@version!=$latestrevhist]">
          <xsl:sort select="db:date/@YYYY-MM-DD"
            order="descending"/>
          <xsl:if test="position()&lt;4">
            <xsl:apply-templates select="."/>
          </xsl:if>
        </xsl:for-each>
        <xsl:text>% \par\bigskip
        <xsl:value-of
          select="/db:book/db:info/db:revhistory/@xml:id"/>
        <xsl:text>} for earlier changes.&#xa;</xsl:text>
        <xsl:apply-templates/>
        <xsl:if
          test="not(db:bibliography) and
                descendant::db:exceptionname and
                (not(@conformance)
                 or
                 (@conformance!='xelatex' and
                  @conformance!='lualatex'))">
          <xsl:text>% \bibliography{</xsl:text>
          <xsl:value-of select="$docname"/>
          <xsl:text>}\bibliographystyle{apalike}%&#xa;</xsl:text>
        </xsl:if>
      </xsl:when>
      <xsl:when test="@xml:id='code'">
      <!-- the invocation of \newgeometry has been withdrawn -->
      <xsl:text>% \StopEventually{\label{endcode}%
        <xsl:value-of
          select="/db:book/db:info/db:revhistory/@xml:id"/>
        <xsl:text>}%
        <!-- Calculate margin width from longest string -->
        <!-- WITHDRAWN 2024-02-04
        <xsl:for-each-group group-by="normalize-space(.)"
          select="descendant::db:command
                  [@role and not(@xml:lang or
                   contains(.,'[') or contains(.,'{'))]
                  |
                  descendant::db:envar
                  [@role and not(@xml:lang or
                   contains(.,'[') or contains(.,'{'))]
                  |
                  descendant::db:tag[@role]
                  |
                  descendant::db:varname[@role]
                  |
                  descendant::db:classname[@role]
                  |
                  descendant::db:package[@role]
                  |
                  descendant::db:annotation/@xreflabel">
          <xsl:sort
            select="string-length(current-grouping-key())"
            data-type="number" order="descending"/>
          <xsl:if test="position()=1">
            <xsl:text>% \divide\CPKrevmarg by2 &#xa;</xsl:text>
            <xsl:text>% \addtolength{\CPKrevmarg}{%&#xa;</xsl:text>
            <xsl:text>%   \widthof{\LabelFont{</xsl:text>
            <xsl:value-of select="current-grouping-key()"/>
            <xsl:text>}}}
            <xsl:text>% \message{Margin reset to </xsl:text>
            <xsl:text>\the\CPKrevmarg, to fit &lt;</xsl:text>
            <xsl:value-of select="current-grouping-key()"/>
            <xsl:text>>}&#xa;</xsl:text>
            <xsl:text>% \makeatletter\ltx@ifpackageloaded{fancyhdr}
          </xsl:if>
        </xsl:for-each-group>
        end of withdrawn margin recalc code -->
        <xsl:if test="count(db:chapter)=0">
          <xsl:text>% \label{</xsl:text>
          <xsl:value-of select="@xml:id"/>
          <xsl:text>}&#xa;</xsl:text>
          <xsl:call-template name="start-tag">
            <xsl:with-param name="identity">
              <xsl:value-of select="$doctype"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:if>
        <xsl:text>% \clearpage\subsection*{Code for the </xsl:text>
        <xsl:apply-templates select="db:title/node()"/>
        <xsl:text>}&#xa;</xsl:text>
        <xsl:apply-templates/>
        <xsl:if test="count(db:chapter) +
                      count(db:appendix) = 0">
          <xsl:call-template name="end-tag">
            <xsl:with-param name="identity">
              <xsl:value-of select="$doctype"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:if>
        <xsl:if test="not(//db:appendix)">
          <xsl:text>% \appendix&#xa;</xsl:text>
        </xsl:if>
        <!-- WITHDRAWN 2024-02-04
        <xsl:text>% \newgeometry{left=3cm}
        -->
        <xsl:apply-templates select="$licence"/>
        <xsl:if test="@role">
          <xsl:for-each
            select="tokenize(normalize-space(@role),
                    ' ')">
            <xsl:apply-templates
              select="document(concat($appdir,'/',.,'.xml'))
                      /db:chapter"/>
          </xsl:for-each>
        </xsl:if>
      </xsl:when>
      <xsl:when test="@xml:id='files'">
        <xsl:apply-templates mode="files"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:message>
          <xsl:text>WARNING: part element being used </xsl:text>
          <xsl:text>that is not doc/code/files</xsl:text>
        </xsl:message>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="count(following-sibling::db:part
                  [@xml:id!='data'])=0">
      <xsl:text>% \Finale&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="db:part[@xml:id='code']/db:subtitle"/>
  <xsl:template match="db:chapter[not(@condition='draft')]">
    <!-- non-empty only -->
    <xsl:if test="normalize-space(.)!=''">
      <xsl:choose>
        <!-- first one in code is THE package or class
             so use that value as the tag -->
        <xsl:when
          test="parent::db:part[@xml:id='code'] and
                count(preceding-sibling::db:chapter)=0">
          <xsl:call-template name="start-tag">
            <xsl:with-param name="identity">
              <xsl:value-of select="$doctype"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:when>
        <!-- in files, use the ID as the tag -->
        <xsl:when
          test="parent::db:part[@xml:id='files']
                and
                db:programlisting
                and
                @xml:id
                and
                @xlink:href">
          <xsl:call-template name="start-tag"/>
        </xsl:when>
      </xsl:choose>
      <xsl:call-template name="checkpackages">
        <xsl:with-param name="pos" select="'before'"/>
        <xsl:with-param name="loc" select="."/>
      </xsl:call-template>
      <xsl:if test="count(preceding-sibling::db:chapter)>0">
        <xsl:text>% \clearpage&#xa;</xsl:text>
      </xsl:if>
      <xsl:text>% \section</xsl:text>
      <!-- subtitle is used for short title -->
      <xsl:if test="db:subtitle">
        <xsl:text>[</xsl:text>
        <xsl:apply-templates select="db:subtitle/node()"/>
        <xsl:text>]</xsl:text>
      </xsl:if>
      <xsl:text>{</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:if test="db:subtitle">
        <xsl:text>~--- </xsl:text>
        <xsl:apply-templates select="db:subtitle/node()"/>
      </xsl:if>
      <xsl:if test="@version">
        <xsl:text> (v\thinspace{}</xsl:text>
        <xsl:value-of select="@version"/>
        <xsl:text>)</xsl:text>
      </xsl:if>
      <xsl:text>}</xsl:text>
      <xsl:if test="@xml:id">
        <xsl:text>\label{</xsl:text>
        <xsl:value-of select="@xml:id"/>
        <xsl:text>}</xsl:text>
      </xsl:if>
      <xsl:text>&#xa;</xsl:text>
      <!-- echo the title/subtitle into comments
           IFF this is NOT a DTD, XML, or XSL file -->
      <xsl:if
        test="parent::db:part[@xml:id='code']
              and @xlink:href
              and @xml:id
              and not(substring-after(@xlink:href,'.')='dtd'
                or substring-after(@xlink:href,'.')='xml'
                or substring-after(@xlink:href,'.')='xsl')">
        <xsl:text>% \iffalse&#xa;</xsl:text>
        <xsl:value-of select="$sentinel"/>
        <xsl:text>&#xa;</xsl:text>
        <xsl:apply-templates select="db:title|db:subtitle"
          mode="readme"/>
        <xsl:text>% \fi&#xa;</xsl:text>
      </xsl:if>
      <!-- start the chapter -->
      <xsl:call-template name="makechapapp"/>
    </xsl:if>
    <xsl:choose>
      <xsl:when
        test="parent::db:part[@xml:id='code'] and
              count(following-sibling::db:chapter)=0">
        <xsl:call-template name="end-tag">
          <xsl:with-param name="identity">
            <xsl:value-of select="$doctype"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when
        test="parent::db:part[@xml:id='files']
              and
              db:programlisting
              and
              @xml:id
              and
              @xlink:href">
        <xsl:call-template name="end-tag"/>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  <!-- ANNOTATED FILES ARE APPENDIXES
       in the code part
       (in the file part, unannotated files are chapters)
       but you might have documentary appendices in the
       documentation -->
  <xsl:template match="db:appendix">
    <!-- ignore empty (placeholder) appendixes -->
    <xsl:if test="normalize-space(.)!=''">
      <!-- LaTeX switch to appendix handling in docs
           for the first one -->
      <xsl:if
        test="count(preceding-sibling::db:appendix)=0 and
              ancestor::db:part/@xml:id='doc'">
        <xsl:text>% \appendix\labelformat{section}</xsl:text>
        <xsl:text>{Appendix~#1}&#xa;</xsl:text>
      </xsl:if>
      <!-- insert latexdoc start-tag for annotated code -->
      <xsl:if test="parent::db:part[@xml:id='code']
                    and
                    descendant::db:programlisting
                    and
                    @xlink:href
                    and
                    @xml:id">
        <xsl:call-template name="start-tag"/>
      </xsl:if>
      <!-- TITLE and SUBTITLE -->
      <xsl:text>% \clearpage&#xa;% \section</xsl:text>
      <xsl:if test="db:subtitle">
        <xsl:text>[</xsl:text>
        <xsl:apply-templates select="db:title/node()"/>
        <xsl:text>]</xsl:text>
      </xsl:if>
      <xsl:text>{</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:if test="db:subtitle">
        <xsl:text>~--- </xsl:text>
        <xsl:apply-templates select="db:subtitle/node()"/>
      </xsl:if>
      <xsl:text>}</xsl:text>
      <!-- LABEL -->
      <xsl:if test="@xml:id">
        <xsl:text>\label{</xsl:text>
        <xsl:value-of select="@xml:id"/>
        <xsl:text>}</xsl:text>
      </xsl:if>
      <xsl:text>&#xa;</xsl:text>
      <!-- start the appendix -->
      <xsl:call-template name="makechapapp"/>
      <xsl:if
        test="parent::db:part[@xml:id='code']
              and descendant::db:programlisting
              and @xlink:href and @xml:id">
        <xsl:call-template name="end-tag"/>
      </xsl:if>
    </xsl:if>
  </xsl:template>
  <xsl:template name="startappcomment">
    <xsl:param name="scriptflag"/>
      <!-- non-script = 'no'
           script, first line not done yet = 'wait'
           script, first line done = "go" -->
    <!-- prefix extracted files with a comment to show
         provenance, date, and time (context is <appendix>);
         THE EXCEPTION is shell scripts where the comment is
         delayed until after the first line for hashbang
         reasons -->
    <xsl:if test="$scriptflag='no' or $scriptflag='go'">
      <xsl:text>% \begin{CPKcomment}{</xsl:text>
      <xsl:value-of select="ancestor-or-self::db:appendix/@xml:id"/>
      <xsl:text>-comment}</xsl:text>
      <xsl:text>\label{comment-</xsl:text>
      <xsl:value-of select="ancestor-or-self::db:appendix/@xml:id"/>
      <xsl:text>}&#xa;</xsl:text>
      <!-- IDENTITY COMMENT -->
      <xsl:text>% Identify the file origin,
      <xsl:text>%    \begin</xsl:text>
      <xsl:text>{macrocode}&#xa;</xsl:text>
      <xsl:value-of select="sil:sentinel(.)"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="normalize-space(
        ancestor-or-self::db:appendix/db:title)"/>
      <xsl:if test="ancestor-or-self::db:appendix/db:subtitle">
        <xsl:text>: </xsl:text>
        <xsl:value-of select="normalize-space(
          ancestor-or-self::db:appendix/db:subtitle)"/>
      </xsl:if>
      <xsl:text> (from the </xsl:text>
      <xsl:value-of select="$docname"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="$doctype"/>
      <xsl:text> v.</xsl:text>
      <xsl:value-of select="$version"/>
      <xsl:text>.</xsl:text>
      <xsl:value-of select="$revision"/>
      <xsl:text>, </xsl:text>
      <xsl:value-of select="$date"/>
      <xsl:text>) </xsl:text>
      <xsl:value-of select="sil:termcommentchar(.)"/>
      <xsl:text>&#xa;</xsl:text>
      <xsl:value-of select="sil:sentinel(.)"/>
      <xsl:text> Generated by ClassPack v.</xsl:text>
      <xsl:value-of select="$thisversion"/>
      <xsl:text> (</xsl:text>
      <xsl:value-of select="$thisverdate"/>
      <xsl:text>) on </xsl:text>
      <!--
      <xsl:value-of
        select="format-dateTime(current-dateTime(),
                '[D] [MNn] [Y] at [H]:[m]:[s] ')"/>
      -->
      <xsl:value-of
        select="format-dateTime(current-dateTime(),
                '[Y03]-[M01]-[D01]T[H01]:[m01]:[s01] ')"/>
      <xsl:value-of select="sil:termcommentchar(.)"/>
      <xsl:text>&#xa;</xsl:text>
      <xsl:text>%    \end</xsl:text>
      <xsl:text>{macrocode}&#xa;</xsl:text>
      <xsl:text>% These lines are automatically prepended to the file
      <xsl:text>% \end{CPKcomment}&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:sect1">
    <xsl:if test="normalize-space(.)!=''">
      <xsl:call-template name="checkpackages">
        <xsl:with-param name="pos" select="'before'"/>
        <xsl:with-param name="loc" select="."/>
      </xsl:call-template>
      <xsl:if test="@condition='newpage'">
        <xsl:text>% \clearpage&#xa;</xsl:text>
      </xsl:if>
      <xsl:text>% \subsection</xsl:text>
      <!-- is @label for a * or an [alt]? -->
      <xsl:value-of select="@label"/>
      <xsl:text>{</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:text>}</xsl:text>
      <xsl:if test="@xml:id">
        <xsl:text>\label{</xsl:text>
        <xsl:value-of select="@xml:id"/>
        <xsl:text>}</xsl:text>
      </xsl:if>
      <xsl:text>&#xa;</xsl:text>
      <xsl:if test="ancestor::db:part[@xml:id='code']">
        <xsl:choose>
          <xsl:when test="ancestor-or-self::db:chapter">
            <xsl:text>% \iffalse&#xa;%% &#xa;</xsl:text>
            <xsl:call-template name="makefence"/>
            <xsl:text>%% &#xa;</xsl:text>
            <xsl:apply-templates mode="readme"
              select="db:title|db:subtitle"/>
            <xsl:if test="@xml:id='options'
                          and not(db:annotation)
                          and count(db:para)=1">
              <xsl:text>%%
%% There are no package options.&#xa;</xsl:text>
            </xsl:if>
            <xsl:text>% \fi&#xa;</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <!--
            <xsl:value-of select="sil:commentchar(.)"/>
            <xsl:apply-templates mode="readme"
              select="db:title|db:subtitle"/>
            <xsl:value-of select="sil:termcommentchar(.)"/>
            -->
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
      <!-- process sect1 content -->
      <xsl:apply-templates/>
      <xsl:if test="@xml:id='options'">
        <xsl:text>% Now invoke the options.\par
%% Now invoke the options.
\ExecuteOptions{</xsl:text>
        <xsl:value-of
          select="descendant::db:annotation
                  [@condition='default']/@xreflabel"/>
        <xsl:text>}
        \ProcessOptions\relax&#xa;%    \end</xsl:text>
        <xsl:text>{macrocode}&#xa;</xsl:text>
      </xsl:if>
      <xsl:call-template name="checkpackages">
        <xsl:with-param name="pos" select="'after'"/>
        <xsl:with-param name="loc" select="."/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:sect2">
    <xsl:if test="normalize-space(.)!=''">
      <xsl:call-template name="checkpackages">
        <xsl:with-param name="pos" select="'before'"/>
        <xsl:with-param name="loc" select="."/>
      </xsl:call-template>
      <xsl:text>% \subsubsection{</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:text>}</xsl:text>
      <xsl:if test="@xml:id">
        <xsl:text>\label{</xsl:text>
        <xsl:value-of select="@xml:id"/>
        <xsl:text>}</xsl:text>
      </xsl:if>
      <xsl:text>&#xa;</xsl:text>
      <xsl:if
        test="ancestor::db:part[@xml:id='code'] and
              ancestor::db:chapter/@xlink:href  and
              ancestor::db:chapter/@xml:id and
              not(substring-after(
               ancestor::db:chapter/@xlink:href,'.')='dtd'
            or substring-after(
               ancestor::db:chapter/@xlink:href,'.')='xml'
            or substring-after(
               ancestor::db:chapter/@xlink:href,'.')='xsl')">
        <xsl:text>% \iffalse&#xa;%% &#xa;</xsl:text>
        <xsl:apply-templates mode="readme"
          select="db:title|db:subtitle"/>
        <xsl:text>% \fi&#xa;</xsl:text>
      </xsl:if>
      <xsl:apply-templates/>
      <xsl:call-template name="checkpackages">
        <xsl:with-param name="pos" select="'after'"/>
        <xsl:with-param name="loc" select="."/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:sect3">
    <xsl:if test="normalize-space(.)!=''">
      <xsl:call-template name="checkpackages">
        <xsl:with-param name="pos" select="'before'"/>
        <xsl:with-param name="loc" select="."/>
      </xsl:call-template>
      <xsl:text>% \paragraph[</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:text>]{</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:if test="not(matches(normalize-space(db:title),
                    '[:\-–—•]$'))">
        <xsl:text> :</xsl:text>
      </xsl:if>
      <xsl:text>}</xsl:text>
      <xsl:if test="@xml:id">
        <xsl:text>\label{</xsl:text>
        <xsl:value-of select="@xml:id"/>
        <xsl:text>}</xsl:text>
      </xsl:if>
      <xsl:text>&#xa;</xsl:text>
      <xsl:apply-templates/>
      <xsl:call-template name="checkpackages">
        <xsl:with-param name="pos" select="'after'"/>
        <xsl:with-param name="loc" select="."/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:sect4">
    <xsl:if test="normalize-space(.)!=''">
      <xsl:call-template name="checkpackages">
        <xsl:with-param name="pos" select="'before'"/>
        <xsl:with-param name="loc" select="."/>
      </xsl:call-template>
      <xsl:text>% \subparagraph[</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:text>]{</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:if test="not(matches(normalize-space(db:title),
                    '[:\-–—•]$'))">
        <xsl:text>  ◦</xsl:text>
      </xsl:if>
      <xsl:text>}</xsl:text>
      <xsl:if test="@xml:id">
        <xsl:text>\label{</xsl:text>
        <xsl:value-of select="@xml:id"/>
        <xsl:text>}</xsl:text>
      </xsl:if>
      <xsl:text>&#xa;</xsl:text>
      <xsl:apply-templates/>
      <xsl:call-template name="checkpackages">
        <xsl:with-param name="pos" select="'after'"/>
        <xsl:with-param name="loc" select="."/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <xsl:template
    match="db:section/db:title
           [name(parent::db:section/parent::*[1])='']"/>
  <xsl:template match="db:section">
    <xsl:if test="@role='newpage'">
      <xsl:text>% \clearpage&#xa;</xsl:text>
    </xsl:if>
    <xsl:text>% \subsection*{</xsl:text>
    <xsl:apply-templates select="db:title/node()"/>
    <xsl:text>}</xsl:text>
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;</xsl:text>
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template
    match="db:info/db:abstract/db:title |
           db:preface/db:title |
           db:preface/db:section/db:title |
           db:part/db:title |
           db:chapter/db:title | db:chapter/db:subtitle |
           db:chapter/db:sect1/db:title |
           db:chapter//db:sect2/db:title |
           db:chapter//db:sect3/db:title |
           db:chapter//db:sect4/db:title |
           db:appendix/db:title |
           db:appendix/db:subtitle |
           db:appendix/db:sect1/db:title |
           db:appendix//db:sect2/db:title |
           db:appendix//db:sect3/db:title |
           db:appendix//db:sect4/db:title"/>
  <xsl:template match="db:acknowledgements">
    <xsl:text>% \subsection*{Acknowledgments}</xsl:text>
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:call-template name="checkpackages">
      <xsl:with-param name="pos" select="'after'"/>
      <xsl:with-param name="loc" select="."/>
    </xsl:call-template>
  </xsl:template>
  <xsl:template match="db:para">
    <xsl:choose>
      <!-- paras from prepost.xml need shielding -->
      <xsl:when
        test="ancestor::db:procedure
              [@xml:id='prepackage'
               or
               @xml:id='postpackage']">
        <xsl:text>% </xsl:text>
      </xsl:when>
      <!-- first paras in list items do nothing WHY? -->
      <xsl:when
        test="(parent::db:listitem or parent::db:step) and
               count(preceding-sibling::db:para)=0">
        <xsl:text></xsl:text>
      </xsl:when>
      <!-- first paras in table cells likewise -->
      <xsl:when test="parent::db:entry and position()=1">
        <xsl:text></xsl:text>
      </xsl:when>
      <!-- all others get a shield -->
      <xsl:otherwise>
        <xsl:text>% </xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <!-- labels on paras (rare) get output at start -->
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <!-- any typographic mods start here, end afterwards -->
    <xsl:if test="@remap">
      <xsl:text>{</xsl:text>
      <xsl:value-of select="@remap"/>
    </xsl:if>
    <!-- now the content -->
    <xsl:apply-templates/>
    <!-- then the fun starts -->
    <xsl:choose>
      <!-- AUTOPUNCT: para elements in lists (NOT inline) which
             * do NOT end with their own punctuation
               AND
             * the container does NOT start with punctuation
               OR
             * there is nothing in scope after the container
           -->
      <xsl:when
        test="(parent::db:listitem[not(ancestor::db:para)]
               or
              parent::db:step[not(ancestor::db:para)])
              and
              not(contains(',.;:!?',
                  substring(normalize-space(.),
                  string-length(normalize-space(.)))))
              and
              (
                not(contains(',.;:!?',
                  substring(normalize-space(
                    (
                    ancestor::db:itemizedlist[1] |
                    ancestor::db:orderedlist[1] |
                    ancestor::db:variablelist[1] |
                    ancestor::db:procedure[1]
                    )[1]/
                    following-sibling::node()[1]),1,1)))
                or
                not(  (
                    ancestor::db:itemizedlist[1] |
                    ancestor::db:orderedlist[1] |
                    ancestor::db:variablelist[1] |
                    ancestor::db:procedure[1]
                    )[1]/
                    following-sibling::node())
              )">
        <!-- if that is TRUE then munge the punct -->
        <xsl:choose>
          <!-- if last in the item, add semicolon or period -->
          <xsl:when
            test="not(following-sibling::db:para)">
            <xsl:choose>
              <xsl:when
                test="parent::db:listitem/
                      following-sibling::db:listitem
                      or
                      ancestor::db:varlistentry[1]/
                      following-sibling::db:varlistentry
                      or
                      parent::db:step/
                      following-sibling::db:step">
                <xsl:text>;</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>.</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <!-- if not the last one, use a \par -->
          <xsl:otherwise>
            <xsl:text>\par </xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <!-- paras in prepost.xml don't get autopunct -->
      <xsl:when
        test="(parent::db:step/parent::db:procedure
              [@xml:id='prepackage']
              or
              parent::db:step/parent::db:procedure
              [@xml:id='postpackage'])
              and
              not(following-sibling::db:para)">
        <xsl:text></xsl:text>
      </xsl:when>
      <!-- everything else gets a \par -->
      <xsl:otherwise>
        <xsl:text>\par</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <!-- close off any formatting changes -->
    <xsl:if test="@remap">
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:if test="not(parent::db:note
                      or
                      parent::db:footnote
                      or
                      parent::db:entry)">
      <xsl:text>&#xa;</xsl:text>
      <xsl:if test="parent::db:entry">
        <xsl:text> </xsl:text>
      </xsl:if>
    </xsl:if>
  <!-- add plaintext documentation that will go through to the
       class or style file being generated. Don't do this here
       for the para of a warning or sidebar because that will
       still be inside a minipage. See the warning|sidebar
       template.
       IFF
       * it's in the code
       * it's not a footnote
       * it's in a chapter being output
       * it's in an ID'd chapter
       * it's not DTD/XML/XSL
       * it's not in a warning or sidebar
       (these are ANDed, so it means all of them!) -->
    <xsl:if
      test="ancestor::db:part[@xml:id='code']
            and not(parent::db:footnote)
            and ancestor::db:chapter[@xlink:href]
            and ancestor::db:chapter[@xml:id]
            and not(
            substring-after(ancestor::db:chapter
             /@xlink:href,'.')='dtd'
         or substring-after(ancestor::db:chapter
             /@xlink:href,'.')='xml'
         or substring-after(ancestor::db:chapter
             /@xlink:href,'.')='xsl')
            and not(parent::db:warning)
            and not(parent::db:sidebar)">
      <xsl:text>% \iffalse&#xa;</xsl:text>
      <xsl:apply-templates select="." mode="readme"/>
      <xsl:text>% \fi&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:literal[not(@language='LaTeXe')
                       and not(@language='LaTeX')]">
    <xsl:call-template name="avoidverb"/>
  </xsl:template>

  <xsl:template match="db:literal[@language='LaTeXe'
                       or @language='LaTeX']">
    <xsl:value-of select="."/>
  </xsl:template>
  <xsl:template match="db:blockquote">
    <xsl:text>% \begin{quotation}</xsl:text>
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>\small\sffamily\parindent0pt
    <xsl:if test="@remap">
      <xsl:value-of select="@remap"/>
    </xsl:if>
    <xsl:text>\noindent&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:if test="@linkend">
      <xsl:text>% \hfill\begingroup</xsl:text>
      <xsl:call-template name="makeref"/>
      <xsl:text>\parfillskip=0pt\parskip=0pt\par</xsl:text>
      <xsl:text>\endgroup&#xa;</xsl:text>
    </xsl:if>
    <xsl:if test="@xlink:href">
      <xsl:text>% \hfill\begingroup
      <xsl:value-of select="@xlink:href"/>
      <xsl:text>}\parfillskip=0pt\par</xsl:text>
      <xsl:text>\endgroup&#xa;</xsl:text>
    </xsl:if>
    <xsl:text>% \end{quotation}&#xa;</xsl:text>
  </xsl:template>
  <xsl:template match="db:warning | db:sidebar">
    <xsl:call-template name="sidewarn"/>
  </xsl:template>

  <!-- named template used to allow "files" chapters
       to share this code from within mode="files" -->

  <xsl:template name="sidewarn">
    <xsl:text>% \par\begingroup
    <xsl:choose>
      <xsl:when test="name()='sidebar'">
        <xsl:text>shadowbox</xsl:text>
      </xsl:when>
      <xsl:when test="name()='warning'">
        <xsl:text>fbox</xsl:text>
      </xsl:when>
    </xsl:choose>
    <xsl:text>{\begin{minipage}{</xsl:text>
    <xsl:choose>
      <xsl:when test="@wordsize">
        <xsl:value-of
          select="number(substring-before(@wordsize,'%'))
                  div 100"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>0.8</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>\columnwidth}\sffamily
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}%</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>% \end{minipage}}\par\endgroup&#xa;</xsl:text>
    <!-- now is the time to output the content in a form
         that will pass through to the sty or cls file. -->
    <!-- not working yet
    <xsl:text>% \iffalse&#xa;%% !!! </xsl:text>
    <xsl:apply-templates select="db:para" mode="readme"/>
    <xsl:text>% \fi&#xa;</xsl:text>
-->
  </xsl:template>

  <xsl:template match="db:warning/db:title
                       |
                       db:sidebar/db:title">
    <xsl:text>% \subsubsection*{\sffamily </xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="db:sidebar/db:address">
    <xsl:text>% \par\raggedleft </xsl:text>
    <xsl:apply-templates/>
    <xsl:text>\par&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="db:bridgehead">
    <xsl:text>% \subsubsection*{</xsl:text>
    <xsl:if test="parent::db:annotation">
      <!--
      <xsl:text>\protect\CPKrunningecho{</xsl:text>
      <xsl:value-of
        select="parent::db:annotation/@xreflabel"/>
      <xsl:text>}</xsl:text>
      -->
      <xsl:text>\llap{\color{OliveDrab}\LabelFont{</xsl:text>
      <xsl:value-of
        select="parent::db:annotation/@xreflabel"/>
      <xsl:text>}\quad}</xsl:text>
    </xsl:if>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}\addcontentsline{toc}</xsl:text>
      <xsl:text>{subparagraph}{</xsl:text>
      <xsl:value-of select="normalize-space(.)"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;</xsl:text>
  </xsl:template>
  <!-- The annotation element used here normally: not for @reuse,
       with a @role set to a known value (not "unused"), and
       with a non-null @xreflabel and NO @annotations and not
       in the <info> header (Phew!) -->
  <xsl:template match="db:annotation
                       [not(@reuse)]
                       [@role and @role!='unused' and
                        @xreflabel and @xreflabel!='']
                       [not(@annotations)]
                       [not(ancestor::db:info)]">
    <xsl:variable name="role" select="@role"/>
    <xsl:if test="$role='color'">
      <xsl:if test="not(@arch) or not(@audience)">
        <xsl:message>
          <xsl:text>CPK: no colour code given for </xsl:text>
          <xsl:value-of select="@xreflabel"/>
        </xsl:message>
      </xsl:if>
      <xsl:if test="parent::db:listitem">
        <xsl:text>&#xa;</xsl:text>
      </xsl:if>
      <xsl:text>% \definecolor{</xsl:text>
      <xsl:value-of select="@xreflabel"/>
      <xsl:text>}{</xsl:text>
      <xsl:value-of select="@arch"/>
      <xsl:text>}{</xsl:text>
      <xsl:value-of select="@audience"/>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
    <xsl:if test="parent::db:listitem and not(@role='color')">
      <xsl:text>&#xa;</xsl:text>
    </xsl:if>
    <xsl:text>% \begin{CPK</xsl:text>
    <xsl:value-of select="$role"/>
    <xsl:text>}{</xsl:text>
    <xsl:if test="$role='macro' or
                  $role='switch' or
                  $prepost//db:command
                    [starts-with(.,'\doxitem')]
                    [contains(.,concat('{CPK',$role,'}'))]
                    [contains(.,'macrolike')]">
      <xsl:text>\</xsl:text>
    </xsl:if>
    <xsl:value-of select="@xreflabel"/>
    <xsl:text>}</xsl:text>
    <xsl:if test="not(//db:annotation[@reuse=current()/@xml:id])">
      <xsl:text>\label{</xsl:text>
      <xsl:choose>
        <xsl:when test="@xml:id">
          <xsl:value-of select="@xml:id"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@role"/>
          <xsl:text>-</xsl:text>
          <xsl:value-of
            select="substring-before(db:programlisting[1]
                    /@xlink:href,'.')"/>
          <xsl:text>-</xsl:text>
          <xsl:choose>
            <xsl:when test="contains(@xreflabel,':')">
              <xsl:value-of
                select="substring-after(@xreflabel,':')"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="@xreflabel"/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:if
            test="count(//db:annotation[@xreflabel=current()
                  /@xreflabel])>1">
            <xsl:value-of
              select="count(preceding::db:annotation
                      [@xreflabel=current()/@xreflabel])+1"/>
          </xsl:if>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;</xsl:text>
  <!--
    <xsl:if test="@arch">
      <xsl:text>% \addcontentsline{toc}{</xsl:text>
      <xsl:value-of select="@arch"/>
      <xsl:text>}{\texttt{</xsl:text>
      <xsl:choose>
        <xsl:when test="contains(@xreflabel,':')">
          <xsl:value-of
            select="substring-after(@xreflabel,':')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@xreflabel"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>}</xsl:text>
      <xsl:if test="contains(@condition,'|')">
        <xsl:text> and others</xsl:text>
      </xsl:if>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
-->
    <xsl:if test="@condition">
      <xsl:text>% \begingroup\raggedright[</xsl:text>
      <xsl:for-each
        select="tokenize(replace(normalize-space(@condition),
                         'xxx:',''),'\|')">
        <xsl:variable name="subject">
          <xsl:choose>
            <xsl:when test="contains(.,'[')">
              <xsl:value-of
                select="replace(substring-before(
                        normalize-space(.),'['),'//','/→/')"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="normalize-space(.)"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable name="predicate">
          <xsl:if test="contains(.,'[')">
            <xsl:value-of
              select="concat('[',
                      substring-after(normalize-space(.),'['))"/>
          </xsl:if>
        </xsl:variable>
        <xsl:if test="position()>1">
          <xsl:text>and </xsl:text>
        </xsl:if>
        <xsl:choose>
          <xsl:when
            test="contains($subject,'/') and position()=1">
            <xsl:text>in \url{</xsl:text>
            <xsl:variable name="lastone"
              select="concat(tokenize($subject,'/')
                      [position()=last()],'$')"/>
            <xsl:value-of
              select="replace($subject,$lastone,'')"/>
            <xsl:text>} </xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>\url{</xsl:text>
            <xsl:value-of select="$subject"/>
            <xsl:text>} </xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="$predicate!=''">
          <xsl:text>with \url{</xsl:text>
          <xsl:value-of select="$predicate"/>
          <xsl:text>} </xsl:text>
        </xsl:if>
        <xsl:text>&#xa;% </xsl:text>
      </xsl:for-each>
      <xsl:text>]\par\endgroup&#xa;</xsl:text>
    </xsl:if>
    <xsl:apply-templates/>
    <xsl:text>% \end{CPK</xsl:text>
    <xsl:value-of select="$role"/>
    <xsl:text>}&#xa;</xsl:text>
  </xsl:template>
  <xsl:template match="db:constraintdef[@reuse]">
    <xsl:variable name="thisapp"
      select="ancestor::db:appendix/@xml:id"/>
    <xsl:variable name="thisdoctype"
      select="substring-after(
              ancestor::db:appendix/@xlink:href,'.')"/>
    <xsl:text>% This is a subset of packages </xsl:text>
    <xsl:text>required for the \texttt{.</xsl:text>
    <xsl:value-of select="$filetype"/>
    <xsl:text>} file.&#xa;</xsl:text>
    <xsl:for-each
      select="//db:constraintdef[@xml:id=current()/@reuse]
              //descendant::db:seglistitem
                [not(contains(@omit,$thisapp))]">
      <xsl:apply-templates
        select="$prepost/db:procedure[@xml:id='prepackage']
                /db:step[contains(@condition,$thisdoctype)]
                [@remap=current()/db:seg]/db:para"/>
      <xsl:if test="@role">
        <xsl:text>% </xsl:text>
        <xsl:value-of select="normalize-space(@role)"/>
        <xsl:text>&#xa;</xsl:text>
      </xsl:if>
      <xsl:text>%    </xsl:text>
      <xsl:text>\begin</xsl:text>
      <xsl:text>{macrocode}</xsl:text>
      <xsl:for-each select="db:seg">
        <!-- should only be one of these per seglistitem -->
        <xsl:text>&#xa;\RequirePackage</xsl:text>
        <xsl:if test="@role">
          <xsl:text>[</xsl:text>
          <xsl:value-of select="normalize-space(@role)"/>
          <xsl:text>]</xsl:text>
        </xsl:if>
        <xsl:text>{</xsl:text>
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>}</xsl:text>
        <xsl:if test="@version">
          <xsl:text>[</xsl:text>
          <xsl:value-of select="translate(@version,'-','/')"/>
          <xsl:text>]</xsl:text>
        </xsl:if>
        <xsl:text>&#xa;%    </xsl:text>
        <xsl:text>\end</xsl:text>
        <xsl:text>{macrocode}&#xa;</xsl:text>
      <xsl:apply-templates
        select="$prepost/db:procedure[@xml:id='postpackage']
                /db:step[contains(@condition,$thisdoctype)]
                        [@remap=current()/db:seg]/db:para"/>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:template>
  <xsl:template match="db:annotation[@reuse]">
    <xsl:for-each select="tokenize(normalize-space(@reuse),' ')">
      <xsl:variable name="block" select="."/>
      <xsl:apply-templates
        select="$thisdoc//db:annotation[@xml:id=$block]"/>
    </xsl:for-each>
  </xsl:template>
  <xsl:template match="db:remark">
    <xsl:text>% \changes{v</xsl:text>
    <xsl:value-of select="@version"/>
    <xsl:text>}{</xsl:text>
    <xsl:choose>
      <xsl:when test="@revision">
        <xsl:value-of select="translate(@revision,'-','/')"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of
          select="translate(
                  //db:revhistory/db:revision
                    [@version=current()/@version]
                  /db:date/@YYYY-MM-DD,'-','/')"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>}{</xsl:text>
    <xsl:value-of select="normalize-space(.)"/>
    <xsl:text>}&#xa;</xsl:text>
    <!-- also reproduce the comments in plaintext as comments -->
    <xsl:if
      test="ancestor::db:part[@xml:id='code']
            and ancestor::db:chapter/@xlink:href
            and ancestor::db:chapter/@xml:id
            and not(substring-after(
              ancestor::db:chapter/@xlink:href,'.')='dtd'
                 or substring-after(
              ancestor::db:chapter/@xlink:href,'.')='xml'
                 or substring-after(
              ancestor::db:chapter/@xlink:href,'.')='xsl')">
      <xsl:text>% \iffalse&#xa;</xsl:text>
      <xsl:value-of select="$sentinel"/>
      <xsl:text>CHANGE for v</xsl:text>
      <xsl:value-of select="@version"/>
      <xsl:text> (</xsl:text>
      <xsl:choose>
        <xsl:when test="@revision">
          <xsl:value-of select="translate(@revision,'-','/')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of
            select="translate(
                    //db:revhistory/db:revision
                      [@version=current()/@version]
                    /db:date/@YYYY-MM-DD,'-','/')"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>):</xsl:text>
      <xsl:text>&#xa;</xsl:text>
      <xsl:apply-templates select="." mode="readme"/>
      <xsl:text>% \fi&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:programlisting">
    <!-- Handles code in annotated and unannotated blocks -->
    <xsl:variable name="curapp"
      select="generate-id(ancestor::db:appendix[1])"/>
    <!-- count the prior listing blocks within this appendix -->
    <xsl:variable name="nthproglist"
      select="count(preceding::db:programlisting
              [generate-id(ancestor::db:appendix[1])=$curapp]) + 1"/>
  <!-- programlisting within an annotation
       OR anywhere in the code part -->
    <xsl:choose>
      <xsl:when test="(parent::db:annotation[@xreflabel]
                      or
                      ancestor::db:part[@xml:id='code'])
                      and not(@condition='ignore')">
        <xsl:if test="@xml:id">
          <xsl:text>% \label{</xsl:text>
          <xsl:value-of select="@xml:id"/>
          <xsl:text>}&#xa;</xsl:text>
        </xsl:if>
        <xsl:choose>
          <!-- EXTERNAL FRAGMENT whatever the language -->
          <xsl:when test="@xlink:href">
            <xsl:choose>
              <!-- frag means file has been pre-split into pieces -->
              <xsl:when test="@conformance='frag'">
                <!-- get the original source filename, strip dir  -->
                <xsl:variable name="fn">
                  <xsl:choose>
                    <xsl:when test="contains(@xlink:href,'/')">
                      <xsl:value-of
                        select="tokenize(@xlink:href,'/')
                                [position()=last()]"/>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:value-of select="@xlink:href"/>
                    </xsl:otherwise>
                  </xsl:choose>
                </xsl:variable>
                <!-- get the path to use on the fragment -->
                <xsl:variable name="dir">
                  <xsl:if test="contains(@xlink:href,'/')">
                    <xsl:for-each
                      select="tokenize(@xlink:href,'/')
                              [not(position()=last())]">
                      <xsl:value-of select="."/>
                      <xsl:text>/</xsl:text>
                    </xsl:for-each>
                  </xsl:if>
                </xsl:variable>
                <!-- construct the fragment filename, format is
                     directory path, slash
                     filename minus the extension
                     MINUS
                     fragname (@startinglinenumber string)
                     DOT
                     value of @conformance ('frag')
                     -->
                <xsl:variable name="fragfn">
                  <xsl:value-of select="$appdir"/>
                  <xsl:text>/</xsl:text>
                  <xsl:value-of select="$dir"/>
                  <xsl:value-of
                    select="substring-before($fn,'.')"/>
                  <xsl:text>-</xsl:text>
                  <xsl:value-of
                    select="@startinglinenumber"/>
                  <xsl:text>.</xsl:text>
                  <xsl:value-of select="@conformance"/>
                </xsl:variable>
                <!-- pass to the routine that decide if this is a shell
                     script or not, and if it's the first fragment or
                     not, and outputs the hashbang first, THEN the
                     comment and then the rest of the file.
                     Or just the file. -->
                <xsl:call-template name="extinc">
                  <xsl:with-param name="extfile" select="$fragfn"/>
                  <xsl:with-param name="nthproglist"
                    select="$nthproglist"/>
                </xsl:call-template>
              </xsl:when>
              <!-- non-frag PART OF FILE RETRIEVED FROM WHOLE FILE -->
              <xsl:when test="@startinglinenumber
                              and
                              @endinglinenumber">
                <xsl:call-template name="makelisting"/>
              </xsl:when>
              <!-- WHOLE FILE NOT A FRAGMENT -->
              <xsl:otherwise>
                <xsl:variable name="wholefn">
                  <xsl:value-of select="$appdir"/>
                  <xsl:text>/</xsl:text>
                  <xsl:value-of select="@xlink:href"/>
                </xsl:variable>
                <!-- pass to the routine that decide if this is a shell
                     script or not, and if it's the first fragment or
                     not, and outputs the hashbang first, THEN the
                     comment and then the rest of the file.
                     Or just the file. -->
                <xsl:call-template name="extinc">
                  <xsl:with-param name="extfile" select="$wholefn"/>
                  <xsl:with-param name="nthproglist"
                    select="$nthproglist"/>
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <!-- NORMAL EMBEDDED LISTING -->
          <xsl:otherwise>
            <xsl:text>%    </xsl:text>
            <xsl:text>\begin</xsl:text>
            <xsl:text>{macrocode}&#xa;</xsl:text>
            <xsl:call-template name="lrtrim">
              <xsl:with-param name="text" select="."/>
            </xsl:call-template>
            <xsl:text>&#xa;%    </xsl:text>
            <xsl:text>\end</xsl:text>
            <xsl:text>{macrocode}&#xa;</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
  <!-- programlisting containing token or xref (!)
       ILLUSTRATIONS ONLY, NOT EXTRACTABLE CODE -->
      <xsl:when test="db:token or db:xref">
        <xsl:text>% \iffalse
        <xsl:choose>
          <xsl:when test="@wordsize and not(contains(@wordsize,'/'))">
            <xsl:value-of select="@wordsize"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>\small</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="@xml:id and @role">
          <xsl:text>,label=</xsl:text>
          <xsl:value-of select="@xml:id"/>
          <xsl:text>,title={</xsl:text>
          <xsl:value-of select="normalize-space(@role)"/>
          <xsl:text>}</xsl:text>
        </xsl:if>
        <xsl:text>]&#xa;</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>&#xa;% \end{Verbatim}
      </xsl:when>
  <!-- programlisting set to be ignored (!)
       ILLUSTRATIONS ONLY, NOT EXTRACTABLE CODE -->
      <xsl:when test="@condition='ignore'">
        <xsl:text>% \iffalse
        <xsl:choose>
          <xsl:when test="@wordsize and not(contains(@wordsize,'/'))">
            <xsl:value-of select="@wordsize"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>\small</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>]&#xa;</xsl:text>
        <xsl:call-template name="lrtrim">
          <xsl:with-param name="text" select="."/>
        </xsl:call-template>
        <xsl:text>
      </xsl:when>
  <!-- ALL OTHER PROGRAMLISTINGS
       especially those using start–end from a whole file -->
      <xsl:otherwise>
        <xsl:call-template name="makelisting"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!-- EXTERNAL FILE INCLUSION -->
  <xsl:template name="extinc">
    <!-- context is a <programlisting> element in an <appendix> -->
    <xsl:param name="extfile"/>
    <xsl:param name="nthproglist"/>
    <xsl:choose>
      <!-- if it's a first fragment, AND it's a shell script,
           output the hashbang line, then the CP header
           followed by the rest -->
      <xsl:when test="ancestor::db:part[@xml:id='code']
                      and
                      ancestor::db:appendix[@arch='script'
                          and @userlevel='bash']
                      and
                      $nthproglist=1">
        <!-- output first line -->
        <xsl:text>%    \begin</xsl:text>
        <xsl:text>{macrocode}&#xa;</xsl:text>
        <xsl:value-of
          select="substring-before(unparsed-text($extfile),'&#xa;')"/>
        <xsl:text>&#xa;%    \end</xsl:text>
        <xsl:text>{macrocode}&#xa;</xsl:text>
        <xsl:text>% \label{</xsl:text>
        <xsl:if test="number(@startinglinenumber)">
          <xsl:text>frag-</xsl:text>
        </xsl:if>
        <xsl:value-of select="@startinglinenumber"/>
        <xsl:text>}&#xa;</xsl:text>
        <!-- go back and do the header comment -->
        <xsl:call-template name="startappcomment">
          <xsl:with-param name="scriptflag">
            <xsl:text>go</xsl:text>
          </xsl:with-param>
        </xsl:call-template>
        <!-- then the remainder -->
        <xsl:text>%    \begin</xsl:text>
        <xsl:text>{macrocode}&#xa;</xsl:text>
        <xsl:call-template name="lrtrim">
          <xsl:with-param name="text"
            select="substring-after(unparsed-text($extfile),'&#xa;')"/>
        </xsl:call-template>
        <xsl:text>&#xa;%    \end</xsl:text>
        <xsl:text>{macrocode}&#xa;</xsl:text>
      </xsl:when>
      <!-- ELSE either it's not a bash script, or not a first file -->
      <xsl:otherwise>
        <!-- otherwise just output it -->
        <xsl:text>%    \begin</xsl:text>
        <xsl:text>{macrocode}&#xa;</xsl:text>
        <xsl:call-template name="lrtrim">
          <xsl:with-param name="text"
            select="unparsed-text($extfile)"/>
        </xsl:call-template>
        <xsl:text>&#xa;%    \end</xsl:text>
        <xsl:text>{macrocode}&#xa;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template
    match="db:userinput[not(parent::db:programlisting)]">
    <xsl:text>\emph{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:itemizedlist">
    <xsl:if test="parent::db:para">
      <xsl:text>&#xa;</xsl:text>
    </xsl:if>
    <xsl:if test="db:title">
      <xsl:text>% \</xsl:text>
      <xsl:if test="ancestor::db:itemizedlist
                    or
                    ancestor::db:orderedlist">
        <xsl:text>sub</xsl:text>
      </xsl:if>
      <xsl:text>paragraph*{</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="parent::db:para">
        <xsl:text>% \begin{enumerate*}</xsl:text>
        <xsl:text>[label=\emph{\alph*}),</xsl:text>
        <xsl:text>itemjoin={{; }},</xsl:text>
        <xsl:text>itemjoin*={{; or }}]</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>% \begin{itemize}</xsl:text>
        <xsl:if test="@spacing='compact'">
          <xsl:text>[noitemsep]</xsl:text>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:choose>
      <xsl:when test="parent::db:para">
        <xsl:text>% \end{enumerate*}</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>% \end{itemize}</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="not(parent::db:para or parent::db:note)">
      <xsl:text>&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="db:itemizedlist/db:title"/>

  <xsl:template
    match="db:varlistentry |
           db:varlistentry/db:listitem |
           db:authorgroup">
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="db:orderedlist |
                       db:procedure |
                       db:simplelist[@arch='enumerate']">
    <xsl:variable name="armour">
      <xsl:choose>
        <xsl:when test="parent::db:para">
          <xsl:text></xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>%</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!-- check format for twocol wrapper -->
    <xsl:if test="@role='twocol'">
      <xsl:value-of select="$armour"/>
      <xsl:text> \begin{multicols}{2}&#xa;</xsl:text>
    </xsl:if>
    <xsl:if test="db:title">
      <xsl:text>% \</xsl:text>
      <xsl:if test="ancestor::db:itemizedlist
                    or
                    ancestor::db:orderedlist">
        <xsl:text>sub</xsl:text>
      </xsl:if>
      <xsl:text>paragraph*{</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
    <xsl:if test="@xml:id">
      <xsl:value-of select="$armour"/>
      <xsl:text> \label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
    <xsl:value-of select="$armour"/>
    <xsl:text> \begin{enumerate</xsl:text>
    <xsl:if test="parent::db:para">
      <xsl:text>*</xsl:text>
    </xsl:if>
    <xsl:text>}</xsl:text>
    <!-- check for options needed -->
    <xsl:if test="parent::db:para
                  or
                  (@linkend and
                   preceding::db:orderedlist
                     [@xml:id=current()/@linkend])
                  or
                  @spacing='compact'
                  or
                  local-name()='simplelist'
                  or
                  @startingnumber
                  or
                  db:listitem/db:annotation">
      <xsl:text>[</xsl:text>
      <!-- inline -->
      <xsl:if test="parent::db:para">
        <xsl:text>label=\emph{\alph*}),</xsl:text>
        <xsl:text>itemjoin={{; }},</xsl:text>
        <xsl:text>itemjoin*={{; and }}</xsl:text>
      </xsl:if>
      <!-- resumption -->
      <xsl:if
        test="@linkend and
              preceding::db:orderedlist
              [@xml:id=current()/@linkend]">
        <xsl:if test="parent::db:para">
          <xsl:text>,</xsl:text>
        </xsl:if>
        <xsl:text>resume</xsl:text>
      </xsl:if>
      <!-- spacing -->
      <xsl:if test="@spacing='compact'
                    or
                    local-name()='simplelist'">
        <xsl:if
          test="parent::db:para or
                (@linkend and
                 preceding::db:orderedlist
                  [@xml:id=current()/@linkend])">
          <xsl:text>,</xsl:text>
        </xsl:if>
        <xsl:text>noitemsep</xsl:text>
      </xsl:if>
      <!-- numbering -->
      <xsl:if test="@startingnumber">
        <xsl:if
          test="parent::db:para or
                (@linkend and
                 preceding::db:orderedlist
                  [@xml:id=current()/@linkend])
                or
                @spacing='compact'
                or
                local-name()='simplelist'">
          <xsl:text>,</xsl:text>
        </xsl:if>
        <xsl:text>start=</xsl:text>
        <xsl:value-of select="@startingnumber"/>
      </xsl:if>
      <!-- no indent if there's an annotation to come -->
      <xsl:if test="db:listitem/db:annotation">
        <xsl:if test="parent::db:para
                  or
                  (@linkend and
                   preceding::db:orderedlist
                     [@xml:id=current()/@linkend])
                  or
                  @spacing='compact'
                  or
                  local-name()='simplelist'
                  or
                  @startingnumber">
          <xsl:text>,</xsl:text>
        </xsl:if>
        <xsl:text>leftmargin=0.5em</xsl:text>
      </xsl:if>
      <xsl:text>]</xsl:text>
    </xsl:if>
    <!-- set indent if the list contains annotations -->
    <xsl:if test="db:listitem/db:annotation">
      <xsl:text>\setlength{\CPKannotationindent}{1em}</xsl:text>
    </xsl:if>
    <!-- do list -->
    <xsl:text>&#xa;</xsl:text>
    <xsl:apply-templates/>
    <!-- no withdrawal of armour needed
         for inline list termination
         because the end of a list item para
         will have created a newline -->
    <xsl:text>% \end{enumerate</xsl:text>
    <xsl:if test="parent::db:para">
      <xsl:text>*</xsl:text>
    </xsl:if>
    <xsl:text>}</xsl:text>
    <!-- unset indent if the list contains annotations -->
    <xsl:if test="db:listitem/db:annotation">
      <xsl:text>\setlength{\CPKannotationindent}{0pt}</xsl:text>
    </xsl:if>
    <xsl:if test="not(parent::db:para)">
      <xsl:text>&#xa;</xsl:text>
    </xsl:if>
    <!-- check format -->
    <xsl:if test="@role='twocol'">
      <xsl:text>% \end{multicols}&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template
    match="db:orderedlist/db:title |
           db:procedure/db:title |
           db:simplelist[@arch='enumerate']/db:title"/>
  <xsl:template match="db:variablelist">
    <xsl:if test="db:title">
      <xsl:value-of select="$armour"/>
      <xsl:text>% \</xsl:text>
      <xsl:if
        test="ancestor::db:itemizedlist
              or
              ancestor::db:orderedlist">
        <xsl:text>sub</xsl:text>
      </xsl:if>
      <xsl:text>paragraph*{</xsl:text>
      <xsl:apply-templates select="db:title/node()"/>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
    <xsl:text>% \begin{description}[style=unboxed</xsl:text>
    <xsl:if test="@spacing='compact'">
      <xsl:text>,noitemsep</xsl:text>
    </xsl:if>
    <xsl:if test="db:varlistentry/db:listitem/db:annotation">
      <xsl:text>,leftmargin=0.5em</xsl:text>
    </xsl:if>
    <xsl:text>]</xsl:text>
    <!-- set indent if the list contains annotations -->
    <xsl:if test="db:varlistentry/db:listitem/db:annotation">
      <xsl:text>\setlength{\CPKannotationindent}{1em}</xsl:text>
    </xsl:if>
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>% \end{description}</xsl:text>
    <!-- unset indent if the list contains annotations -->
    <xsl:if test="db:varlistentry/db:listitem/db:annotation">
      <xsl:text>\setlength{\CPKannotationindent}{0pt}</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="db:variablelist/db:title"/>
  <xsl:template match="db:simplelist[@role='twocol']">
    <xsl:text>% \begin{multicols}{2}</xsl:text>
    <xsl:if test="@xreflabel">
      <xsl:text>[\paragraph*{</xsl:text>
      <xsl:value-of select="normalize-space(@xreflabel)"/>
      <xsl:text>}]</xsl:text>
    </xsl:if>
    <xsl:text>\begin{itemize}[noitemsep]&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>% \end{itemize}
  </xsl:template>

  <xsl:template match="db:simplelist[not(@role)
                       and
                       not(@arch)]">
    <xsl:text>% \begin{itemize}</xsl:text>
    <xsl:text>[label={},noitemsep]&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>% \end{itemize}&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="db:simplelist[@role]">
    <xsl:text>&#xa;% \</xsl:text>
    <xsl:value-of select="@role"/>
    <xsl:if test="@remap='longest'">
      <xsl:text>\longestline{</xsl:text>
      <xsl:value-of
        select="normalize-space(db:member
                [not(../db:member/string-length()
                      >string-length(.))][1])"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>\stanza&#xa;</xsl:text>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template
    match="db:member[parent::db:simplelist[@role]]">
    <xsl:text>% </xsl:text>
    <xsl:apply-templates/>
    <xsl:if test="position()=last()">
      <xsl:text>\</xsl:text>
    </xsl:if>
    <xsl:text>&amp;&#xa;</xsl:text>
  </xsl:template>
  <xsl:template
    match="db:itemizedlist/db:listitem |
           db:orderedlist/db:listitem |
           db:procedure/db:step |
           db:simplelist[not(@role)]/db:member">
    <xsl:text>% \item</xsl:text>
    <xsl:choose>
      <!-- remap a bullet or number -->
      <xsl:when test="@remap or parent::db:simplelist">
        <xsl:text>[</xsl:text>
        <xsl:value-of select="@remap"/>
        <xsl:text>]</xsl:text>
      </xsl:when>
      <!-- or prefix the number with a symbol
           (what was this for?!) -->
      <xsl:when test="@role">
        <xsl:text>\leavevmode\llap{</xsl:text>
        <xsl:value-of select="@role"/>
        <xsl:text>\thinspace}</xsl:text>
      </xsl:when>
      <!-- normally just a space -->
      <xsl:otherwise>
        <xsl:text> </xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <!-- label it if needed -->
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:apply-templates/>
    <!-- syntactic punctuation now handled by db:para template
         except for db:member, which needs terminating -->
    <xsl:if test="local-name()='member'">
      <xsl:text>&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:term">
    <xsl:if
      test="ancestor::db:part[@xml:id='code']
            and ancestor::db:chapter/@xlink:href
            and ancestor::db:chapter/@xml:id
            and not(substring-after(
                    ancestor::db:chapter/@xlink:href,'.')='dtd'
                 or substring-after(
                    ancestor::db:chapter/@xlink:href,'.')='xml'
                 or substring-after(
                    ancestor::db:chapter/@xlink:href,'.')='xsl')">
      <xsl:text>% \iffalse&#xa;</xsl:text>
      <xsl:value-of select="$sentinel"/>
      <xsl:text>&#xa;</xsl:text>
      <xsl:apply-templates select="." mode="readme"/>
      <xsl:text>% \fi&#xa;</xsl:text>
    </xsl:if>
    <xsl:text>% \item[</xsl:text>
    <xsl:if test="@annotations">
      <xsl:text>\llap{</xsl:text>
      <xsl:value-of select="@annotations"/>
      <xsl:text>\ }</xsl:text>
    </xsl:if>
    <xsl:if test="ancestor::db:variablelist[@role='numbered']">
      <xsl:number count="db:varlistentry" format="1"/>
      <xsl:text>. </xsl:text>
    </xsl:if>
    <xsl:apply-templates/>
    <xsl:if test="not(db:command[@conformance or @condition])">
      <xsl:text>:</xsl:text>
    </xsl:if>
    <xsl:text>]</xsl:text>
    <xsl:if test="following-sibling::db:listitem/db:annotation">
      <xsl:text>\par </xsl:text>
    </xsl:if>
    <xsl:if test="../@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="../@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="db:termdef">
    <xsl:text>\textbf{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}\index{</xsl:text>
    <xsl:value-of select="normalize-space(.)"/>
    <xsl:text>|textbf}</xsl:text>
  </xsl:template>
  <xsl:template match="db:formalpara[not(parent::db:abstract)]">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="db:formalpara/db:title">
    <xsl:text>% \par\textbf{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}\par\noindent\ignorespaces </xsl:text>
  </xsl:template>

  <xsl:template match="db:formalpara[parent::db:abstract]">
    <xsl:text>% \begin{itemize}[nosep]&#xa;%   \item </xsl:text>
    <xsl:apply-templates/>
    <xsl:text>% \end{itemize}&#xa;</xsl:text>
  </xsl:template>
  <xsl:template match="db:acronym">
    <xsl:call-template name="compensate-space"/>
    <xsl:choose>
      <!-- ID present: defining instance -->
      <xsl:when test="@xml:id">
        <!-- output the content (the expansion),
             quoted if required -->
        <xsl:if test="@remap='quoted'">
          <xsl:text>‘</xsl:text>
        </xsl:if>
        <xsl:apply-templates/>
        <xsl:text>\phantomsection\label{</xsl:text>
        <xsl:value-of select="@xml:id"/>
        <xsl:text>}</xsl:text>
        <xsl:if test="not(ancestor::db:table
                      or ancestor::db:informaltable
                      or ancestor::db:figure)">
          <xsl:text>\marginpar{\small\HandRight}</xsl:text>
        </xsl:if>
        <!-- output any plural specified in @role -->
        <xsl:if test="@role">
          <xsl:value-of select="@role"/>
        </xsl:if>
        <!-- end quote -->
        <xsl:if test="@remap='quoted'">
          <xsl:text>’</xsl:text>
        </xsl:if>
        <!-- output the acronym in parentheses -->
        <xsl:text> (</xsl:text>
        <xsl:choose>
          <!-- use a logo -->
          <xsl:when test="@remap">
            <xsl:value-of select="@remap"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="casestyle">
              <xsl:with-param name="text"
                select="normalize-space(@xml:id)"/>
            </xsl:call-template>
            <!-- pluralise the acronym too if needed -->
            <xsl:choose>
              <xsl:when test="@role">
                <xsl:value-of select="@role"/>
              </xsl:when>
              <xsl:when test="@arch='plural'">
                <xsl:text>s</xsl:text>
              </xsl:when>
            </xsl:choose>
          </xsl:otherwise>
        </xsl:choose>
        <!-- index it as the locus classicus
             NOT index the acro, see expansion
             BUT index expansion, see acro
-->
        <xsl:text>)\index{</xsl:text>
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>|see{</xsl:text>
        <xsl:value-of select="upper-case(@xml:id)"/>
        <xsl:text>}}\index{</xsl:text>
        <xsl:value-of select="upper-case(@xml:id)"/>
        <xsl:text>|textbf}</xsl:text>
      </xsl:when>
      <!-- no ID: normal use -->
      <xsl:otherwise>
        <!-- output the acronym -->
        <xsl:variable name="definst"
          select="//db:acronym[@xml:id=current()/.]"/>
        <xsl:choose>
          <!-- logo? -->
          <xsl:when test="$definst and @remap">
            <xsl:value-of
              select="//db:acronym[@xml:id=current()/.]
                      /@remap"/>
            <xsl:text>{}</xsl:text>
          </xsl:when>
          <!-- not in the same chapter, and never within the
               term argument to a description list -->
          <xsl:when test="$definst
                          and
                          count($definst/ancestor::db:chapter
                          |
                          ./ancestor::db:chapter)>1
                          and not(ancestor::db:term)">
            <xsl:text>\hyperref[</xsl:text>
            <xsl:value-of select="."/>
            <xsl:text>]{</xsl:text>
            <xsl:call-template name="casestyle">
              <xsl:with-param name="text"
                select="normalize-space(.)"/>
            </xsl:call-template>
            <xsl:text>}</xsl:text>
            <!-- pluralised if necessary -->
            <xsl:choose>
              <xsl:when test="@role">
                <xsl:value-of select="@role"/>
              </xsl:when>
              <xsl:when test="@arch='plural'">
                <xsl:text>s</xsl:text>
              </xsl:when>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="casestyle">
              <xsl:with-param name="text"
                select="normalize-space(.)"/>
            </xsl:call-template>
            <!-- pluralised if necessary -->
            <xsl:if test="@role">
              <xsl:value-of select="@role"/>
            </xsl:if>
          </xsl:otherwise>
        </xsl:choose>
        <!-- index it -->
        <xsl:if test="//db:acronym[@xml:id=current()/.]">
          <xsl:if test="ancestor::db:footnote">
            <xsl:text>\protect</xsl:text>
          </xsl:if>
          <xsl:text>\index{</xsl:text>
          <xsl:value-of select="normalize-space(.)"/>
          <xsl:text>}</xsl:text>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="db:citetitle">
    <xsl:call-template name="compensate-space"/>
    <xsl:choose>
      <xsl:when
        test="not(@linkend) or
              //*[@xml:id=current()/@linkend]/@areflabel='book'">
        <xsl:text>\emph{</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>“</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:choose>
      <xsl:when test="@linkend">
        <xsl:apply-templates
          select="//*[@xml:id=current()/@linkend]/db:title/node()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:choose>
      <xsl:when
        test="not(@linkend) or
              //*[@xml:id=current()/@linkend]/@areflabel='book'">
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>”</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="//db:biblioentry[@xml:id=current()/@linkend]">
      <xsl:text> \parencite{</xsl:text>
      <xsl:value-of
        select="translate(normalize-space(@linkend),' ',',')"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:code">
    <xsl:call-template name="compensate-space"/>
    <xsl:choose>
      <xsl:when test="@language='RE'">
        <xsl:text>\verb`/</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>/`</xsl:text>
      </xsl:when>
      <xsl:when test="@language='Cocoon'">
        <xsl:text>\verb`{</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>}`</xsl:text>
      </xsl:when>
      <xsl:when test="@language='bash'">
        <xsl:text>\verb`</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>`</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\texttt{</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>}</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="db:classname">
    <xsl:call-template name="compensate-space"/>
    <xsl:choose>
      <xsl:when
        test="ancestor::db:part[@xml:id='code'] and
              not(ancestor::db:footnote) and
              not(ancestor::db:title) and
              not(ancestor::db:term) and
              not(.=ancestor::db:annotation/@xreflabel) and
              (@language='TeX'
              or @language='LaTeXe'
              or @language='LaTeX'
              or not(@language))
              and not(preceding-sibling::db:classname
                      [.=current()/.])">
        <xsl:text>\DescribeClass{</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\index{</xsl:text>
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>@\textsf{</xsl:text>
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>} (class)</xsl:text>
        <xsl:text>}</xsl:text>
        <xsl:text>\index{classes:!</xsl:text>
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>@\textsf{</xsl:text>
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>}}</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <!-- always sans -->
    <xsl:text>\textsf{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
    <xsl:if test="@condition">
      <xsl:text>\oarg{</xsl:text>
      <xsl:value-of select="normalize-space(@condition)"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:if test="@conformance">
      <xsl:text>\marg{</xsl:text>
      <xsl:value-of select="normalize-space(@conformance)"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:command">
    <xsl:call-template name="compensate-space"/>
    <xsl:if
      test="@role and not(.='TeX') and not(.='LaTeX') and
            not(ancestor::db:term) and
            not(.=ancestor::db:annotation/@xreflabel) and
            (@language='TeX'
            or @language='LaTeXe'
            or @language='LaTeX'
            or not(@language))
            and not(preceding-sibling::db:command[.=current()/.])">
      <xsl:text>\DescribeMacro{\</xsl:text>
      <xsl:choose>
        <xsl:when
          test="@language='TeX' or
                @language='LaTeXe' or
                @language='LaTeX' or
                not(@language)
                and contains(.,'{')">
          <xsl:value-of
            select="substring-before(normalize-space(.),'{')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="normalize-space(.)"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:choose>
      <xsl:when
        test="(@language='TeX' or
              @language='LaTeXe' or
              @language='LaTeX' or
              not(@language))
              and (not(@role) or parent::db:term)">
        <xsl:if test="parent::db:term and
                      @userlevel!='optional'">
          <xsl:text>\llap{$\star$\enspace}</xsl:text>
        </xsl:if>
        <xsl:choose>
          <xsl:when test="parent::db:term">
            <xsl:text>\texttt{\textbackslash{}</xsl:text>
            <xsl:apply-templates/>
            <xsl:text>}</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="avoidverb"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <!-- use \verb for TeX commands if needed -->
      <xsl:when
        test="@language='TeX' or
              @language='LaTeXe' or
              @language='LaTeX' or
              not(@language)">
        <xsl:call-template name="avoidverb"/>
      </xsl:when>
      <!-- bash -->
      <xsl:when test="@language='bash'">
        <xsl:call-template name="avoidverb"/>
        <xsl:if test="@userlevel">
          <xsl:text>\thinspace(</xsl:text>
          <xsl:value-of select="@userlevel"/>
          <xsl:text>)</xsl:text>
        </xsl:if>
      </xsl:when>
      <!-- other language commands have no backslash -->
      <xsl:when test="@condition='nolit'">
        <xsl:call-template name="avoidverb"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="avoidverb"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="@condition and
                  @condition!='nolit'">
      <xsl:text>\oarg{</xsl:text>
      <xsl:value-of
        select="normalize-space(@condition)"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:if test="@conformance">
      <xsl:text>\marg{</xsl:text>
      <xsl:value-of
        select="normalize-space(@conformance)"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:emphasis">
    <xsl:call-template name="compensate-space"/>
    <xsl:choose>
      <xsl:when test="@role='strong'">
        <xsl:text>\textbf{</xsl:text>
      </xsl:when>
      <xsl:when test="ancestor::db:emphasis">
        <xsl:text>\textbf{\upshape </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\emph{</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:email">
    <xsl:text>\url{</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:envar">
    <xsl:call-template name="compensate-space"/>
    <xsl:if test="@role and
                  not(ancestor::db:term) and
                  not(.=ancestor::db:annotation/@xreflabel) and
                  (@language='TeX'
                  or @language='LaTeXe'
                  or @language='LaTeX'
                  or not(@language))
                  and not(preceding-sibling::db:envar
                          [.=current()/.])">
      <xsl:text>\DescribeEnv{</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>\texttt{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
    <xsl:if test="@condition">
      <xsl:text>\oarg{</xsl:text>
      <xsl:value-of select="normalize-space(@condition)"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:if test="@conformance">
      <xsl:text>\marg{</xsl:text>
      <xsl:value-of select="normalize-space(@conformance)"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:errortext">
    <xsl:call-template name="compensate-space"/>
    <xsl:text>\DescribeError{</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>}\textsf{</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:exceptionname">
    <xsl:call-template name="compensate-space"/>
    <xsl:text>{\sffamily </xsl:text>
    <xsl:call-template name="casestyle">
      <xsl:with-param name="text" select="normalize-space(.)"/>
    </xsl:call-template>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:filename | db:systemitem">
    <xsl:call-template name="compensate-space"/>
    <xsl:call-template name="avoidverb"/>
    <xsl:if
      test="contains('.,;:?!',
            substring(following-sibling::node()[1],1,1))">
      <xsl:text>\thinspace</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:firstterm">
    <xsl:call-template name="compensate-space"/>
    <xsl:text>\textbf{\emph{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}}\index{</xsl:text>
    <xsl:value-of select="normalize-space(.)"/>
    <xsl:text>|textbf}</xsl:text>
  </xsl:template>
  <xsl:template match="db:footnote">
    <xsl:text>\footnote{%&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:foreignphrase">
    <xsl:call-template name="compensate-space"/>
    <xsl:choose>
      <xsl:when test="@xml:lang='jp'">
        <xsl:text>\cjktext{</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:when test="@xml:lang='ga'">
        <!--        <xsl:text>\texteiad{</xsl:text>-->
        <xsl:text>{</xsl:text>
        <xsl:value-of select="@remap"/>
        <xsl:apply-templates/>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:when test="@xml:lang!=/db:book/@xml:lang">
        <xsl:text>\foreignlanguage{</xsl:text>
        <xsl:value-of
          select="$langs/language[concat(@Lang,'-',@Country)
                  =current()/@xml:lang]/@babel"/>
        <xsl:text>}{</xsl:text>
        <xsl:choose>
          <xsl:when test="$langs/language
                          [@iso=current()/@xml:lang]
                          /@wsys='la'">
            <xsl:text>\emph{</xsl:text>
            <xsl:apply-templates/>
            <xsl:text>}</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates/>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\emph{</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>}</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="db:function">
    <xsl:call-template name="compensate-space"/>
    <xsl:if test="not(@role) and not(@language)">
      <xsl:text>\DescribeFunction{</xsl:text>
      <xsl:value-of select="."/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>\texttt{</xsl:text>
    <xsl:value-of select="."/>
    <xsl:if test="@language='XSLT'">
      <xsl:text>()</xsl:text>
    </xsl:if>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <!-- gui menu stuff -->
  <xsl:template match="db:guimenu">
    <xsl:call-template name="compensate-space"/>
    <xsl:choose>
      <xsl:when
        test="starts-with(local-name(
              preceding-sibling::node()[1]),'gui')">
        <xsl:message>
          <xsl:text>ERROR: top-level guimenu "</xsl:text>
          <xsl:value-of select="."/>
          <xsl:text>" preceded by </xsl:text>
          <xsl:value-of select="name(preceding-sibling::*[1])"/>
        </xsl:message>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\menu{</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates/>
    <xsl:if
      test="not(starts-with(local-name(
            following-sibling::node()[1]),'gui'))">
      <xsl:text>}</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:guisubmenu">
    <xsl:choose>
      <xsl:when
        test="starts-with(local-name(
              preceding-sibling::node()[1]),'gui')">
        <xsl:text>></xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\menu{</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates/>
    <xsl:if
      test="not(starts-with(local-name(
            following-sibling::node()[1]),'gui'))">
      <xsl:text>}</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:guimenuitem">
    <xsl:choose>
      <xsl:when
        test="starts-with(local-name(
              preceding-sibling::node()[1]),'gui')">
        <xsl:text>></xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\menu{</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates/>
    <xsl:if
      test="not(starts-with(local-name(
            following-sibling::node()[1]),'gui'))">
      <xsl:text>}</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:guiicon">
    <xsl:call-template name="compensate-space"/>
    <xsl:choose>
      <!-- use FontAwesome icon -->
      <xsl:when test="@remap">
        <xsl:text>\fa</xsl:text>
        <xsl:value-of select="@remap"/>
        <xsl:text>{}</xsl:text>
      </xsl:when>
      <!-- otherwise supplied image -->
      <xsl:otherwise>
        <xsl:text>\includegraphics</xsl:text>
        <xsl:if test="@annotations">
          <xsl:text>[</xsl:text>
          <xsl:value-of select="@annotations"/>
          <xsl:text>]</xsl:text>
        </xsl:if>
        <xsl:text>{</xsl:text>
        <xsl:value-of select="@xlink:href"/>
        <xsl:text>}</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <!-- element is normally empty -->
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="db:guilabel">
    <xsl:call-template name="compensate-space"/>
    <xsl:text>{\renewmenumacro{\keys}{angularkeys}\keys{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}}</xsl:text>
  </xsl:template>
  <xsl:template match="db:guibutton">
    <xsl:call-template name="compensate-space"/>
    <xsl:text>{\changemenucolor{keys}{bg}%</xsl:text>
    <xsl:text>  {HTML}{CDCDCD}\renewmenumacro{\keys}%</xsl:text>
    <xsl:text>  {shadowedangularkeys}\keys{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}}</xsl:text>
  </xsl:template>
  <xsl:template match="db:important">
    <xsl:text>\begingroup\sffamily\bfseries&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>%   \endgroup </xsl:text>
  </xsl:template>
  <xsl:template match="db:indexterm">
    <xsl:text>% \index{</xsl:text>
    <xsl:apply-templates/>
    <xsl:choose>
      <xsl:when test="@xml:id and not(db:see)">
        <xsl:text>|textbf</xsl:text>
      </xsl:when>
    </xsl:choose>
    <xsl:text>}&#xa;</xsl:text>
  </xsl:template>
  <xsl:template match="db:keycap">
    <xsl:call-template name="compensate-space"/>
    <xsl:text>\keys{</xsl:text>
    <xsl:choose>
      <xsl:when test="contains('$%&#38;#',.)">
        <xsl:text>\</xsl:text>
        <xsl:apply-templates/>
      </xsl:when>
      <xsl:when test=".='^' or .='~'">
        <xsl:text>\</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>{}</xsl:text>
      </xsl:when>
      <xsl:when test="contains('{&lt;|&gt;}',/)">
        <xsl:text>\[</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>\]</xsl:text>
      </xsl:when>
      <xsl:when test=".='\\'">
        <xsl:text>\textbackslash</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:link">
    <xsl:call-template name="compensate-space"/>
    <xsl:choose>
      <!-- with hyperref or hypdoc -->
      <xsl:when
        test="//db:seg[.='hyperref' or .='hypdoc']
              or
              $prepost/db:procedure[@xml:id='prepackage']
              /db:step[contains(@condition,'doc')
                       and
                       @remap='hypdoc']
                       [db:constructorsynopsis='']">
        <xsl:text>\href{</xsl:text>
        <xsl:value-of select="@xlink:href"/>
        <xsl:text>}{</xsl:text>
        <xsl:choose>
          <xsl:when test="normalize-space(.)=''">
            <xsl:text>\nolinkurl{</xsl:text>
            <xsl:value-of
              select="replace(@xlink:href,'#','\\#')"/>
            <xsl:text>}</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates/>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
          <!-- empty link text -->
          <xsl:when test=".=''">
            <xsl:text>\url{</xsl:text>
            <xsl:value-of select="@xlink:href"/>
            <xsl:text>}</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>{\color{blue}\uline{</xsl:text>
            <xsl:apply-templates/>
            <xsl:text>}}\footnote{\protect\url{</xsl:text>
            <xsl:value-of select="@xlink:href"/>
            <xsl:text>}}</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if
    test="contains('.,;:?!',
                    substring(following-sibling::node()[1],1,1))">
      <xsl:text>\thinspace</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:literallayout[not(@audience='none')]">
    <xsl:if test="parent::db:para">
      <xsl:call-template name="compensate-space"/>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="@language='LaTeXe' or
                      @language='LaTeX' or
                      @language='TeX' or
                      not(@language)">
        <xsl:value-of select="."/>
      </xsl:when>
      <xsl:when test="@language='bash'">
        <xsl:if
          test="ancestor::db:part[@xml:id='code'] and
                ancestor::db:chapter/@xlink:href and
                ancestor::db:chapter/@xml:id and
                not(substring-after(
                    ancestor::db:chapter/@xlink:href,'.')='dtd'
                 or substring-after(
                    ancestor::db:chapter/@xlink:href,'.')='xml'
                 or substring-after(
                    ancestor::db:chapter/@xlink:href,'.')='xsl')">
          <xsl:text>% \iffalse&#xa;</xsl:text>
          <xsl:value-of select="$sentinel"/>
          <xsl:text>&#xa;</xsl:text>
          <xsl:apply-templates select="." mode="readme"/>
          <xsl:text>% \fi&#xa;</xsl:text>
        </xsl:if>
        <xsl:text>% \iffalse
</xsl:text>
        <xsl:text>\begin{lstlisting}</xsl:text>
        <xsl:text>[basicstyle=\small\color{DarkGreen}</xsl:text>
        <xsl:text>\ttfamily,language=</xsl:text>
        <xsl:value-of select="/db:book/@userlevel"/>
        <xsl:text>]&#xa;</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>\end{lstlisting}&#xa;</xsl:text>
        <xsl:text>% \iffalse
</xsl:text>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="db:methodname">
    <xsl:call-template name="compensate-space"/>
    <xsl:if test="not(@role) and not(ancestor::db:tgroup)">
      <xsl:text>\DescribeTemplate{</xsl:text>
      <xsl:value-of select="."/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>\texttt{</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:note[not(parent::db:para)]">
    <xsl:text>% \begin{Sbox}\begin{minipage}</xsl:text>
    <xsl:text>{.85\columnwidth}\raggedright\sffamily</xsl:text>
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>&#xa;% \end{minipage}\end{Sbox}</xsl:text>
    <xsl:text>{\fboxsep1em\centering\fbox{\TheSbox}</xsl:text>
    <xsl:text>\par\smallskip}&#xa;</xsl:text>
  </xsl:template>
  <xsl:template match="db:note[parent::db:para]">
    <xsl:choose>
      <xsl:when test="@role='marginal'">
        <xsl:text>\marginnote{\raggedright </xsl:text>
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="compensate-space"/>
        <xsl:text>{\bfseries </xsl:text>
        <xsl:apply-templates/>
        <xsl:text>}</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="db:option">
    <xsl:call-template name="compensate-space"/>
    <xsl:if test="@role and
                  not(ancestor::db:title) and
                  not(ancestor::db:term) and
                  not(ancestor::db:footnote) and
                  not(ancestor::db:table) and
                  not(ancestor::db:informaltable) and
                  not(ancestor::db:figure)
                  and not(preceding-sibling::db:option
                          [.=current()/.])">
      <xsl:text>\DescribeOption{</xsl:text>
      <xsl:value-of select="."/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="@condition">
        <xsl:text>\oarg{</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:when test="@conformance">
        <xsl:text>\marg{</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <!-- otherwise it’s for a package or command -->
      <xsl:otherwise>
        <xsl:text>\textbf{\texttt{</xsl:text>
        <xsl:if test="@language='bash'">
          <xsl:text>-</xsl:text>
        </xsl:if>
        <xsl:apply-templates/>
        <xsl:if test="@language='bash' and @conformance='colon'">
          <xsl:text>:</xsl:text>
        </xsl:if>
        <xsl:text>}}</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="db:package">
    <xsl:call-template name="compensate-space"/>
    <xsl:if test="@role and
                  not(ancestor::db:title) and
                  not(ancestor::db:abstract) and
                  not(ancestor::db:acknowledgements) and
                  not(ancestor::db:term) and
                  not(ancestor::db:footnote) and
                  not(ancestor::db:table) and
                  not(ancestor::db:informaltable) and
                  not(ancestor::db:figure)
                  and not(preceding-sibling::db:package
                          [.=current()/.])">
      <xsl:text>\DescribePackage{</xsl:text>
      <xsl:choose>
      <xsl:when test=".=''">
        <xsl:value-of select="/*[1]/@xml:id"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <!-- use sf but heavier when already sf -->
    <xsl:text>\textsf{</xsl:text>
    <xsl:choose>
      <xsl:when test=".=''">
        <xsl:value-of select="/*[1]/@xml:id"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:personname">
    <xsl:call-template name="compensate-space"/>
    <xsl:apply-templates/>
    <xsl:text>\index{</xsl:text>
    <xsl:value-of select="normalize-space(db:surname)"/>
    <xsl:text>!</xsl:text>
    <xsl:value-of select="normalize-space(db:firstname)"/>
    <xsl:if test="db:othername">
      <xsl:text> </xsl:text>
      <xsl:value-of select="normalize-space(db:othername)"/>
    </xsl:if>
    <xsl:text>}</xsl:text>
  </xsl:template>

  <xsl:template match="db:personname/db:firstname">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="db:personname/db:othername">
    <xsl:if test="preceding-sibling::node()!=' '">
      <xsl:text> </xsl:text>
    </xsl:if>
    <xsl:value-of select="."/>
    <xsl:if
      test="string-length(normalize-space(.)) = 1
            or
            (.=upper-case(.)
             and
             substring(.,string-length(.))!='.')">
      <xsl:text>.</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="db:personname/db:surname">
    <xsl:if test="preceding-sibling::node()!=' '">
      <xsl:text> </xsl:text>
    </xsl:if>
    <xsl:value-of select="."/>
  </xsl:template>
  <xsl:template match="db:primary">
    <xsl:value-of select="normalize-space(.)"/>
    <xsl:if test="@sortas">
      <xsl:text>@</xsl:text>
      <xsl:value-of select="normalize-space(@sortas)"/>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:productname">
    <xsl:call-template name="compensate-space"/>
    <xsl:text>\emph{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
    <xsl:if test="@userlevel">
      <xsl:text>\thinspace(</xsl:text>
      <xsl:value-of select="@userlevel"/>
      <xsl:text>)</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:quote | db:phrase | db:wordasword">
    <xsl:call-template name="compensate-space"/>
    <xsl:if test="count(preceding-sibling::node())=0">
      <xsl:text>\leavevmode\llap</xsl:text>
    </xsl:if>
    <xsl:text>`</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>'</xsl:text>
    <xsl:if test="@linkend">
      <xsl:text>&#x0a;</xsl:text>
      <xsl:call-template name="makeref"/>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:replaceable">
    <xsl:call-template name="compensate-space"/>
    <!-- make sure we use the same delimiter
         as the parent element does -->
    <xsl:variable name="delim">
      <!-- in order of preference -->
      <xsl:choose>
        <xsl:when test="not(contains(.,'|'))">
          <xsl:text>|</xsl:text>
        </xsl:when>
        <xsl:when test="not(contains(.,'+'))">
          <xsl:text>+</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>`</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <!-- stop-start \verb when in a command or systemitem -->
      <xsl:when test="parent::db:command
                      or parent::db:systemitem
                      or parent::db:filename">
        <xsl:value-of select="$delim"/>
        <xsl:text>{\ttfamily\itshape </xsl:text>
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>}</xsl:text>
        <xsl:text>\verb</xsl:text>
        <xsl:value-of select="$delim"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\uline{</xsl:text>
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>}</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="db:secondary | db:tertiary">
    <xsl:text>!</xsl:text>
    <xsl:value-of select="normalize-space(.)"/>
    <xsl:if test="@sortas">
      <xsl:text>@</xsl:text>
      <xsl:value-of select="normalize-space(@sortas)"/>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:see">
    <xsl:text>|see{</xsl:text>
    <xsl:value-of select="normalize-space(.)"/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:superscript">
    <xsl:text>\textsuperscript{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:markup">
    <xsl:call-template name="compensate-space"/>
    <xsl:text>\texttt{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:tag">
    <xsl:call-template name="compensate-space"/>
    <xsl:variable name="tagval" select="normalize-space(.)"/>
    <!-- unclear what @role was destined for -->
    <xsl:if
      test="@role and
            not(ancestor::db:title) and
            not(ancestor::db:term) and
            not(ancestor::db:footnote) and
            not(ancestor::db:table) and
            not(ancestor::db:informaltable) and
            not(ancestor::db:figure)
            and not(preceding-sibling::db:tag[.=$tagval])">
      <xsl:text>\Describe</xsl:text>
      <xsl:choose>
        <xsl:when test="@class='element' or not(@class)">
          <xsl:text>Element</xsl:text>
        </xsl:when>
        <xsl:when test="@class='attribute'">
          <xsl:text>Attribute</xsl:text>
        </xsl:when>
        <xsl:when test="@class='attvalue'">
          <xsl:text>AttributeValue</xsl:text>
        </xsl:when>
        <xsl:when test="@class='genentity'">
          <xsl:text>Entity</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>Error</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>{</xsl:text>
      <xsl:apply-templates/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>\texttt{</xsl:text>
    <xsl:choose>
      <xsl:when test="@class='starttag'
                      or
                      @class='emptytag'
                      or
                      @class='endtag'
                      or
                      not(@class)">
        <xsl:text>&lt;</xsl:text>
        <xsl:if test="@class='endtag'">
          <xsl:text>/</xsl:text>
        </xsl:if>
      </xsl:when>
      <xsl:when test="@class='attvalue'">
        <xsl:if
          test="name(preceding-sibling::node()[1])='tag'
                and preceding-sibling::*[1]/@class='attribute'">
          <xsl:text>\unskip=</xsl:text>
        </xsl:if>
        <xsl:text>"</xsl:text>
      </xsl:when>
      <xsl:when
        test="@class='attribute' and not(@conformance='noat')">
        <xsl:if
          test="name(preceding-sibling::node()[1])='tag'
                and (
                preceding-sibling::*[1]/@class='element' or
                preceding-sibling::*[1]/@class='emptytag' or
                preceding-sibling::*[1]/@class='starttag' or
                preceding-sibling::*[1][not(@class)]
                )">
          <xsl:text>\unskip/</xsl:text>
        </xsl:if>
        <xsl:text>@</xsl:text>
      </xsl:when>
      <xsl:when test="@class='genentity'">
        <xsl:text>\&amp;</xsl:text>
      </xsl:when>
      <xsl:when test="@class='declaration'">
        <xsl:text>!</xsl:text>
      </xsl:when>
      <xsl:when test="@class='numcharref'">
        <xsl:text>\&amp;\#x</xsl:text>
      </xsl:when>
    </xsl:choose>
    <xsl:value-of select="$tagval"/>
    <xsl:choose>
      <xsl:when test="@class='starttag'
                      or
                      @class='emptytag'
                      or
                      @class='endtag'
                      or
                      not(@class)">
        <xsl:if test="@class='emptytag'">
          <xsl:text>/</xsl:text>
        </xsl:if>
        <xsl:text>&gt;</xsl:text>
      </xsl:when>
      <xsl:when test="@class='attvalue'">
        <xsl:text>"</xsl:text>
      </xsl:when>
      <xsl:when test="@class='genentity' or @class='numcharref'">
        <xsl:text>;</xsl:text>
      </xsl:when>
    </xsl:choose>
    <xsl:text>}</xsl:text>
    <!-- INDEXING MARKUP -->
    <xsl:text>\index{</xsl:text>
    <!-- sort value -->
    <xsl:value-of select="$tagval"/>
    <!-- style for entry (different delimiter to makeindex default) -->
    <xsl:text>=\texttt{</xsl:text>
    <!-- STAGO -->
    <xsl:choose>
      <xsl:when test="@class='element' or not(@class)">
        <xsl:text>\char'074 </xsl:text>
      </xsl:when>
      <!-- safety first in outputting an @ sign in an index -->
      <xsl:when test="@class='attribute'">
        <xsl:text>\char'100 </xsl:text>
      </xsl:when>
      <xsl:when test="@class='attvalue'">
        <xsl:text>"</xsl:text>
      </xsl:when>
      <xsl:when test="@class='emptytag'">
        <xsl:text>\char'074 </xsl:text>
      </xsl:when>
      <xsl:when test="@class='genentity'">
        <xsl:text>\&amp;</xsl:text>
      </xsl:when>
      <xsl:when test="@class='localname'">
        <xsl:text></xsl:text>
      </xsl:when>
      <xsl:when test="@class='starttag'">
        <xsl:text>\char'074 </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>?</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="$tagval"/>
    <!-- TAGC -->
    <xsl:choose>
      <xsl:when test="@class='element' or not(@class)">
        <xsl:text>\char'076 </xsl:text>
      </xsl:when>
      <xsl:when test="@class='attribute'">
        <xsl:text></xsl:text>
      </xsl:when>
      <xsl:when test="@class='attvalue'">
        <xsl:text>"</xsl:text>
      </xsl:when>
      <xsl:when test="@class='emptytag'">
        <xsl:text>/\char'076 </xsl:text>
      </xsl:when>
      <xsl:when test="@class='genentity'">
        <xsl:text>;</xsl:text>
      </xsl:when>
      <xsl:when test="@class='localname'">
        <xsl:text></xsl:text>
      </xsl:when>
      <xsl:when test="@class='starttag'">
        <xsl:text>\char'076 </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>?</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>} (</xsl:text>
    <!-- human names -->
    <xsl:choose>
      <xsl:when test="@class='element' or not(@class)">
        <xsl:text>element</xsl:text>
      </xsl:when>
      <xsl:when test="@class='attribute'">
        <xsl:text>attribute</xsl:text>
      </xsl:when>
      <xsl:when test="@class='attvalue'">
        <xsl:text>attribute value</xsl:text>
      </xsl:when>
      <xsl:when test="@class='emptytag'">
        <xsl:text>empty element</xsl:text>
      </xsl:when>
      <xsl:when test="@class='genentity'">
        <xsl:text>general entity</xsl:text>
      </xsl:when>
      <xsl:when test="@class='localname'">
        <xsl:text>local name</xsl:text>
      </xsl:when>
      <xsl:when test="@class='starttag'">
        <xsl:text>start-tag</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="@class"/>
      </xsl:otherwise>
    </xsl:choose>
    <!-- two-level index -->
    <xsl:text>)}\index{</xsl:text>
    <!-- top level: pluralised class with colon -->
    <xsl:choose>
      <xsl:when test="@class='element' or not(@class)">
        <xsl:text>elements</xsl:text>
      </xsl:when>
      <xsl:when test="@class='attribute'">
        <xsl:text>attributes</xsl:text>
      </xsl:when>
      <xsl:when test="@class='attvalue'">
        <xsl:text>attribute values</xsl:text>
      </xsl:when>
      <xsl:when test="@class='emptytag'">
        <xsl:text>empty elements</xsl:text>
      </xsl:when>
      <xsl:when test="@class='genentity'">
        <xsl:text>general entities</xsl:text>
      </xsl:when>
      <xsl:when test="@class='localname'">
        <xsl:text>local names</xsl:text>
      </xsl:when>
      <xsl:when test="@class='starttag'">
        <xsl:text>start-tags</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="@class"/>
        <xsl:text>s</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <!-- colon and second-level delimiter -->
    <xsl:text>:!</xsl:text>
    <!-- sort value -->
    <xsl:value-of select="$tagval"/>
    <xsl:text>=\texttt{</xsl:text>
    <!-- STAGO -->
    <xsl:choose>
      <xsl:when test="@class='element' or not(@class)">
        <xsl:text>\char'074 </xsl:text>
      </xsl:when>
      <xsl:when test="@class='attribute'">
        <xsl:text>\char'100 </xsl:text>
      </xsl:when>
      <xsl:when test="@class='attvalue'">
        <xsl:text>"</xsl:text>
      </xsl:when>
      <xsl:when test="@class='emptytag'">
        <xsl:text>\char'074 </xsl:text>
      </xsl:when>
      <xsl:when test="@class='genentity'">
        <xsl:text>\&amp;</xsl:text>
      </xsl:when>
      <xsl:when test="@class='localname'">
        <xsl:text></xsl:text>
      </xsl:when>
      <xsl:when test="@class='starttag'">
        <xsl:text>\char'074 </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>?</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="$tagval"/>
    <!-- TAGC -->
    <xsl:choose>
      <xsl:when test="@class='element' or not(@class)">
        <xsl:text>\char'076 </xsl:text>
      </xsl:when>
      <xsl:when test="@class='attribute'">
        <xsl:text></xsl:text>
      </xsl:when>
      <xsl:when test="@class='attvalue'">
        <xsl:text>"</xsl:text>
      </xsl:when>
      <xsl:when test="@class='emptytag'">
        <xsl:text>/\char'076</xsl:text>
      </xsl:when>
      <xsl:when test="@class='genentity'">
        <xsl:text>;</xsl:text>
      </xsl:when>
      <xsl:when test="@class='localname'">
        <xsl:text></xsl:text>
      </xsl:when>
      <xsl:when test="@class='starttag'">
        <xsl:text>\char'076</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>?</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>}}</xsl:text>
    <xsl:if
      test="name(following-sibling::node()[1])='' and
            starts-with(following-sibling::node()[1],'s')">
      <xsl:text>\kern1pt{}</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:task">
    <xsl:text>% \par\smallskip\leavevmode
    <xsl:value-of
      select="parent::db:annotation/@xreflabel
              |
              ancestor::db:varlistentry/db:term"/>
    <xsl:text>}%
    <xsl:for-each select="*">
      <xsl:text> </xsl:text>
      <xsl:value-of select="normalize-space(.)"/>
    </xsl:for-each>
    <xsl:text>}}\par\smallskip&#xa;</xsl:text>
  </xsl:template>
  <xsl:template match="db:note/db:title">
    <xsl:text>% \subsubsection*{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}&#xa;</xsl:text>
  </xsl:template>
  <xsl:template match="db:token">
    <xsl:call-template name="compensate-space"/>
    <xsl:text>{\normalfont\itshape</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:type[@role='font' or not(@role)]">
    <xsl:text>{</xsl:text>
    <xsl:choose>
      <!-- empty and fontfamily means identify the font -->
      <xsl:when test=".='' and @fontfamily">
        <xsl:choose>
          <xsl:when test="@fontfamily='sans'">
            <xsl:text>\sffamily </xsl:text>
          </xsl:when>
          <xsl:when test="@fontfamily='roman'">
            <xsl:text>\rmfamily </xsl:text>
          </xsl:when>
          <xsl:when test="@fontfamily='monospace'">
            <xsl:text>\ttfamily </xsl:text>
          </xsl:when>
        </xsl:choose>
        <xsl:text>\printexternalcurrentfont</xsl:text>
      </xsl:when>
      <!-- otherwise format something -->
      <xsl:otherwise>
        <xsl:if test="@fontcolor">
          <xsl:text>\color</xsl:text>
          <xsl:if test="@conformance">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="@conformance"/>
            <xsl:text>[</xsl:text>
          </xsl:if>
          <xsl:text>{</xsl:text>
          <xsl:value-of select="@fontcolor"/>
          <xsl:text>}</xsl:text>
        </xsl:if>
        <xsl:if test="@fontencoding">
          <xsl:text>\fontencoding{</xsl:text>
          <xsl:value-of select="upper-case(@fontencoding)"/>
          <xsl:text>}</xsl:text>
        </xsl:if>
        <xsl:if test="@fontfamily">
          <!-- special cases -->
          <xsl:choose>
            <xsl:when test="@fontfamily='sans'">
              <xsl:text>\sffamily </xsl:text>
            </xsl:when>
            <xsl:when test="@fontfamily='roman'">
              <xsl:text>\rmfamily </xsl:text>
            </xsl:when>
            <xsl:when test="@fontfamily='monospace'">
              <xsl:text>\ttfamily </xsl:text>
            </xsl:when>
          </xsl:choose>
        </xsl:if>
        <xsl:if test="@fontface">
          <xsl:text>\fontfamily{</xsl:text>
          <xsl:value-of select="@fontface"/>
          <xsl:text>}</xsl:text>
        </xsl:if>
        <xsl:if test="@fontshape">
          <xsl:text>\</xsl:text>
          <xsl:value-of select="@fontshape"/>
          <xsl:text>shape</xsl:text>
        </xsl:if>
        <xsl:if test="@fontseries">
          <xsl:text>\fontseries{</xsl:text>
          <xsl:value-of select="@fontseries"/>
          <xsl:text>}</xsl:text>
        </xsl:if>
        <xsl:if test="@fontsize">
          <xsl:text>\fontsize{</xsl:text>
          <xsl:choose>
            <xsl:when test="contains(@fontsize,'/')">
              <xsl:value-of
                select="substring-before(@fontsize,'/')"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="@fontsize"/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>}{</xsl:text>
          <xsl:choose>
            <xsl:when test="contains(@fontsize,'/')">
              <xsl:value-of
                select="substring-after(@fontsize,'/')"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>0</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>}</xsl:text>
        </xsl:if>
        <xsl:text>\selectfont </xsl:text>
        <xsl:apply-templates/>
        <xsl:if test="@fontsize and contains(@fontsize,'/')">
          <xsl:text>\par</xsl:text>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:type[@role='color']">
    <xsl:call-template name="compensate-space"/>
    <xsl:text>\DescribeColor{</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>}\texttt{</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  <xsl:template match="db:uri">
    <xsl:call-template name="compensate-space"/>
    <xsl:choose>
      <xsl:when test="@xlink:href and .!=''">
        <xsl:apply-templates/>
        <xsl:text>\footnote{\url{</xsl:text>
        <xsl:value-of select="@xlink:href"/>
        <xsl:text>}}</xsl:text>
      </xsl:when>
      <xsl:when test="not(@xlink:href) and .!=''">
        <xsl:text>\url{</xsl:text>
        <xsl:if test="@type">
          <xsl:value-of select="@type"/>
          <xsl:text>:</xsl:text>
        </xsl:if>
        <xsl:apply-templates/>
        <xsl:text>}</xsl:text>
      </xsl:when>
    </xsl:choose>
    <xsl:if test="@xreflabel and @type='news'">
      <xsl:text> at \textlangle\verb`</xsl:text>
      <xsl:value-of select="@xreflabel"/>
      <xsl:text>`\textrangle{}</xsl:text>
    </xsl:if>
  </xsl:template>
  <!-- Counters and Lengths and Variables -->
  <xsl:template
    match="db:varname[not(ancestor::db:footnote
                       or ancestor::db:abstract)]
           |
           db:parameter[not(ancestor::db:footnote
                         or ancestor::db:abstract)]">
    <xsl:call-template name="compensate-space"/>
    <xsl:if
      test="@role and @role!=''
            and
            not(@condition='nodesc')
            and
            not(count(parent::db:para
                      /preceding-sibling::db:para) = 0
                and
                ancestor::db:annotation
                    [(@role=current()/@role
                     or
                     @role=current()/name())
                     and
                     @xreflabel=current()/.])
            and
            not(preceding-sibling::db:varname
                [.=current()/.]
                [@role=current()/@role])">
      <xsl:text>\Describe</xsl:text>
      <!-- Counter Function Length Template Variable Parameter -->
      <xsl:value-of
        select="translate(
                substring(@role,1,1),'cfltvp','CFLTVP')"/>
      <xsl:value-of select="substring(@role,2)"/>
      <xsl:text>{</xsl:text>
      <xsl:value-of select="normalize-space(.)"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:call-template name="avoidverb"/>
  </xsl:template>

  <xsl:template
    match="db:varname[ancestor::db:footnote
                      or
                      ancestor::db:abstract]
           |
           db:parameter[ancestor::db:footnote
                        or
                        ancestor::db:abstract]">
    <xsl:call-template name="compensate-space"/>
    <xsl:call-template name="avoidverb"/>
  </xsl:template>
  <xsl:template match="db:coref">
    <xsl:text>\setcounter{CPKcoref}{</xsl:text>
    <xsl:value-of
      select="count(//*[@xml:id=current()/@linkend]
              /descendant::*[name()=current()/@xreflabel])"/>
    <xsl:text>}\numberstring{CPKcoref}</xsl:text>
  </xsl:template>
  <xsl:template match="db:revision">
    <xsl:text>% \subsection*{v.</xsl:text>
    <xsl:value-of select="@version"/>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="db:date/@YYYY-MM-DD"/>
    <xsl:text>)}&#xa;</xsl:text>
    <xsl:apply-templates select="db:revdescription/*"/>
    <xsl:if test="//*[@revisionflag and
                     @revision=current()/db:date/@YYYY-MM-DD]">
      <xsl:text>% \begin{</xsl:text>
      <xsl:choose>
        <xsl:when
          test="count(//*[@revisionflag and
                @revision=current()/db:date/@YYYY-MM-DD])&gt;1">
          <xsl:text>enumerate</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>itemize</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>}[nosep]&#xa;</xsl:text>
      <xsl:for-each
        select="//*[@revisionflag and
                   @revision=current()/db:date/@YYYY-MM-DD]">
        <xsl:text>%   \item \vref{</xsl:text>
        <xsl:value-of select="@xml:id"/>
        <xsl:text>}</xsl:text>
        <xsl:value-of select="@revisionflag"/>
        <xsl:text>: </xsl:text>
        <xsl:value-of select="normalize-space(@annotations)"/>
        <xsl:text>&#xa;</xsl:text>
      </xsl:for-each>
      <xsl:text>% \end{</xsl:text>
      <xsl:choose>
        <xsl:when
          test="count(//*[@revisionflag
                          and
                          @revision=current()
                            /db:date/@YYYY-MM-DD])&gt;1">
          <xsl:text>enumerate</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>itemize</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template match="db:inlineequation">
    <xsl:apply-templates/>
    <xsl:if test="@xml:id">
      <xsl:text>\label{</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="db:mathphrase">
    <xsl:choose>
      <!-- assume @remap is LaTeX -->
      <xsl:when test="@remap">
        <xsl:text>\(</xsl:text>
        <xsl:value-of select="@remap"/>
        <xsl:text>\)</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="db:xref[@linkend]">
    <xsl:call-template name="compensate-space"/>
    <!-- remember where we started -->
    <xsl:variable name="context" select="."/>
    <!-- visit each target in turn -->
    <xsl:for-each select="tokenize(normalize-space(@linkend),' ')">
      <xsl:variable name="idref" select="."/>
      <xsl:variable name="target"
        select="$thisdoc//*[@xml:id=$idref]"/>
      <xsl:if test="position()>1">
        <xsl:if test="position()!=last()">
          <xsl:text>; </xsl:text>
        </xsl:if>
        <xsl:if test="position()=last()">
          <xsl:text> and </xsl:text>
        </xsl:if>
      </xsl:if>
      <!-- for normal references -->
      <xsl:if test="not($context/@role='short')">
        <xsl:choose>
          <!-- WHOLE DESCRIPTION LIST -->
          <xsl:when test="name($target)='variablelist'">
            <xsl:text>the list in </xsl:text>
          </xsl:when>
          <!-- DESCRIPTION LIST ENTRY-->
          <xsl:when test="name($target)='varlistentry'">
            <xsl:text>‘\textbf{\textsf{</xsl:text>
            <xsl:apply-templates
              select="$target/db:term/node()"/>
            <xsl:text>}}’</xsl:text>
            <!-- add ordinal if NOT a pageref -->
            <xsl:if test="not($context/@xrefstyle='page')">
              <xsl:text>, the </xsl:text>
              <xsl:variable name="nth"
                select="number(count($target/
                        preceding-sibling::db:varlistentry)+1)"/>
              <xsl:choose>
                <!-- 1st -->
                <xsl:when test="$nth = 1">
                  <xsl:text>first</xsl:text>
                </xsl:when>
                <!-- last -->
                <xsl:when
                  test="$nth = count($target/
                        parent::db:variablelist/db:varlistentry)">
                  <xsl:text>last</xsl:text>
                </xsl:when>
                <!-- digits over 12 -->
                <xsl:when test="$nth > 12">
                  <xsl:number value="$nth"
                    ordinal="yes" format="1"/>
                </xsl:when>
                <!-- words under/= 12 -->
                <xsl:otherwise>
                  <xsl:number value="$nth"
                    ordinal="yes" format="w"/>
                </xsl:otherwise>
              </xsl:choose>
              <xsl:text> item in </xsl:text>
              <xsl:choose>
                <!-- same list -->
                <xsl:when
                  test="generate-id($context
                        /ancestor::db:variablelist[1])
                        =
                        generate-id($target
                        /parent::db:variablelist)">
                  <xsl:text>this list</xsl:text>
                </xsl:when>
                <!-- different list -->
                <xsl:otherwise>
                  <xsl:text>the list in </xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:if>
            <!-- add afterwards, on p.nn or in sec ss-->
          </xsl:when><!-- END DESC LIST ENTRY -->
          <!-- UNNUMBERED LIST ITEMS NEED HELP -->
          <xsl:when
            test="name($target)='listitem' and
                  name($target/ancestor::*[1])='itemizedlist'">
            <xsl:text>the </xsl:text>
            <xsl:variable name="nth"
              select="number(count($target/
                      preceding-sibling::db:listitem)+1)"/>
            <xsl:choose>
              <xsl:when test="$nth=1">
                <xsl:text>first</xsl:text>
              </xsl:when>
              <xsl:when
                test="$nth=count($target/
                      ancestor::db:itemizedlist[1]/db:listitem)">
                <xsl:text>last</xsl:text>
              </xsl:when>
              <!-- from the fmtcount package -->
              <xsl:otherwise>
                <xsl:text>\ordinal</xsl:text>
                <xsl:if test="$nth &lt; 10">
                  <xsl:text>string</xsl:text>
                </xsl:if>
                <xsl:text>num{</xsl:text>
                <xsl:value-of select="$nth"/>
                <xsl:text>}</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
            <xsl:text> item in the list </xsl:text>
            <xsl:if test="not($context/@xrefstyle='page')">
              <xsl:text>in </xsl:text>
            </xsl:if>
          </xsl:when>
          <!-- ORDERED LIST ITEMS or PROCEDURE STEPS -->
          <xsl:when
            test="(local-name($target)='listitem' and
                  local-name($target/parent::*[1])='orderedlist')
                  or
                  (local-name($target)='step' and
                  local-name($target/parent::*[1])='procedure')">
            <xsl:choose>
              <xsl:when test="$target/ancestor::procedure">
                <xsl:text>step</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>item</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
            <xsl:text>~\ref{</xsl:text>
            <xsl:value-of select="$idref"/>
            <xsl:text>}</xsl:text>
            <xsl:choose>
              <!-- same list -->
              <xsl:when
                test="($target/ancestor::db:procedure and
                      generate-id($context
                              /ancestor::db:procedure[1])=
                      generate-id($target
                              /parent::db:procedure)
                      )
                      or
                      ($context/ancestor::db:orderedlist and
                      generate-id($context
                              /ancestor::db:orderedlist[1])=
                      generate-id($target
                              /parent::db:orderedlist))">
                <xsl:choose>
                  <xsl:when
                    test="count($context/preceding::*
                          |
                          $target)=count($context/preceding::*)">
                    <xsl:text> above</xsl:text>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:text> below</xsl:text>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <!-- elsewhere -->
              <xsl:otherwise>
                <xsl:text> in the list on </xsl:text>
                <xsl:text>p.\thinspace\pageref{</xsl:text>
                <xsl:value-of select="$idref"/>
                <xsl:text>}</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <!-- TITLED LISTS -->
          <xsl:when
            test="(local-name($target)='orderedlist'
                   or
                   local-name($target)='itemizedlist')
                  and $target/db:title">
            <xsl:text>the list `</xsl:text>
            <xsl:apply-templates
              select="$target/db:title/node()"/>
            <xsl:text>' in </xsl:text>
          </xsl:when>
          <!-- UNTITLED LISTS -->
          <xsl:when
            test="local-name($target)='orderedlist'
                  or
                  local-name($target)='itemizedlist'">
            <xsl:text>the list in </xsl:text>
          </xsl:when>
          <!-- INFORMAL TABLES -->
          <xsl:when
            test="local-name($target)='informaltable'">
            <xsl:text>the table </xsl:text>
          </xsl:when>
          <!-- PROGRAMLISTING/@condition="ignore"
               in package code -->
          <xsl:when
            test="local-name($target)='programlisting'">
            <xsl:text>the code </xsl:text>
          </xsl:when>
          <!-- NOTE, WARNING, SIDEBAR -->
          <xsl:when
            test="local-name($target)='note' or
                  local-name($target)='warning' or
                  local-name($target)='sidebar'">
            <xsl:text>the </xsl:text>
            <xsl:value-of select="local-name($target)"/>
            <xsl:text> in </xsl:text>
          </xsl:when>
          <!-- there is no otherwise -->
        </xsl:choose>
      </xsl:if>
      <!-- AUTOMATED or special case of ref to extractable file -->
      <xsl:choose>
        <xsl:when
          test="$target/ancestor::db:part[@xml:id='files']
                and
                $target/@xlink:href">
          <xsl:text>\url{</xsl:text>
          <xsl:value-of select="$target/@xlink:href"/>
          <xsl:text>}</xsl:text>
        </xsl:when>
        <xsl:when
          test="$context/@xrefstyle='page'
                or local-name($target)='informaltable'
                or local-name($target)='programlisting'">
          <xsl:text>\vpageref</xsl:text>
        </xsl:when>
        <!-- omit numbered list items done manually above -->
        <xsl:when
          test="(local-name($target)='listitem' and
                local-name($target/ancestor::*[1])='orderedlist')
                or
                ($context/ancestor::db:procedure and
                generate-id($context/ancestor::db:procedure)=
                generate-id($target/parent::db:procedure))">
          <xsl:text></xsl:text>
        </xsl:when>
        <!-- exclude link when varlistentries are
             in the same variablelist (so no \vref) -->
        <xsl:when
          test="$context/ancestor::db:variablelist
                and
                name($target)='varlistentry'
                and
                generate-id($context/ancestor::db:variablelist[1])
                =
                generate-id($target/parent::db:variablelist)">
          <xsl:text></xsl:text>
        </xsl:when>
        <!-- everything else except ranges, done below -->
        <xsl:when test="not($context/@endterm)">
          <xsl:text>\vref</xsl:text>
          <!-- defeat added space when preceding character
               was open-paren -->
          <xsl:if
            test="contains('([{&lt;',
                  substring($context/
                             preceding-sibling::node()[1],
                            string-length(
                            $context/
                             preceding-sibling::node()[1])))">
            <xsl:text>*</xsl:text>
          </xsl:if>
        </xsl:when>
        <xsl:when test="$context/@endterm">
          <xsl:text></xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>\ClassError{</xsl:text>
          <xsl:value-of select="$name"/>
          <xsl:text>}{xref attributes:</xsl:text>
          <xsl:for-each select="$context/@*">
            <xsl:text> </xsl:text>
            <xsl:value-of select="name()"/>
            <xsl:text>="</xsl:text>
            <xsl:value-of select="normalize-space(.)"/>
            <xsl:text>"</xsl:text>
          </xsl:for-each>
          <xsl:text>do not parse: contact author}</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
      <!-- now the argument[s]: not wanted for list items
           done manually or for filename refs to extractable
           files in part/files or when source and target in
           same variablelist -->
      <xsl:if
        test="not(local-name($target)='listitem' and
              local-name($target/ancestor::*[1])='orderedlist')
              and
              not($context/ancestor::db:procedure
              and
              $target/parent::db:procedure)
              and
              not($target/ancestor::db:part[@xml:id='files'] and
              $target/@xlink:href)
              and
              not(name($target)='varlistentry'
              and
              generate-id($context/ancestor::db:variablelist[1])
              =
              generate-id($target/parent::db:variablelist))">
        <xsl:if test="$context/@endterm">
          <xsl:text>\vrefrange</xsl:text>
        </xsl:if>
        <xsl:text>{</xsl:text>
        <xsl:value-of select="$idref"/>
        <xsl:text>}</xsl:text>
        <xsl:if test="$context/@endterm">
          <xsl:text>{</xsl:text>
          <xsl:value-of select="$context/@endterm"/>
          <xsl:text>}</xsl:text>
        </xsl:if>
      </xsl:if>
      <xsl:if test="$context/@xreflabel">
        <xsl:text> (</xsl:text>
        <xsl:apply-templates
          select="($target/child::*[local-name()=$context//
                  @xreflabel])[1]/node()"/>
        <xsl:text>)</xsl:text>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
  <xsl:template match="processing-instruction('LaTeX')">
    <xsl:value-of select="."/>
  </xsl:template>
  <xsl:template match="processing-instruction('cp')">
    <xsl:variable name="args"
      select="tokenize(normalize-space(.),' ')"/>
    <xsl:choose>
      <xsl:when test="$args[1]='include'">
        <xsl:apply-templates
          select="document($args[2])//*[@xml:id=$args[3]]"/>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  <!-- Former UCC bulk declarations now used for options
       Must have a para of explanation and the code in @annotations -->
  <xsl:template match="db:annotation[@annotations and @role='option']">
    <xsl:apply-templates select="db:para"/>
    <xsl:text>% \begin{CPKoption}{</xsl:text>
    <xsl:value-of select="@xreflabel"/>
    <xsl:text>}&#xa;%    </xsl:text>
    <xsl:text>\begin</xsl:text>
    <xsl:text>{macrocode}&#xa;\DeclareOption{</xsl:text>
    <xsl:value-of select="@xreflabel"/>
    <xsl:text>}{%&#xa;  </xsl:text>
    <xsl:value-of select="@annotations"/>
    <xsl:text>%&#xa;}&#xa;</xsl:text>
    <xsl:text>%    </xsl:text>
    <xsl:text>\end</xsl:text>
    <xsl:text>{macrocode}</xsl:text>
    <xsl:text>&#xa;% \end{CPKoption}&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="db:annotation/db:title">
    <xsl:text>% \subsection{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}&#xa;</xsl:text>
  </xsl:template>
  <xsl:template match="db:methodsynopsis" mode="declareoption">
    <!-- make a low-level title -->
    <xsl:text>% \paragraph{</xsl:text>
    <xsl:value-of
      select="normalize-space(db:methodparam[last()]
                              /db:parameter)"/>
    <!-- start a Code block for the option -->
    <xsl:text>}&#xa;% \begin{CPKoption}{</xsl:text>
    <xsl:value-of select="@xml:id"/>
    <xsl:text>}&#xa;% </xsl:text>
    <!-- now the descriptive text as a paragraph -->
    <!-- multiple parameters only for divisions, not degrees -->
    <!-- in document order, which is hierarchical top-down -->
    <xsl:for-each select="db:methodparam/db:parameter">
      <xsl:sort select="position()" order="descending"/>
      <xsl:if test="position()>1">
        <xsl:text> in </xsl:text>
      </xsl:if>
      <xsl:value-of select="normalize-space(.)"/>
      <xsl:if test="not(@role='degree')">
        <xsl:text> (</xsl:text>
        <xsl:value-of select="@role"/>
        <xsl:text>)</xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text>; </xsl:text>
    <xsl:if test="not(db:methodparam/db:parameter/@role='degree')">
      <xsl:text>&#xa;% using the </xsl:text>
      <!-- langs in methodsynopsis/@language for babel -->
      <xsl:value-of select="normalize-space(db:methodname)"/>
      <xsl:text> citation format&#xa;% from the \verb`</xsl:text>
      <xsl:value-of select="db:methodparam[1]/db:initializer"/>
      <xsl:text>.bst` \BibTeX\ style</xsl:text>
      <xsl:if test="db:methodparam[1]/db:modifier">
        <xsl:text> with the </xsl:text>
        <xsl:for-each select="db:methodparam[1]/db:modifier">
          <xsl:if test="preceding-sibling::db:modifier">
            <xsl:text> and </xsl:text>
          </xsl:if>
          <xsl:text>\textsf{</xsl:text>
          <xsl:value-of select="."/>
          <xsl:text>}</xsl:text>
        </xsl:for-each>
        <xsl:text> package</xsl:text>
        <xsl:if test="count(db:methodparam[1]/db:modifier)>1">
          <xsl:text>s</xsl:text>
        </xsl:if>
      </xsl:if>
      <xsl:choose>
        <xsl:when test="contains(@xlink:href,'@')">
          <xsl:text> (confirmed by \url{</xsl:text>
          <xsl:value-of select="@xlink:href"/>
          <xsl:text>})</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text> (unconfirmed)</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    <!-- followed by the actual code for the declared option -->
    <xsl:text>.&#xa;%    </xsl:text>
    <xsl:text>\begin</xsl:text>
    <xsl:text>{macrocode}&#xa;\DeclareOption{</xsl:text>
    <xsl:value-of select="@xml:id"/>
    <xsl:text>}{%&#xa;</xsl:text>
    <!-- order doesn't matter here,
         but the line length does (maxcodelen) -->
    <!-- so does duplication,
         because biblatex entries are replacements,
         not additions, but as \school etc use \gdef,
         there shouldn't be errors
         when they are duplicated -->
    <xsl:for-each select="db:methodparam/db:parameter">
      <xsl:text>  \</xsl:text>
      <xsl:value-of select="@role"/>
      <xsl:if test="@remap=''">
        <xsl:text>[]</xsl:text>
      </xsl:if>
      <xsl:text>{</xsl:text>
      <!-- break at parenthesis if present -->
      <xsl:choose>
        <xsl:when test="contains(.,'(')">
          <xsl:value-of
            select="normalize-space(substring-before(.,'('))"/>
          <xsl:text>&#xa;          (</xsl:text>
          <xsl:value-of
            select="normalize-space(substring-after(.,'('))"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="normalize-space(.)"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:for-each>
    <!-- use found for \@usebib 4th param -->
    <xsl:if test="db:methodparam[1]/db:modifier or
                  db:methodparam[1]/db:initializer">
      <xsl:text>  \@usebib</xsl:text>
      <!-- optional first param is package name[s] -->
      <xsl:if test="db:methodparam[1]/db:modifier">
        <xsl:text>[</xsl:text>
        <xsl:for-each select="db:methodparam[1]/db:modifier">
          <xsl:if test="position()>1">
            <xsl:text>,</xsl:text>
          </xsl:if>
          <xsl:value-of select="."/>
        </xsl:for-each>
        <xsl:text>]</xsl:text>
      </xsl:if>
      <!-- second param is bib[la]tex style name -->
      <xsl:text>{</xsl:text>
      <xsl:value-of select="db:methodparam[1]/db:initializer"/>
      <!-- third param is human-readable name of the style -->
      <xsl:text>}{</xsl:text>
      <xsl:value-of select="db:methodname"/>
      <!-- fourth param is experimental signal for biblatex-->
      <xsl:text>}{</xsl:text>
      <xsl:choose>
        <xsl:when test="db:methodparam/@role">
          <xsl:value-of select="db:methodparam/@role"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>\relax</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
    <!-- dept-specific languages one day
    <xsl:if test="@xml:lang">
      <xsl:text>  \use@babel[</xsl:text>
      <xsl:value-of
        select="translate(normalize-space(@xml:lang),' ',',')"/>
      <xsl:text>]{</xsl:text>
      <xsl:text>british</xsl:text>
      <xsl:text>}&#xa;</xsl:text>
      <xsl:variable name="langsused">
        <xsl:value-of select="normalize-space(@xml:lang)"/>
      </xsl:variable>
      <xsl:if test="@audience">
        <xsl:for-each select="tokenize(@audience,' ')">
          <xsl:variable name="curtok" select="position()"/>
          <xsl:text>  \def\</xsl:text>
          <xsl:value-of select="upper-case(.)"/>
          <xsl:text>{\foreignlanguage{</xsl:text>
          <xsl:value-of
            select="tokenize($langsused,' ')
                             [position()=$curtok]"/>
          <xsl:text>}}&#xa;</xsl:text>
        </xsl:for-each>
      </xsl:if>
    </xsl:if>
-->
    <xsl:text>}&#xa;</xsl:text>
    <xsl:text>%    </xsl:text>
    <xsl:text>\end</xsl:text>
    <xsl:text>{macrocode}
  </xsl:template>
  <xsl:template match="text()">
    <xsl:choose>
      <xsl:when
        test="ancestor::db:programlisting[@language='bash']">
        <xsl:choose>
          <!-- remove leading newlines from initial strings -->
          <xsl:when test="count(preceding-sibling::text())=0
                          and starts-with(.,'&#xa;')">
            <xsl:value-of select="substring(.,2)"/>
          </xsl:when>
          <!-- remove trailing space from terminals -->
          <xsl:when test="count(following-sibling::text())=0
                          and matches(.,'\n[\s]*$')">
            <xsl:value-of select="replace(.,'\n[\s]*$','')"/>
          </xsl:when>
          <!-- otherwise don't prefix or suffix -->
          <xsl:otherwise>
            <xsl:value-of select="."/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when
        test="ancestor::db:command or ancestor::db:biblioentry">
        <!-- replace a single leading WS token with a space -->
        <xsl:if test="starts-with(.,' ') or
                      starts-with(.,'&#x9;') or
                      starts-with(.,'&#xa;')">
          <xsl:text> </xsl:text>
        </xsl:if>
        <xsl:value-of select="normalize-space(.)"/>
        <!-- replace a single trailing WS token with a space -->
        <xsl:if test="substring(.,string-length(.))=' ' or
                      substring(.,string-length(.))='&#x9;' or
                      substring(.,string-length(.))='&#xa;'">
          <xsl:text> </xsl:text>
        </xsl:if>
      </xsl:when>
      <!-- otherwise check for percent prefixes -->
      <xsl:otherwise>
        <xsl:call-template name="dtxtext">
          <xsl:with-param name="text" select="."/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="*">
    <xsl:message>
      <xsl:text>Element default: </xsl:text>
      <xsl:for-each select="ancestor::*">
        <xsl:sort select="count(ancestor::*)"/>
        <xsl:text>/</xsl:text>
        <xsl:value-of select="local-name()"/>
      </xsl:for-each>
      <xsl:text>/</xsl:text>
      <xsl:value-of select="local-name()"/>
      <xsl:text> ("</xsl:text>
      <xsl:value-of select="."/>
      <xsl:text>")</xsl:text>
    </xsl:message>
    <xsl:apply-templates/>
  </xsl:template>
  <!-- Called from the document root template -->
  <xsl:template name="makepackages">
    <!-- Go through every PRE-step in prepost.xml for DOC part
         and see if any of them match known conditions for use.
         We are creating $packages here.
         The CONTEXT here is <step> for the purposes of detection,
         but we create <seglistitem>s in the result tree
         so that they can be processed on a par with any author
         requests for packages in the document -->
    <xsl:for-each
      select="$prepost/db:procedure
              [@xml:id='prepackage']
              /db:step[contains(@condition,'doc')]">
      <!-- context here is db:step[@condition='doc'] -->
      <xsl:variable name="element-match"
        select="$thisdoc/db:book/db:part[@xml:id='doc']/
                  descendant-or-self::*
                  [for $c in current()/db:constructorsynopsis
                       [@condition!='' and not(db:methodparam)]
                       return (.[name()=$c/@condition])]
                |
                $thisdoc/db:book/db:info/
                  (db:title|db:author|
                   db:abstract|db:annotation)/
                  descendant-or-self::*
                  [for $c in current()/db:constructorsynopsis
                       [@condition!='' and not(db:methodparam)]
                       return (.[name()=$c/@condition])]
                |
                $licence/descendant-or-self::*
                  [for $c in current()/db:constructorsynopsis
                       [@condition!='' and not(db:methodparam)]
                       return (.[name()=$c/@condition])]
                "/>
      <!-- context here is db:step[@condition='doc'] -->
      <xsl:variable name="attribute-match"
        select="$thisdoc/db:book/db:part[@xml:id='doc']/
                descendant-or-self::*
                [for $c in current()/db:constructorsynopsis
                     [@condition!='' and
                      db:methodparam[db:parameter and
                      not(db:modifier)]]
                 return (.[name()=$c/@condition and
                        @*[name()=$c/db:methodparam/db:parameter]])]
                |
                $thisdoc/db:book/db:info/
                  (db:title|db:author|
                   db:abstract|db:annotation)/
                  descendant-or-self::*
                [for $c in current()/db:constructorsynopsis
                     [@condition!='' and
                      db:methodparam[db:parameter and
                      not(db:modifier)]]
                 return (.[name()=$c/@condition and
                        @*[name()=$c/db:methodparam/db:parameter]])]
                |
                $licence/descendant-or-self::*
                [for $c in current()/db:constructorsynopsis
                     [@condition!='' and
                      db:methodparam[db:parameter and
                      not(db:modifier)]]
                 return (.[name()=$c/@condition and
                        @*[name()=$c/db:methodparam/db:parameter]])]
                "/>
      <!-- context here is db:step[@condition='doc'] -->
      <xsl:variable name="attribute-value-match"
        select="$thisdoc/db:book/db:part[@xml:id='doc']/
                 descendant-or-self::*
                 [for $c in current()/db:constructorsynopsis
                     [@condition!='' and
                      db:methodparam[db:parameter and
                      db:modifier]]
                  return (.[name()=$c/@condition and
                          @*[name()=$c/db:methodparam/db:parameter]=
                            $c/db:methodparam/db:modifier])]
                |
                $thisdoc/db:book/db:info/
                  (db:title|db:author|
                   db:abstract|db:annotation)/
                  descendant-or-self::*
                 [for $c in current()/db:constructorsynopsis
                     [@condition!='' and
                      db:methodparam[db:parameter and
                      db:modifier]]
                  return (.[name()=$c/@condition and
                          @*[name()=$c/db:methodparam/db:parameter]=
                            $c/db:methodparam/db:modifier])]
                |
                $licence/descendant-or-self::*
                 [for $c in current()/db:constructorsynopsis
                     [@condition!='' and
                      db:methodparam[db:parameter and
                      db:modifier]]
                  return (.[name()=$c/@condition and
                          @*[name()=$c/db:methodparam/db:parameter]=
                            $c/db:methodparam/db:modifier])]
                |
                $thisdoc/db:book
                [for $c in current()/db:constructorsynopsis
                     [@condition!='' and
                      db:methodparam[db:parameter and
                      db:modifier]]
                  return (.[name()=$c/@condition and
                          @*[name()=$c/db:methodparam/db:parameter]=
                            $c/db:methodparam/db:modifier])]
                "/>
      <!-- added db:book alone to handle matches
           to (eg) status="draft" -->
      <!-- context here is db:step[@condition='doc'] -->
      <xsl:variable name="attribute-value-pointer"
        select="$thisdoc/db:book/db:part[@xml:id='doc']
                  /descendant-or-self::*
                  [name()=current()/db:constructorsynopsis/@condition]
                  [for $m in
                       current()/db:constructorsynopsis/db:methodparam,
                       $e in tokenize(@*[name()=$m/db:funcparams])
                   return (id($e)[name()=$m/db:modifier])]
                |
                $thisdoc/db:book/db:info/
                  (db:title|db:author|db:abstract|db:annotation)/
                  descendant-or-self::*[name()=
                    current()/db:constructorsynopsis/@condition]
                  [for $m in
                       current()/db:constructorsynopsis/db:methodparam,
                       $e in tokenize(@*[name()=$m/db:funcparams])
                   return (id($e)[name()=$m/db:modifier])]
                |
                $licence/descendant-or-self::*
                  [name()=current()/db:constructorsynopsis/@condition]
                  [for $m in
                       current()/db:constructorsynopsis/db:methodparam,
                       $e in tokenize(@*[name()=$m/db:funcparams])
                   return (id($e)[name()=$m/db:modifier])]
                "/>
      <!-- context here is db:step[@condition='doc'] -->
      <xsl:variable name="defaults"
        select="$thisdoc/db:book/db:part[@xml:id='doc']
                [current()/db:constructorsynopsis/@condition='']
                "/>
      <!-- context here is db:step[@condition='doc'] -->
      <!-- A dependency is signalled in prepost.xml when a package
           has a constructorsynopsis/@role AND a
           constructorsynopsis/methodname, meaning that IF a
           package in <methodname> (space-separated) is requested,
           OR a package in <methodname> is one that is also
           required BY DEFAULT, then the current package will
           become required, with the @role option (if non-null).
           So the package dependency MAY be signalled by using a
           null @role, meaning that no specific option is required,
           just the package.
       -->
      <xsl:variable name="dependencies"
        select="$thisdoc//db:constraintdef[@xml:id='docpackages']//
                db:seglistitem
                [db:seg=tokenize(normalize-space(
                 current()/db:constructorsynopsis[@role]
                 /db:methodname),' ')]
                |
                current()[
                $prepost/db:procedure[@xml:id='prepackage']/db:step
                [db:constructorsynopsis/@condition='']
                [@remap=tokenize(normalize-space(
                 current()/db:constructorsynopsis[@role]
                 /db:methodname),' ')]]
                 "/>
      <xsl:variable name="elements"
        select="
                $element-match |
                $attribute-match |
                $attribute-value-match |
                $attribute-value-pointer |
                $defaults |
                $dependencies
                "/>
      <!-- REMINDER: context is still <step> in prepost,
           but the "elements" variable now contains every
           element FROM THE DOCUMENT which meets the conditions
           needed to justify the current package's inclusion.
           Process this ONLY IFF any elements at all were selected:
           this is where we create the result tree -->
      <xsl:if test="count($elements)>0">
        <!-- IDENTIFY THE CURRENT PACKAGE AND WHY [DE-]SELECTED -->
        <xsl:variable name="prepostno">
          <!-- this represents the number of the procedure in
               prepost.xml (1=pre, 2=post) followed by the
               (3-digit decimal) sequence of the position of
               the package (the seglistitem element type),
               so for example, 'dox' is 1.001 and fontawesome
               is 1.127 -->
          <xsl:value-of
            select="count(parent::db:procedure/
                    preceding-sibling::db:procedure) + 1"/>
          <xsl:text>.</xsl:text>
          <xsl:value-of
            select="format-integer(
                    count(preceding-sibling::db:step)+1,
                    '999')"/>
        </xsl:variable>
        <!-- still in context <step> for the moment -->
        <xsl:variable name="thispackage" select="@remap"/>
        <!-- 1. ELEMENTS -->
        <xsl:if test="count($element-match)>0">
          <xsl:message>
            <xsl:value-of select="@remap"/>
            <xsl:text> (</xsl:text>
            <xsl:value-of select="$prepostno"/>
            <xsl:text>)</xsl:text>
            <xsl:text> element-match=</xsl:text>
            <xsl:value-of select="count($element-match)"/>
            <xsl:text> </xsl:text>
            <!-- element-match just needs element type name[s] -->
            <xsl:for-each-group select="$element-match"
              group-by="name()">
              <xsl:if test="position()>1">
                <xsl:text>,</xsl:text>
              </xsl:if>
              <xsl:value-of select="current-grouping-key()"/>
            </xsl:for-each-group>
          </xsl:message>
          <xsl:call-template name="makeseg">
            <xsl:with-param name="arch" select="'elements'"/>
            <xsl:with-param name="prepostno" select="$prepostno"/>
          </xsl:call-template>
        </xsl:if>
        <!-- 2. ATTRIBUTES -->
        <xsl:if test="count($attribute-match)>0">
          <xsl:message>
            <xsl:value-of select="@remap"/>
            <xsl:text> (</xsl:text>
            <xsl:value-of select="$prepostno"/>
            <xsl:text>)</xsl:text>
            <xsl:text> attribute-match=</xsl:text>
            <xsl:value-of select="count($attribute-match)"/>
            <xsl:text> </xsl:text>
            <!-- attribute-match needs element/@attribute -->
            <xsl:for-each-group select="$attribute-match"
              group-by="name()">
              <xsl:if test="position()>1">
                <xsl:text>,</xsl:text>
              </xsl:if>
              <xsl:value-of select="current-grouping-key()"/>
              <xsl:text>/</xsl:text>
              <!-- variable contains entries which are the
                   whole elements within which an attribute
                   match was found, so iterate over the
                   attributes as groups and only print the
                   one which matches -->
              <xsl:variable name="element-key"
                select="current-grouping-key()"/>
              <xsl:variable name="attribs">
                <xsl:for-each select="current-group()">
                  <xsl:for-each select="@*">
                    <att>
                      <xsl:value-of select="name()"/>
                    </att>
                  </xsl:for-each>
                </xsl:for-each>
              </xsl:variable>
              <xsl:for-each-group select="$attribs/*"
                                  group-by="name()">
                <xsl:if
                  test=".=
                        $prepost/db:procedure/db:step
                          [@remap=$thispackage]
                        /db:constructorsynopsis
                          [@condition=$element-key]
                        /db:methodparam/db:parameter">
                  <xsl:text>@</xsl:text>
                  <xsl:value-of
                    select="$prepost/db:procedure/db:step
                              [@remap=$thispackage]
                            /db:constructorsynopsis
                              [@condition=$element-key]
                            /db:methodparam/db:parameter"/>
                </xsl:if>
              </xsl:for-each-group>
            </xsl:for-each-group>
          </xsl:message>
          <xsl:call-template name="makeseg">
            <xsl:with-param name="arch" select="'attributes'"/>
            <xsl:with-param name="prepostno" select="$prepostno"/>
          </xsl:call-template>
        </xsl:if>
        <!-- 3. ATTRIBUTE VALUES -->
        <xsl:if test="count($attribute-value-match)>0">
          <xsl:message>
            <xsl:value-of select="@remap"/>
            <xsl:text> (</xsl:text>
            <xsl:value-of select="$prepostno"/>
            <xsl:text>)</xsl:text>
            <xsl:text> attribute-value match=</xsl:text>
            <xsl:value-of select="count($attribute-value-match)"/>
            <xsl:text> </xsl:text>
            <xsl:for-each-group select="$attribute-value-match"
              group-by="name()">
              <xsl:if test="position()>1">
                <xsl:text>,</xsl:text>
              </xsl:if>
              <xsl:value-of select="current-grouping-key()"/>
              <xsl:text>/</xsl:text>
              <xsl:for-each select="current-group()">
                <xsl:variable name="matchgi" select="."/>
                <xsl:for-each
                  select="$prepost/db:procedure/db:step
                            [@remap=$thispackage]
                          /db:constructorsynopsis
                            [@condition=current-grouping-key()]
                          /db:methodparam">
                  <xsl:if
                    test="$matchgi/@*
                          [name()=current()/db:parameter]
                            =current()/db:modifier">
                    <xsl:text>@</xsl:text>
                    <xsl:value-of select="db:parameter"/>
                    <xsl:text>=</xsl:text>
                    <xsl:value-of select="db:modifier"/>
                    <xsl:text>:</xsl:text>
                  </xsl:if>
                </xsl:for-each>
              </xsl:for-each>
            </xsl:for-each-group>
          </xsl:message>
          <xsl:call-template name="makeseg">
            <xsl:with-param name="arch" select="'attvals'"/>
            <xsl:with-param name="prepostno" select="$prepostno"/>
          </xsl:call-template>
        </xsl:if>
        <!-- 4. ATTRIBUTE VALUE POINTERS -->
        <xsl:if test="count($attribute-value-pointer)>0">
          <xsl:message>
            <xsl:value-of select="@remap"/>
            <xsl:text> (</xsl:text>
            <xsl:value-of select="$prepostno"/>
            <xsl:text>)</xsl:text>
            <xsl:text> attribute-value-pointer (</xsl:text>
            <xsl:value-of select="count($attribute-value-pointer)"/>
            <xsl:text>) </xsl:text>
            <xsl:for-each-group select="$attribute-value-pointer"
              group-by="name()">
              <xsl:if test="position()>1">
                <xsl:text>,</xsl:text>
              </xsl:if>
              <xsl:value-of select="current-grouping-key()"/>
              <xsl:text>/</xsl:text>
              <xsl:variable name="elementname"
                select="current-grouping-key()"/>
              <xsl:for-each
                select="$prepost/db:procedure/db:step
                          [@remap=$thispackage]
                        /db:constructorsynopsis
                          [@condition=current-grouping-key()]
                        /db:methodparam">
                <!-- any element in the document with an xml:id
                     pointed TO by one of the current group
                     which element is of the type specified -->
                <xsl:if
                  test="count(current-group()/@*
                        [name()=current()/db:funcparams]
                        [id(.)/name()=current()/db:modifier])>0">
                  <xsl:text>@</xsl:text>
                  <xsl:value-of select="db:funcparams"/>
                  <xsl:text>→</xsl:text>
                  <xsl:value-of select="db:modifier"/>
                  <xsl:text>; </xsl:text>
                </xsl:if>
              </xsl:for-each>
            </xsl:for-each-group>
          </xsl:message>
          <xsl:call-template name="makeseg">
            <xsl:with-param name="arch" select="'attvalidrefs'"/>
            <xsl:with-param name="prepostno" select="$prepostno"/>
          </xsl:call-template>
        </xsl:if>
        <!-- 5. DEPENDENCIES match seglistitems from user requests
                or from other packages included as defaults -->
        <xsl:if test="count($dependencies)>0">
          <xsl:message>
            <xsl:value-of select="@remap"/>
            <xsl:text> (</xsl:text>
            <xsl:value-of select="$prepostno"/>
            <xsl:text>)</xsl:text>
            <xsl:text> dependency match </xsl:text>
            <xsl:for-each-group select="$dependencies"
              group-by="name()">
              <xsl:if test="position()>1">
                <xsl:text>,</xsl:text>
              </xsl:if>
              <xsl:value-of
                select="db:constructorsynopsis/db:methodname"/>
            </xsl:for-each-group>
          </xsl:message>
          <xsl:call-template name="makeseg">
            <xsl:with-param name="arch" select="'dependencies'"/>
            <xsl:with-param name="prepostno" select="$prepostno"/>
          </xsl:call-template>
        </xsl:if>
        <!-- 6. DEFAULT -->
        <xsl:if test="count($defaults)>0">
          <xsl:message>
            <xsl:value-of select="@remap"/>
            <xsl:text> (</xsl:text>
            <xsl:value-of select="$prepostno"/>
            <xsl:text>)</xsl:text>
            <xsl:text> default </xsl:text>
            <xsl:if test="$prepost/db:procedure[@xml:id='prepackage']
                          /db:step[db:constructorsynopsis
                          [contains(@remap,current()/@remap)]]">
              <xsl:text>will be omitted because of </xsl:text>
              <xsl:value-of
                select="$prepost/db:procedure[@xml:id='prepackage']
                          /db:step[db:constructorsynopsis
                        [contains(@remap,current()/@remap)]]/@remap"/>
            </xsl:if>
          </xsl:message>
          <xsl:call-template name="makeseg">
            <xsl:with-param name="arch" select="'defaults'"/>
            <xsl:with-param name="prepostno" select="$prepostno"/>
          </xsl:call-template>
        </xsl:if>
      </xsl:if>
    </xsl:for-each>
    <!-- 7. FINALLY, ADD PACKAGES REQUESTED BY THE AUTHOR
         in the document itself for use in the documentation.
         The list is accessed via the 'doc' pre packages,
         matching those mentioned in the author's document.
         This ensures that the context is <step> as earlier.
         The $authorloc variable points at the matching entry
         in the author's document so that options from BOTH
         $prepost AND the author's document can be merged. -->
    <xsl:for-each
      select="$prepost/
              db:procedure[@xml:id='prepackage']/
              db:step[contains(@condition,'doc')]
              [@remap=
                $thisdoc/db:book/db:info/db:cover
                /db:constraintdef[@xml:id='docpackages']
                /db:segmentedlist/db:seglistitem
                [not(db:seg/@condition='off')]/db:seg]">
      <xsl:variable name="prepostno">
        <xsl:value-of
          select="count(parent::db:procedure/
                  preceding-sibling::db:procedure) + 1"/>
        <xsl:text>.</xsl:text>
        <xsl:value-of
          select="format-integer(
                  count(preceding-sibling::db:step)+1,
                  '999')"/>
      </xsl:variable>
      <xsl:variable name="authorloc"
        select="$thisdoc/db:book/db:info/db:cover
                /db:constraintdef[@xml:id='docpackages']
                /db:segmentedlist/db:seglistitem
                [db:seg=current()/@remap]"/>
      <!-- These WERE flagged by making $prepostno 999 +
           the author's <seglistitem> but now with ordering
           in prepost.xml, this has reverted to 1 + -->
      <xsl:call-template name="makeseg">
        <xsl:with-param name="arch" select="'requests'"/>
        <!--
        <xsl:with-param name="prepostno">
          <xsl:text>999.</xsl:text>
          <xsl:value-of
            select="format-integer(
                    count($authorloc/preceding-sibling::db:seglistitem)
                      +1,'999')"/>
        </xsl:with-param>
        -->
        <xsl:with-param name="prepostno" select="$prepostno"/>
        <xsl:with-param name="authorloc" select="$authorloc"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>
  <!-- end of makepackages -->
  <!-- called from within makepackages
       with context db:step from prepost.xml -->
  <xsl:template name="makeseg">
    <xsl:param name="arch"/>
    <xsl:param name="prepostno"/>
    <xsl:param name="authorloc"/>
    <!-- $authorloc is the <seglistitem> from the author's document
         where this package is being requested (requests only)
         OR from prepost.xml entry where the methodname contained
         a required package (doc only) -->
    <db:seglistitem arch="{$arch}" userlevel="{$prepostno}">
      <!-- add @condition to reflect an attribute match
           OR a package match for a dependency -->
      <xsl:choose>
        <xsl:when test="($arch='attributes' or $arch='attvals')
                        and db:constructorsynopsis/@condition">
          <xsl:attribute name="condition"
            select="db:constructorsynopsis/@condition"/>
        </xsl:when>
        <xsl:when test="$arch='dependencies'">
          <xsl:attribute name="condition"
            select="current()/db:constructorsynopsis[@role]
                    /db:methodname"/>
        </xsl:when>
      </xsl:choose>
      <!-- SOMETHING WRONG HERE: role being included when not
           needed eg hyphens on url -->
      <!-- specify dependency option as @role only when non-null
           (a null @role may have been used to signal a dependency
           but without the need for any specific option)... -->
      <xsl:if
        test="db:constructorsynopsis/@role
              and db:constructorsynopsis/@role!=''
              and $arch='dependencies'
              or (db:constructorsynopsis/db:methodname
              and db:constructorsynopsis/db:methodname=$authorloc)">
        <xsl:attribute name="role"
          select="db:constructorsynopsis/@role"/>
      </xsl:if>
      <!-- ...so when setting a dependency as a result of another
           package, the methodname must really have been used,
           so we check the author location -->
      <xsl:if
        test="db:constructorsynopsis/@role
              and $arch='dependencies'
              and db:constructorsynopsis/db:methodname
              and db:constructorsynopsis/db:methodname=$authorloc">
        <xsl:attribute name="audience"
          select="db:constructorsynopsis/db:methodname"/>
      </xsl:if>
      <xsl:if test="db:constructorsynopsis/@remap">
        <xsl:attribute name="remap"
          select="db:constructorsynopsis/@remap"/>
      </xsl:if>
      <!-- only add @conformance when it's an attribute
           or attval match -->
      <xsl:if
        test="($arch='attributes' or $arch='attvals')
               and
               db:constructorsynopsis/db:methodparam/db:parameter">
        <xsl:attribute name="conformance"
          select="db:constructorsynopsis/db:methodparam
                  /db:parameter"/>
      </xsl:if>
      <!-- @annotations in <seglistitem> is DIFFERENT to
           @annotations in <seg> -->
      <xsl:if
        test="db:constructorsynopsis/db:methodparam/db:modifier">
        <xsl:attribute name="annotations"
          select="db:constructorsynopsis/db:methodparam
                  /db:modifier"/>
      </xsl:if>
      <xsl:if
        test="db:constructorsynopsis/db:methodparam/db:funcparams">
        <xsl:attribute name="xreflabel"
          select="db:constructorsynopsis/db:methodparam
          /db:funcparams"/>
      </xsl:if>
      <db:seg>
        <xsl:if test="(@performance and $arch='dependencies') or
                       @performance='required'">
          <xsl:attribute name="annotations" select="@performance"/>
        </xsl:if>
        <!-- $authorloc is missing EXCEPT when doing author-specified
             packages so here we create an @role attribute (options)
             if the @role is present (whether or not $authorloc is
             present) but we ADD any author-supplied options if
             $authorloc IS pesent (that is, if the author requested
             any options for a package that is already in $prepost
             with default options) -->
        <xsl:if test="$authorloc">
          <xsl:attribute name="security">
            <xsl:value-of select="normalize-space($authorloc)"/>
          </xsl:attribute>
        </xsl:if>
        <xsl:choose>
          <!-- @role only: use it -->
          <xsl:when test="@role and not($authorloc)">
            <xsl:attribute name="role">
              <xsl:value-of select="@role"/>
            </xsl:attribute>
          </xsl:when>
          <!-- no @role but $authorloc has a @role of its own -->
          <xsl:when test="$authorloc and
                          not(@role) and
                          $authorloc/db:seg/@role">
            <xsl:attribute name="role">
              <xsl:value-of select="$authorloc/db:seg/@role"/>
            </xsl:attribute>
          </xsl:when>
          <!-- @role and $authorloc and $authorloc has a
               @role of its own -->
          <xsl:when test="@role and
                          $authorloc and
                          $authorloc/db:seg/@role">
            <xsl:attribute name="role">
              <xsl:value-of select="@role | $authorloc/db:seg/@role"/>
            </xsl:attribute>
          </xsl:when>
          <!-- neither @role nor $authorloc means no @role needed
          <xsl:otherwise>
            <xsl:attribute name="security">
              <xsl:if test="@role">
                <xsl:value-of select="@role"/>
                <xsl:text> role</xsl:text>
              </xsl:if>
              <xsl:if test="$authorloc and $authorloc/db:seg/@role!=''">
                <xsl:value-of select="$authorloc/db:seg/@role"/>
                <xsl:text> authorloc</xsl:text>
              </xsl:if>
            </xsl:attribute>
          </xsl:otherwise>
          -->
        </xsl:choose>
        <xsl:if test="@revision">
          <xsl:attribute name="revision" select="@revision"/>
        </xsl:if>
        <xsl:if test="@conformance">
          <xsl:attribute name="conformance" select="@conformance"/>
        </xsl:if>
        <xsl:if test="@condition">
          <xsl:attribute name="condition" select="@condition"/>
        </xsl:if>
        <xsl:value-of select="@remap"/>
      </db:seg>
    </db:seglistitem>
  </xsl:template>
  <!-- context is still programlisting -->
  <xsl:template name="makelisting">
    <xsl:variable name="fileext"
      select="tokenize(@xlink:href,'\.')[position()=last()]"/>
    <xsl:choose>
      <xsl:when test="@xlink:href">
        <xsl:text>% \lstinputlisting[</xsl:text>
        <xsl:text>language=</xsl:text>
        <xsl:choose>
          <xsl:when test="@language='LaTeXe'
                          or @language='LaTeX'
                          or not(@language)">
            <xsl:text>{LaTeXe}</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="@language"/>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="@startinglinenumber or @endinglinenumber">
          <xsl:if
            test="(not(number(@startinglinenumber))
                  or
                  not(number(@endinglinenumber)))
                  and
                  not(@conformance='frag')">
            <xsl:choose>
              <xsl:when test="$fileext='tex'
                              or
                              $fileext='bib'">
                <xsl:text>,rangeprefix=\%\%\ </xsl:text>
                <xsl:text>,rangesuffix=\ \%\%</xsl:text>
              </xsl:when>
              <xsl:when test="$fileext='xml'
                              or
                              $fileext='xsl'
                              or
                              $fileext='xmap'">
                <xsl:text>,rangeprefix={\&lt;?cpdoc\ }</xsl:text>
                <xsl:text>,rangesuffix={?\>}</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>,rangeprefix=\#\#\ </xsl:text>
                <xsl:text>,rangesuffix=\ \#\#</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:if>
          <xsl:if test="not(@conformance='frag')">
            <!-- from -->
            <xsl:choose>
              <xsl:when test="@startinglinenumber">
                <xsl:choose>
                  <xsl:when
                    test="number(@startinglinenumber)">
                    <xsl:text>,firstline=</xsl:text>
                    <xsl:value-of
                      select="@startinglinenumber"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:text>,linerange=</xsl:text>
                    <xsl:value-of
                      select="@startinglinenumber"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>,firstline=1</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
            <!-- to -->
            <xsl:choose>
              <xsl:when test="@endinglinenumber">
                <xsl:choose>
                  <xsl:when
                    test="number(@endinglinenumber)
                          or
                          number(@startinglinenumber)">
                    <xsl:text>,lastline=</xsl:text>
                    <xsl:value-of
                      select="@endinglinenumber"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:text>-</xsl:text>
                    <xsl:value-of
                      select="@endinglinenumber"/>
                    <xsl:text>,includerangemarker=false</xsl:text>
                    <xsl:text>,showlines=false</xsl:text>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text></xsl:text>
              </xsl:otherwise>
            </xsl:choose>
            <!-- both -->
            <xsl:text>,numbers=left</xsl:text>
            <xsl:text>,numberstyle=\ttfamily</xsl:text>
            <xsl:text>\scriptsize\color{Blue}</xsl:text>
            <xsl:variable name="thisapp"
              select="(ancestor::db:chapter
                      |
                      ancestor::db:appendix)/@xml:id"/>
            <xsl:variable name="pth"
              select="count(preceding::db:programlisting
                      [@xlink:href=current()/@xlink:href]
                      [(ancestor::db:chapter|ancestor::db:appendix)
                        /@xml:id=$thisapp])+1"/>
            <xsl:variable name="proglistgroup">
              <xsl:value-of select="$thisapp"/>
              <xsl:text>-</xsl:text>
              <xsl:value-of select="$pth"/>
            </xsl:variable>
            <xsl:choose>
              <!-- if this is the first of more than one -->
              <xsl:when
                test="$pth=1
                      and
                      count(following::db:programlisting
                      [@xlink:href=current()/@xlink:href]
                      [(ancestor::db:chapter|ancestor::db:appendix)
                        /@xml:id=$thisapp])>0">
                <xsl:text>,name=</xsl:text>
                <xsl:value-of select="$proglistgroup"/>
                <xsl:text>,firstnumber=1</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>,name=</xsl:text>
                <xsl:value-of select="$proglistgroup"/>
                <xsl:text>,firstnumber=last</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:if>
        </xsl:if>
      </xsl:when>
        <!-- still waiting for the closing square bracket -->
      <xsl:when test="contains(.,'\begin{lstlisting}')">
        <xsl:text>% \iffalse
        <xsl:text>\begin{listingsdoc}</xsl:text>
      </xsl:when>
      <!-- otherwise do a begin and end -->
      <xsl:otherwise>
        <xsl:text>% \iffalse
        <xsl:text>\begin{lstlisting}[</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="not(@xlink:href)">
      <xsl:text>language=</xsl:text>
      <xsl:choose>
        <xsl:when test="@language='LaTeXe'
                        or @language='LaTeX'
                        or not(@language)">
          <xsl:text>{[LaTeX]TeX}</xsl:text>
        </xsl:when>
        <xsl:when test="@language='DocBook'">
          <xsl:text>DocBook</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@language"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    <xsl:if test="not(contains(.,'\begin{lstlisting}'))">
      <!-- font size -->
      <xsl:if test="@wordsize">
        <xsl:text>,breaklines=true</xsl:text>
        <xsl:text>,basicstyle=</xsl:text>
        <xsl:if test="@wordsize!=''">
          <xsl:choose>
            <xsl:when test="contains(@wordsize,'/')">
              <xsl:text>\fontsize{</xsl:text>
              <xsl:value-of
                select="substring-before(@wordsize,'/')"/>
              <xsl:text>}{</xsl:text>
              <xsl:value-of
                select="substring-after(@wordsize,'/')"/>
              <xsl:text>}\selectfont</xsl:text>
            </xsl:when>
            <xsl:when test="number(@wordsize)">
              <xsl:text>\fontsize{</xsl:text>
              <xsl:value-of select="@wordsize"/>
              <xsl:text>}{</xsl:text>
              <xsl:value-of
                select="number(@wordsize) * 1.2"/>
              <xsl:text>}\selectfont</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="@wordsize"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
        <xsl:text>\color{Black}\ttfamily,</xsl:text>
      </xsl:if>
      <!-- default is unframed -->
      <xsl:if test="@arch='framed'">
        <xsl:text>,frame=single,framesep=1em</xsl:text>
      </xsl:if>
      <!-- remap changes style of "emphasis" keywords -->
      <xsl:if test="@remap">
        <xsl:text>,emphstyle=</xsl:text>
        <xsl:value-of select="@remap"/>
      </xsl:if>
      <!-- the emphasis keywords themselves -->
      <xsl:if test="@annotations or db:userinput">
        <xsl:text>,emph={</xsl:text>
        <xsl:value-of select="@annotations"/>
        <xsl:if test="db:userinput">
          <xsl:for-each select="db:userinput">
            <xsl:if test="not(preceding-sibling::db:userinput
                              [.=current()/.])">
              <xsl:if test="(position()=1 and ../@annotations)
                            or
                            position()&gt;1">
                <xsl:text>,</xsl:text>
              </xsl:if>
              <xsl:value-of select="."/>
            </xsl:if>
          </xsl:for-each>
        </xsl:if>
        <xsl:text>}</xsl:text>
      </xsl:if>
      <!-- numbering -->
      <xsl:if test="@linenumbering='numbered'">
        <xsl:text>,numbers=left</xsl:text>
        <xsl:text>,numberstyle=\sffamily\footnotesize</xsl:text>
        <xsl:text>,numbersep=2em</xsl:text>
      </xsl:if>
      <!-- label -->
      <xsl:if test="@xml:id">
        <xsl:text>,label=</xsl:text>
        <xsl:value-of select="@xml:id"/>
      </xsl:if>
      <xsl:if test="@linenumbering='numbered' or @arch='framed'">
        <xsl:if test="@arch='framed'">
          <xsl:text>,xrightmargin=1em</xsl:text>
        </xsl:if>
        <xsl:choose>
          <xsl:when
            test="@linenumbering='numbered' and @arch='framed'">
            <xsl:text>,xleftmargin=3.5em</xsl:text>
          </xsl:when>
          <xsl:when test="@linenumbering='numbered'">
            <xsl:text>,xleftmargin=2.5em</xsl:text>
          </xsl:when>
          <xsl:when test="@arch='framed'">
            <xsl:text>,xleftmargin=1em</xsl:text>
          </xsl:when>
        </xsl:choose>
      </xsl:if>
      <xsl:text>]</xsl:text>
    </xsl:if>
    <xsl:choose>
      <!-- for an included file, here’s the filename -->
      <xsl:when test="@xlink:href">
        <xsl:text>{</xsl:text>
        <xsl:choose>
          <xsl:when test="@conformance='frag'">
            <xsl:variable name="fn">
              <xsl:choose>
                <xsl:when test="contains(@xlink:href,'/')">
                  <xsl:value-of
                    select="tokenize(@xlink:href,'/')
                            [position()=last()]"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="@xlink:href"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <!-- add directory prefix if needed -->
            <xsl:if test="contains(@xlink:href,'/')">
              <xsl:for-each
                select="tokenize(@xlink:href,'/')
                        [not(position()=last())]">
                <xsl:value-of select="."/>
                <xsl:text>/</xsl:text>
              </xsl:for-each>
            </xsl:if>
            <xsl:value-of
              select="substring-before($fn,'.')"/>
            <xsl:text>-</xsl:text>
            <xsl:value-of select="@startinglinenumber"/>
            <!--
            <xsl:text>-</xsl:text>
            <xsl:value-of select="@endinglinenumber"/>
            -->
            <xsl:text>.</xsl:text>
            <xsl:value-of select="@conformance"/>
            <!--
            <xsl:value-of select="$fileext"/>
            -->
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="@xlink:href"/>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>}&#xa;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>&#xa;</xsl:text>
        <!-- output the content -->
        <xsl:call-template name="lrtrim">
          <xsl:with-param name="text" select="."/>
          <xsl:with-param name="indent">
            <xsl:choose>
              <xsl:when
                test="contains(.,'\end{lstlisting}') or
                      contains(.,'\end{verbatim}') or
                      contains(.,'\end{Verbatim}')">
                <xsl:text>  </xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text></xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:with-param>
        </xsl:call-template>
        <xsl:choose>
          <xsl:when
            test="contains(.,'\begin{lstlisting}')">
            <xsl:text>&#xa;\end{listingsdoc}&#xa;</xsl:text>
        <xsl:text>% \iffalse
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>&#xa;\end{lstlisting}&#xa;</xsl:text>
        <xsl:text>% \iffalse
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="@conformance='frag'">
      <xsl:text>% \label{</xsl:text>
      <xsl:if test="number(@startinglinenumber)">
        <xsl:text>frag-</xsl:text>
      </xsl:if>
      <xsl:value-of select="@startinglinenumber"/>
      <xsl:text>}&#xa;</xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template name="casestyle">
    <!-- disabled temporarily: just emits \textsc{...} -->
    <xsl:param name="text"/>
    <xsl:variable name="nosc">
      <xsl:for-each
        select="$prepost//db:step[@conformance='nosc']">
        <xsl:value-of
          select="$thisdoc//db:constraintdef
                  [@xml:id='docpackages']
                  //db:seg[.=current()/@remap]"/>
      </xsl:for-each>
    </xsl:variable>
    <!--
    <xsl:choose>
      <xsl:when test="count($nosc)>0">
        <xsl:text>\SMC{</xsl:text>
        <xsl:value-of select="lower-case($text)"/>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        -->
        <xsl:text>\textsc{</xsl:text>
        <xsl:value-of select="lower-case($text)"/>
        <xsl:text>}</xsl:text>
        <!--
      </xsl:otherwise>
    </xsl:choose>
    -->
  </xsl:template>
  <!-- CURRENTLY NON-OPERATIONAL -->
  <xsl:template name="nondeferredpackages">
    <!-- if there are any authorial non-null <seg> requests -->
    <xsl:if
      test="/db:book/db:info/db:cover/db:constraintdef
            [@xml:id=concat($filetype,'packages')]
            [not(@linkend)]
            /db:segmentedlist/db:seglistitem[db:seg!='']">
      <xsl:text>%&lt;package>%
      <!-- call packages on each seglistitem -->
      <xsl:for-each
        select="/db:book/db:info/db:cover/db:constraintdef
                [@xml:id=concat($filetype,'packages')]
                [not(@linkend)]
                /db:segmentedlist/db:seglistitem
                  [db:seg!='']">
        <xsl:call-template name="packages">
          <xsl:with-param name="pkg" select="."/>
          <xsl:with-param name="dest" select="$filetype"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
  </xsl:template>
  <xsl:template name="packages">
    <xsl:param name="pkg"/>
    <xsl:param name="dest"/>
    <xsl:param name="cmdblocks"/>
    <xsl:param name="mode">
      <xsl:text></xsl:text>
    </xsl:param>
    <xsl:param name="loc"/>
    <xsl:choose>
      <!-- OMIT BECAUSE THE PACKAGE IS ADDED MANUALLY -->
      <xsl:when
        test="($dest='cls' or $dest='sty')
              and
              $loc/ancestor::db:part[@xml:id='code']
              and
              ($loc/ancestor::db:chapter
                /descendant::db:annotation
                [@xreflabel=$pkg/db:seg][@role='package']
               or
               $loc/ancestor::db:appendix
                /descendant::db:annotation
               [@xreflabel=$pkg/db:seg][@role='package'])">
        <xsl:text>% Package \textsf{</xsl:text>
        <xsl:value-of select="$pkg/db:seg"/>
        <xsl:text>} omitted here (coded manually)</xsl:text>
        <xsl:text> on p.\pageref{ann-</xsl:text>
        <xsl:value-of select="$pkg/db:seg"/>
        <xsl:text>}.\par&#xa;%% Package \textsf{</xsl:text>
        <xsl:value-of select="$pkg/db:seg"/>
        <xsl:text>} omitted (coded manually)&#xa;</xsl:text>
      </xsl:when>
      <!-- no need now we have ifpackageloaded
      <xsl:when test="$dest='sty'
                      and
                      $thisdoc/db:book/@xlink:role
                      and
                      $pkg/db:seg='xcolor'">
        <xsl:text>% Package \textsf{</xsl:text>
        <xsl:value-of select="$pkg/db:seg"/>
        <xsl:text>} may get omitted to avoid clashes&#xa;</xsl:text>
      </xsl:when>
      -->
      <!-- OTHERWISE NORMAL INCLUSION -->
      <xsl:otherwise>
        <xsl:if test="$mode='deferred'">
          <xsl:text>% \begin{CPKpackage}{</xsl:text>
          <xsl:value-of select="$pkg/db:seg"/>
          <xsl:text>}&#xa;</xsl:text>
          <xsl:apply-templates
            select="$prepost/db:procedure
                    [@xml:id='prepackage']
                    /db:step[contains(@condition,$dest)]
                    [@remap=$pkg/db:seg]/db:para"/>
          <!-- warn about package not in prepost -->
          <xsl:if
            test="count($prepost/db:procedure
                        [@xml:id='prepackage']
                        /db:step[contains(@condition,$dest)]
                        [@remap=$pkg/db:seg]/db:para)=0">
            <xsl:message>
              <xsl:text>WARNING: Package </xsl:text>
              <xsl:value-of select="$pkg/db:seg"/>
              <xsl:text> for </xsl:text>
              <xsl:value-of select="$filetype"/>
              <xsl:text> has no doc in prepost.xml</xsl:text>
            </xsl:message>
          </xsl:if>
          <!-- add comment from ClassPack document -->
          <xsl:if test="@role">
            <xsl:text>% </xsl:text>
            <xsl:value-of select="normalize-space(@role)"/>
            <xsl:text>&#xa;</xsl:text>
          </xsl:if>
          <!-- add explanation from prepost.xml if present -->
          <xsl:if test="count($prepost/db:procedure
                        [@xml:id='prepackage']
                        /db:step[contains(@condition,$dest)]
                        [@remap=$pkg/db:seg]/db:para)>0">
            <xsl:text>% \iffalse&#xa;%% </xsl:text>
            <xsl:value-of
              select="normalize-space(
                        $prepost/db:procedure
                        [@xml:id='prepackage']
                        /db:step[contains(@condition,$dest)]
                        [@remap=$pkg/db:seg]/db:para)"/>
            <xsl:text>&#xa;% \fi&#xa;</xsl:text>
          </xsl:if>
          <xsl:text>%    </xsl:text>
          <xsl:text>\begin</xsl:text>
          <xsl:text>{macrocode}&#xa;</xsl:text>
        </xsl:if>
        <!-- output pre-commands -->
        <xsl:for-each
          select="$prepost/db:procedure
                    [@xml:id='prepackage']
                  /db:step[contains(@condition,$dest)]
                  [@remap=$pkg/db:seg]
                  /db:constraintdef/db:cmdsynopsis
                    /db:command">
          <!-- check to escape internals in doc mode -->
          <xsl:if test="contains(.,'@') and $mode=''">
            <xsl:text>  \makeatletter&#xa;</xsl:text>
          </xsl:if>
          <xsl:text>  </xsl:text>
          <xsl:value-of select="replace(.,'^&#x9;','')"/>
          <xsl:text>&#xa;</xsl:text>
          <!-- end of escaping internals in doc mode -->
          <xsl:if test="contains(.,'@') and $mode=''">
            <xsl:text>  \makeatother&#xa;</xsl:text>
          </xsl:if>
        </xsl:for-each>
        <xsl:if test="$dest='sty' and (
                      $pkg/db:seg='xcolor'
                      or
                      $pkg/db:seg='hyperref'
                      or
                      $pkg/db:seg='url'
                      )">
          <xsl:text>\IfPackageLoaded{</xsl:text>
          <xsl:value-of select="$pkg"/>
          <xsl:text>}{\relax}{%&#xa;  </xsl:text>
          <!-- .true. requirepackage/usepackage goes here,
               .false. relax occurs further down -->
        </xsl:if>
        <xsl:choose>
          <xsl:when test="$mode='deferred'">
            <xsl:text>\RequirePackage</xsl:text>
          </xsl:when>
          <xsl:when test="$dest=$filetype">
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>>\RequirePackage</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>\usepackage</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:if
          test="$pkg/db:seg/@role
                or
                $prepost/db:procedure[@xml:id='prepackage']/
                db:step[contains(@condition,$dest)]
                [@remap=$pkg/db:seg]/@role">
          <xsl:choose>
            <!-- elide null options -->
            <xsl:when test="$pkg/db:seg/@role and $pkg/db:seg/@role=''
                            and
                            $prepost/db:procedure[@xml:id='prepackage']/
                            db:step[contains(@condition,$dest)]
                            [@remap=$pkg/db:seg]/@role=''">
              <xsl:text></xsl:text>
            </xsl:when>
            <!-- disallow options on package url disabled 2023-06-19
            <xsl:when test="$pkg/db:seg[.='url']">
              <xsl:text></xsl:text>
            </xsl:when>
            -->
            <!-- auto add svgnames option for xcolor
                 when package requires itself in doc -->
            <xsl:when test="$pkg/db:seg='xcolor' and
                            /db:book/@xlink:role and
                            $filetype='sty'">
              <xsl:text>[svgnames]</xsl:text>
            </xsl:when>
            <!-- look no further -->
            <xsl:when
              test="$pkg/db:seg/@role
                    and
                    $pkg/db:seg/@condition='only'">
              <xsl:text>[</xsl:text>
              <xsl:value-of select="$pkg/db:seg/@role"/>
              <xsl:text>]</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <!-- DEDUPLICATE -->
              <!-- string both sources together -->
              <xsl:variable name="reqs">
                <xsl:value-of
                  select="normalize-space($pkg/db:seg/@role)"/>
                <xsl:text>,</xsl:text>
                <xsl:value-of
                  select="normalize-space($prepost/db:procedure
                          [@xml:id='prepackage']/db:step
                            [contains(@condition,$dest)]
                            [@remap=$pkg/db:seg]
                            [contains('$pkg/db:seg/@role',@role)]
                            /@role)"/>
              </xsl:variable>
              <!-- catch any null values -->
              <xsl:variable name="args">
                <xsl:for-each
                  select="tokenize($reqs,',')[.!='']">
                  <arg>
                    <xsl:value-of select="normalize-space(.)"/>
                  </arg>
                </xsl:for-each>
              </xsl:variable>
              <xsl:if test="count($args/arg[.!=''])>0">
                <xsl:text>[</xsl:text>
                <xsl:call-template name="rewrap">
                  <xsl:with-param name="text">
                    <xsl:for-each
                      select="$args/arg[.!='']
                              [not(preceding-sibling::arg=.)]">
                      <xsl:if test="position()>1">
                        <xsl:text>,</xsl:text>
                      </xsl:if>
                      <xsl:value-of select="."/>
                    </xsl:for-each>
                  </xsl:with-param>
                </xsl:call-template>
                <xsl:text>]</xsl:text>
              </xsl:if>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
        <!-- PACKAGE NAME -->
        <xsl:text>{</xsl:text>
        <xsl:value-of select="$pkg/db:seg"/>
        <xsl:text>}</xsl:text>
        <!-- POST-OPTIONS -->
        <xsl:if test="($pkg/db:seg/@version
                       and
                       $pkg/db:seg/@version!='')
                      or
                      $prepost/db:procedure
                        [@xml:id='prepackage']/
                      db:step[contains(@condition,$dest)]
                        [@remap=$pkg/db:seg]/@revision">
          <xsl:text>[</xsl:text>
          <xsl:choose>
            <xsl:when test="$pkg/db:seg/@version[.!='']">
              <xsl:value-of
                select="translate(substring(
                        $pkg/db:seg/@version,1,10),
                        '-','/')"/>
            </xsl:when>
            <xsl:when
              test="$prepost/db:procedure
                    [@xml:id='prepackage']/
                    db:step[contains(@condition,$dest)]
                    [@remap=$pkg/db:seg]
                    [@role=$pkg/db:seg/@role]/@revision">
              <xsl:value-of
                select="translate(substring(
                        $prepost/db:procedure
                        [@xml:id='prepackage']/db:step
                          [contains(@condition,$dest)]
                          [@remap=$pkg/db:seg]
                          [@role=$pkg/db:seg/@role]
                        /@revision,1,10),'-','/')"/>
            </xsl:when>
          </xsl:choose>
          <xsl:text>]</xsl:text>
        </xsl:if>
        <xsl:if test="$pkg/@role and $mode=''">
          <xsl:text>% </xsl:text>
          <xsl:value-of
            select="normalize-space($pkg/@role)"/>
          <xsl:if test="@userlevel">
            <xsl:text> (</xsl:text>
            <xsl:value-of select="@userlevel"/>
            <xsl:text>)</xsl:text>
          </xsl:if>
        </xsl:if>
        <xsl:text>%&#xa;</xsl:text>
        <!-- complete the enclosure of ifpackageloaded -->
        <xsl:if test="$dest='sty' and
                      ($pkg/db:seg='xcolor'
                       or $pkg/db:seg='hyperref'
                       or $pkg/db:seg='url')">
          <xsl:text>}%&#xa;</xsl:text>
        </xsl:if>
        <!-- output post-commands -->
        <xsl:if test="not($pkg/db:seg/@condition='only')">
          <!-- go through any supplied commands
               that are not excluded -->
          <xsl:for-each
            select="$prepost/db:procedure
                    [@xml:id='postpackage']/db:step
                      [contains(@condition,$dest)]
                      [@remap=$pkg/db:seg]
                    /db:constraintdef/db:cmdsynopsis
                    /db:command">
            <!-- exclude commands because of another package -->
            <xsl:if test="not(current()/@conformance=$cmdblocks)">
              <xsl:variable name="matchconditions"
                select="$prepost/db:procedure
                        [@xml:id='prepackage']
                        /db:step
                        [contains(@condition,$dest)]
                        [@remap=$pkg/db:seg]
                        /db:constructorsynopsis"/>
              <!-- load any command without an xml:id
                   OR
                   with an xml:id matching a condition from
                   the prepost section (ie programlisting
                   with a language whose name matches this
                   xml:id. Silent if no matches. -->
              <xsl:if
                test="not(@xml:id)
                      or
                      @xml:id=$thisdoc//*
                      [name()=$matchconditions/@condition]
                      /@*
                      [name()=$matchconditions
                      /db:methodparam/db:parameter]">
                <xsl:if test="contains(.,'@') and $mode=''">
                  <xsl:text>  \makeatletter&#xa;</xsl:text>
                </xsl:if>
                <xsl:text>  </xsl:text>
                <xsl:value-of
                  select="replace(.,'^&#x9;','')"/>
                <xsl:text>&#xa;</xsl:text>
                <xsl:if test="contains(.,'@') and $mode=''">
                  <xsl:text>  \makeatother&#xa;</xsl:text>
                </xsl:if>
              </xsl:if>
            </xsl:if>
          </xsl:for-each>
          <!-- special for programlisting and the
               listing package: iterate over languages -->
          <xsl:if test="$pkg/db:seg='listings'">
            <xsl:for-each-group group-by="@language"
              select="$thisdoc
                      //(db:programlisting|db:literallayout)">
              <xsl:text>  \lstloadlanguages{</xsl:text>
              <xsl:value-of select="current-grouping-key()"/>
              <xsl:text>}&#xa;</xsl:text>
            </xsl:for-each-group>
            <xsl:text>  \lstloadlanguages{dummy}&#xa;</xsl:text>
          </xsl:if>
        </xsl:if>
        <xsl:if test="$mode='deferred'">
          <xsl:text>%    </xsl:text>
          <xsl:text>\end</xsl:text>
          <xsl:text>{macrocode}&#xa;</xsl:text>
          <!-- post-package comments from prepost.xml -->
          <xsl:apply-templates
            select="$prepost/db:procedure
                    [@xml:id='postpackage']
                    /db:step[contains(@condition,$dest)]
                    [@remap=current()/db:seg]/db:para"/>
          <xsl:text>% \end{CPKpackage}&#xa;</xsl:text>
        </xsl:if>
        <xsl:if test="$mode=''
                      and
                      $dest='doc'
                      and
                      $doctype='package'
                      and
                      /db:book/@role">
          <xsl:text>% \usepackage</xsl:text>
          <xsl:if test="/db:book/@role!=''">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="/db:book/@role"/>
            <xsl:text>]</xsl:text>
          </xsl:if>
          <xsl:text>{</xsl:text>
          <xsl:value-of select="$docname"/>
          <xsl:text>}% added by spec.&#xa;</xsl:text>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template name="rewrap">
    <xsl:param name="text"/>
    <xsl:param name="maxlen">
      <xsl:text>32</xsl:text>
    </xsl:param>
    <xsl:choose>
      <!-- if text is an overlong string
           but contains a comma -->
      <xsl:when
        test="string-length($text)>$maxlen
        and contains($text,',')">
        <!-- see if there is a comma
             within the first $maxlen chars -->
        <xsl:variable name="prelastcomma">
          <xsl:choose>
            <xsl:when
              test="contains(substring($text,1,$maxlen),
                    ',')">
              <xsl:analyze-string
                select="substring($text,1,$maxlen)"
                regex="^(.*)(,)([^,]*)$">
                <xsl:matching-substring>
                  <xsl:value-of
                    select="concat(regex-group(1),',')"/>
                </xsl:matching-substring>
                <xsl:non-matching-substring>
                  <xsl:text>NO "," IN SUBSTRING</xsl:text>
                </xsl:non-matching-substring>
              </xsl:analyze-string>
            </xsl:when>
            <!-- No comma in range,
                 test whole string -->
            <xsl:otherwise>
              <xsl:analyze-string select="$text"
                regex="^(.*)(,)([^,]*)$">
                <xsl:matching-substring>
                  <xsl:value-of
                    select="concat(regex-group(1),',')"/>
                </xsl:matching-substring>
                <xsl:non-matching-substring>
                  <xsl:text>NO "," IN STRING</xsl:text>
                </xsl:non-matching-substring>
              </xsl:analyze-string>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="$prelastcomma"/>
        <xsl:choose>
          <xsl:when
            test="ancestor::db:constraintdef
                  [@xml:id=concat($filetype,'packages')]">
            <xsl:text>&#xa;  </xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>&#xa;  </xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:call-template name="rewrap">
          <xsl:with-param name="text"
            select="normalize-space(
                    substring-after(
                    $text,$prelastcomma))"/>
          <xsl:with-param name="maxlen">
            <xsl:text>50</xsl:text>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template name="makechapapp">
    <!-- output contents of chapter or appendix -->
    <!-- full output file name -->
    <xsl:variable name="thisdocfullname">
      <xsl:choose>
        <xsl:when test="@xlink:href">
          <xsl:value-of
            select="tokenize(@xlink:href,'/')
                    [position()=last()]"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$docname"/>
          <xsl:text>.</xsl:text>
          <xsl:value-of select="$filetype"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="thisdocname"
      select="tokenize($thisdocfullname,'\.')[1]"/>
    <xsl:variable name="thisdoctype"
      select="tokenize($thisdocfullname,'\.')
              [position()=last()]"/>
    <xsl:variable name="thisdocfulltype">
      <xsl:choose>
        <xsl:when test="$thisdoctype='cls'">
          <xsl:text>Class</xsl:text>
        </xsl:when>
        <xsl:when test="$thisdoctype='sty'">
          <xsl:text>Package</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>File</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="thisdocid">
      <xsl:choose>
        <xsl:when test="@xml:id">
          <xsl:value-of select="@xml:id"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="/db:book/@xml:id"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!-- choices are: 'doc', 'code', and otherwise -->
    <xsl:choose>
      <!-- EXTRACT ALL THE DOCUMENTATION -->
      <xsl:when test="parent::db:part[@xml:id='doc']">
        <xsl:apply-templates/>
      </xsl:when>
      <xsl:when test="parent::db:part[@xml:id='code']">
        <!-- CODE COUNTER NEEDS TO BE RESET to 0 here -->
        <xsl:if test="$thisdoctype='dtd' or
                      $thisdoctype='xml' or
                      $thisdoctype='xsl' or
                      $thisdoctype='awk' or
                      $thisdoctype='sh' or
                      $thisdoctype='cp' or
                      $thisdoctype='bash'">
          <xsl:text>% \setcounter{CodelineNo}{0}</xsl:text>
          <xsl:text>&#xa;</xsl:text>
        </xsl:if>
        <!-- OUTPUT ANYTHING BEFORE THE FIRST <sect1>
             (usually some kind of preamble or explanation)
             provided there is NO code -->
        <xsl:if test="not(db:sect1[1]/preceding-sibling::*/
                      descendant-or-self::db:programlisting)">
          <xsl:apply-templates
            select="db:sect1[1]/preceding-sibling::*"/>
          <!-- if there is no sect1, or if there IS code,
               this does nothing -->
        </xsl:if>
        <!-- if there is a programlisting and an output
             filename and an ID, start with a comment. -->
        <xsl:if test="ancestor-or-self::db:appendix
                      [descendant::db:programlisting
                      and @xlink:href
                      and @xml:id]">
          <!--- use sentinel to enclose the comment of
               identity in the right format -->
          <xsl:call-template name="startappcomment">
            <xsl:with-param name="scriptflag">
              <xsl:choose>
                <xsl:when test="@arch='script' and @userlevel='bash'">
                  <xsl:text>wait</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>no</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:if>
        <xsl:if test="$thisdoctype!='dtd' and
                      $thisdoctype!='xml' and
                      $thisdoctype!='xsl' and
                      $thisdoctype!='awk' and
                      $thisdoctype!='sh' and
                      $thisdoctype!='cp' and
                      $thisdoctype!='bash'">
          <xsl:text>% \subsection{Auto-initialisation}</xsl:text>
          <xsl:text>\label{</xsl:text>
          <xsl:value-of select="@xml:id"/>
          <xsl:text>:autoinit}
          <!-- reset the counter if this is being done
               from an <appendix>, because the counter
               will still show the last line of the main
               (<chapter>) block of code -->
          <xsl:if test="name()='appendix'">
            <!-- it should probably be reset anyway -->
            <xsl:text>% \setcounter{CodelineNo}{0}</xsl:text>
            <xsl:text>&#xa;</xsl:text>
          </xsl:if>
  <!-- working on ways to make this use lstlisting
       as it doesn't need macrocode, having been tagged
       <*package> or <*class> earlier -->
          <xsl:text>% \begingroup\color{DarkRed}</xsl:text>
          <xsl:text>\footnotesize&#xa;% \leavevmode</xsl:text>
          <!-- removed leading \enspace as it now interferes
               with the alignment of the first line 2023-05-15.
               Put it back 2023-05-16 -->
          <xsl:text>\enspace{\scriptsize1}\quad</xsl:text>
          <xsl:text>{\ttfamily\textbackslash </xsl:text>
          <xsl:text>NeedsTeXFormat\{</xsl:text>
          <xsl:value-of select="/db:book/@conformance"/>
          <xsl:text>\}[</xsl:text>
          <xsl:value-of
            select="translate(/db:book/@condition,'-','/')"/>
          <xsl:text>]}\\&#xa;% \leavevmode</xsl:text>
          <xsl:text>\enspace{\scriptsize2}\quad</xsl:text>
          <xsl:text>{\ttfamily\textbackslash </xsl:text>
          <xsl:text>Provides</xsl:text>
          <xsl:value-of select="$thisdocfulltype"/>
          <xsl:text>\{</xsl:text>
          <xsl:value-of select="$thisdocname"/>
          <xsl:text>\}[</xsl:text>
          <xsl:value-of select="translate($date,'-','/')"/>
          <xsl:text> v</xsl:text>
          <xsl:value-of select="/db:book/@version"/>
          <xsl:text>.</xsl:text>
          <xsl:value-of select="/db:book/@revision"/>
          <xsl:text>}\\&#xa;% \leavevmode</xsl:text>
          <xsl:text>\enspace{\scriptsize3}\qquad</xsl:text>
          <xsl:text>{\ttfamily </xsl:text>
          <xsl:choose>
            <xsl:when
              test="//db:part[@xml:id='code']/db:title">
              <xsl:value-of
                select="normalize-space(replace(
                        //db:part[@xml:id='code']/db:title,
                        '\\LaTeX\{\}','LaTeX'))"/>
            </xsl:when>
            <xsl:when test="name()='appendix'">
              <xsl:value-of
                select="normalize-space(replace(db:title,
                        '\\LaTeX\{\}','LaTeX'))"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of
                select="normalize-space(/db:book/db:info
                        /db:title)"/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>]}\\\endgroup
          <!-- APPENDICES ONLY -->
          <xsl:if
            test="ancestor-or-self::db:appendix and (
                  ancestor-or-self::db:appendix/@userlevel='tex' or
                  ancestor-or-self::db:appendix/@userlevel='sty' or
                  ancestor-or-self::db:appendix/@userlevel='cls'
                        )">
            <xsl:text>% \iffalse&#xa;%&lt;</xsl:text>
            <xsl:value-of select="$thisdocid"/>
            <xsl:text>>\NeedsTeXFormat{LaTeX2e}</xsl:text>
            <xsl:text>[2015/01/01]% </xsl:text>
            <xsl:value-of select="name()"/>
            <xsl:text>/</xsl:text>
            <xsl:value-of select="@xml:id"/>
            <xsl:text>&#xa;%&lt;</xsl:text>
            <xsl:value-of select="$thisdocid"/>
            <xsl:text>>\Provides</xsl:text>
            <xsl:value-of select="$thisdocfulltype"/>
            <xsl:text>{</xsl:text>
            <xsl:value-of select="$thisdocname"/>
            <xsl:text>}[</xsl:text>
            <xsl:value-of
              select="translate($date,'-','/')"/>
            <xsl:text> v</xsl:text>
            <xsl:value-of select="/db:book/@version"/>
            <xsl:text>.</xsl:text>
            <xsl:value-of select="/db:book/@revision"/>
            <xsl:text>&#xa;</xsl:text>
            <xsl:text>%&lt;</xsl:text>
            <xsl:value-of select="$thisdocid"/>
            <xsl:text>>  </xsl:text>
            <xsl:value-of
              select="normalize-space(replace(
                      db:title,'\\LaTeX\{\}','LaTeX'))"/>
            <xsl:text>]&#xa;% \fi&#xa;</xsl:text>
          </xsl:if>
        </xsl:if>
        <!-- end of selective chapcpk -->
        <!-- CLASSES ONLY -->
        <xsl:if test="$doctype='class'
                      and
                      not(name()='appendix')">
          <xsl:text>% \begin{CPKpackage}{fix-cm}
          <xsl:text>\begin</xsl:text>
          <xsl:text>{macrocode}
\RequirePackage{fix-cm}
          <xsl:text>\end</xsl:text>
          <xsl:text>{macrocode}
        </xsl:if>
        <xsl:choose>
          <xsl:when test="@condition='noauto'">
            <xsl:text>% \par Auto-generation of options to
          </xsl:when>
          <xsl:otherwise>
            <!-- special handling for [svgnames]{xcolor} -->
            <xsl:if
              test="//db:constraintdef[@xml:id='clspackages']
                    //db:seg[.='xcolor']
                    [contains(@role,'svgnames')
                     or
                     not(@role)]
                    [not(@condition='only')]">
              <xsl:text>% \begin{CPKoption}{svgnames}
<!-- EXCEPTION HANDLER WHEN PACKAGE ITSELF INCLUDED removed
              <xsl:if test="/db:book/@xlink:role
                            and
                            $filetype='sty'">
                <xsl:text>% However, we have to make
                <xsl:text>\begin</xsl:text>
                <xsl:text>{macrocode}
\def\CPKthispackage{</xsl:text>
                <xsl:value-of select="$docname"/>
                <xsl:text>}
\edef\CPKthispackage{\meaning\CPKthispackage}
\edef\CPKthisjob{\jobname}
\edef\CPKthisjob{\meaning\CPKthisjob}
\ifx\CPKthispackage\CPKthisjob
%% this is the documentation: omit PassOptionsToPackage
  \PackageWarning{classpack}{Option svgnames has not
  been passed ^^Jto package xcolor (because this is
  the package documentation)}
\else
%% this is a user job: include PassOptionsToPackage
  \PackageWarning{classpack}{Option svgnames is being
  passed ^^Jto package xcolor (to avoid later clashes)}
                <xsl:text>\end</xsl:text>
                <xsl:text>{macrocode}&#xa;</xsl:text>
              </xsl:if>
-->
              <xsl:text>%    </xsl:text>
              <xsl:text>\begin</xsl:text>
              <xsl:text>{macrocode}
  \PassOptionsToPackage{svgnames}{xcolor}
              <xsl:text>\end</xsl:text>
              <xsl:text>{macrocode}&#xa;</xsl:text>
              <!--
              <xsl:if
                test="/db:book/@xlink:role
                      and
                      $filetype='sty'">
                <xsl:text>%    </xsl:text>
                <xsl:text>\begin</xsl:text>
                <xsl:text>{macrocode}
\fi
                <xsl:text>\end</xsl:text>
                <xsl:text>{macrocode}&#xa;</xsl:text>
              </xsl:if>
-->
              <xsl:text>% \end{CPKoption}&#xa;</xsl:text>
            </xsl:if>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:choose>
          <!-- we did anything prior to the sect1 earlier
               so now do the sect1 and everything following -->
          <xsl:when test="db:sect1">
            <xsl:apply-templates
              select="db:sect1[1] |
                      db:sect1[1]/following-sibling::*"/>
          </xsl:when>
          <!-- if there is no sect1, nothing will have been
               output earlier, so do it all now -->
          <xsl:otherwise>
            <xsl:apply-templates/>
          </xsl:otherwise>
        </xsl:choose>
        <!-- Unclear if this is still necessary -->
        <xsl:call-template name="checkpackages">
          <xsl:with-param name="pos" select="'after'"/>
          <xsl:with-param name="loc" select="."/>
        </xsl:call-template>
        <!-- scripts only -->
        <xsl:if test="$doctype='script'">
          <xsl:text>% [Exit added by \textit{ClassPack}
## Exit added by ClassPack to shield shell from DocTeX.
          <xsl:text>\begin</xsl:text>
          <xsl:text>{macrocode}&#xa;exit 0</xsl:text>
          <xsl:text>&#xa;%    </xsl:text>
          <xsl:text>\end</xsl:text>
          <xsl:text>{macrocode}&#xa;</xsl:text>
        </xsl:if>
      </xsl:when><!-- end of dealing with part/code -->
      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template name="checkpackages">
    <xsl:param name="pos"/><!-- before or after -->
    <xsl:param name="loc"/><!-- name of calling element type -->
    <xsl:variable name="thisgi"
      select="$thisdoc/db:book/db:info/db:cover/db:constraintdef
              [@xml:id=concat($filetype,'packages')]/@linkend"/>
    <xsl:variable name="thisrole"
      select="$thisdoc/db:book/db:info/db:cover/db:constraintdef
              [@xml:id=concat($filetype,'packages')]/@role"/>
    <!--
    <xsl:text>%% Checking in </xsl:text>
    <xsl:value-of select="name()"/>
    <xsl:text>/</xsl:text>
    <xsl:value-of select="@xml:id"/>
    <xsl:text> for package location </xsl:text>
    <xsl:value-of select="$pos"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="name()"/>
    <xsl:text>/</xsl:text>
    <xsl:value-of select="$thisgi"/>
    <xsl:text>&#xa;</xsl:text>
    -->
    <xsl:if
      test="$thisgi
            and
            current()/@xml:id
            and
            $thisgi=current()/@xml:id
            and
            ($pos=$thisrole
             or
             not($thisrole))">
      <xsl:variable name="gi" select="local-name()"/>
      <xsl:text>%\iffalse
%%&#xa;</xsl:text>
      <!--
      <xsl:text>%% [in checkpackages with pos=</xsl:text>
      <xsl:value-of select="$pos"/>
      <xsl:text>, loc=</xsl:text>
      <xsl:value-of select="substring($loc,1,32)"/>
      <xsl:text>, thisgi=</xsl:text>
      <xsl:value-of select="$thisgi"/>
      <xsl:text>, thisrole=</xsl:text>
      <xsl:value-of select="$thisrole"/>
      <xsl:text>&#xa;</xsl:text>
      -->
      <xsl:text>%% Packages required for the </xsl:text>
      <xsl:value-of select="$doctype"/>
      <xsl:text>
%%
      <xsl:choose>
        <!-- $gi is the element type nominated by the @linkend
             attribute in the stypackage or clspackages
             constraintdef where the class/package packages
             are declared -->
        <xsl:when test="$gi='chapter' or $gi='appendix'">
          <xsl:text>clearpage&#xa;% \section</xsl:text>
        </xsl:when>
        <xsl:when test="$gi='sect1'">
          <xsl:text>subsection</xsl:text>
        </xsl:when>
        <xsl:when test="$gi='sect2'">
          <xsl:text>subsubsection</xsl:text>
        </xsl:when>
        <xsl:when test="$gi='sect3'">
          <xsl:text>paragraph</xsl:text>
        </xsl:when>
        <xsl:when test="$gi='sect4'">
          <xsl:text>subparagraph</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:message>
            <xsl:text>WARNING: </xsl:text>
            <xsl:value-of select="$gi"/>
            <xsl:text> is an unidentifiable </xsl:text>
            <xsl:text>place to declare packages</xsl:text>
          </xsl:message>
          <xsl:text>\PackageWarning{classpack}{Unidentifiable place `</xsl:text>
          <xsl:value-of select="$thisgi"/>
          <xsl:text>' to declare packages}</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>{</xsl:text>
      <xsl:choose>
        <xsl:when
          test="/db:book/db:info/db:cover
                /db:constraintdef
                 [@linkend=current()/@xml:id]
                /db:segmentedlist/db:segtitle">
          <xsl:apply-templates
            select="/db:book/db:info/db:cover
                    /db:constraintdef
                     [@linkend=current()/@xml:id]
                    /db:segmentedlist/db:segtitle/node()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>Packages loaded</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>}\label{</xsl:text>
      <xsl:value-of
        select="/db:book/db:info/db:cover
                /db:constraintdef
                 [@linkend=current()/@xml:id]
                /@xml:id"/>
      <xsl:text>}&#xa;</xsl:text>
      <!-- INCLUDE COMPULSORY PACKAGE[s] for cls/sty -->
      <xsl:if test="($filetype='cls' or $filetype='sty')
                    and
                    count($prepost/db:procedure[@xml:id='prepackage']
                      /db:step[contains(@condition,$filetype)]
                      [db:constructorsynopsis/@condition='']) > 0">
        <xsl:text>% Packages required for operation:&#xa;</xsl:text>
        <xsl:for-each
          select="$prepost/db:procedure[@xml:id='prepackage']
                  /db:step[contains(@condition,$filetype)]
                  [db:constructorsynopsis/@condition='']">
          <xsl:text>% \begin{CPKpackage}{</xsl:text>
          <xsl:value-of select="@remap"/>
          <xsl:text>}&#xa;</xsl:text>
          <xsl:apply-templates select="db:para"/>
          <xsl:text>% \iffalse&#xa;%% </xsl:text>
          <xsl:value-of select="normalize-space(db:para[1])"/>
          <xsl:text>&#xa;% \fi&#xa;</xsl:text>
          <xsl:text>%    \begin{macrocode}&#xa;</xsl:text>
          <xsl:text>\RequirePackage{</xsl:text>
          <xsl:value-of select="@remap"/>
          <xsl:text>}%&#xa;</xsl:text>
          <xsl:if
            test="db:constraintdef[db:cmdsynopsis[db:command]]">
            <xsl:for-each
              select="db:constraintdef/db:cmdsynopsis/db:command">
              <xsl:value-of select="."/>
              <xsl:text>&#xa;</xsl:text>
            </xsl:for-each>
          </xsl:if>
          <xsl:text>%    \end</xsl:text>
          <xsl:text>{macrocode}&#xa;</xsl:text>
          <!-- and add on any code from the postpackage section -->
          <xsl:for-each
            select="$prepost/db:procedure[@xml:id='postpackage']
                      /db:step[contains(@condition,$filetype)]
                      [@remap=current()/@remap]">
            <xsl:text>% </xsl:text>
            <xsl:value-of select="normalize-space(db:para[1])"/>
            <xsl:text>&#xa;</xsl:text>
            <xsl:text>%    \begin{macrocode}&#xa;</xsl:text>
            <xsl:if
              test="db:constraintdef[db:cmdsynopsis[db:command]]">
              <xsl:for-each
                select="db:constraintdef/db:cmdsynopsis/db:command">
                <xsl:value-of select="."/>
                <xsl:text>&#xa;</xsl:text>
              </xsl:for-each>
            </xsl:if>
            <xsl:text>%    \end</xsl:text>
            <xsl:text>{macrocode}&#xa;</xsl:text>
          </xsl:for-each>
          <xsl:text>% \end{CPKpackage}&#xa;</xsl:text>
        </xsl:for-each>
      </xsl:if><!-- end of required packages for cls/sty -->
      <!-- now each requested package -->
      <xsl:for-each
        select="/db:book/db:info/db:cover
                /db:constraintdef
                 [@linkend=current()/@xml:id]
                /db:segmentedlist/db:seglistitem
                 [db:seg!='']">
        <xsl:call-template name="packages">
          <!-- context is seglistitem from cls/sty packages -->
          <xsl:with-param name="pkg" select="."/>
          <xsl:with-param name="dest" select="$filetype"/>
          <xsl:with-param name="cmdblocks"
            select="$prepost/db:procedure[@xml:id='postpackage']/
                    db:step[@remap=current()/db:seg]/
                    db:constraintdef/db:cmdsynopsis/
                    db:command/@conformance"/>
          <xsl:with-param name="mode" select="'deferred'"/>
          <xsl:with-param name="loc" select="$loc"/>
        </xsl:call-template>
      </xsl:for-each>
      <xsl:text>% &#xa;</xsl:text>
      <xsl:if
        test="/db:book/db:info/db:cover
              /db:constraintdef
               [@linkend=current()/@xml:id]
              /@xml:id!='stypackages'
              and
              /db:book/db:info/db:cover
              /db:constraintdef
               [@linkend=current()/@xml:id]
              /@xml:id!='clspackages'">
        <xsl:message>
          <xsl:text>WARNING: packages listed in "</xsl:text>
          <xsl:value-of select="$gi"/>
          <xsl:text>" that doesn't belong to this </xsl:text>
          <xsl:value-of select="$doctype"/>
          <xsl:text> (xml:id="</xsl:text>
          <xsl:value-of select="@xml:id"/>
          <xsl:text>").</xsl:text>
        </xsl:message>
        <xsl:text>% WARNING: packages listed in a \verb`</xsl:text>
        <xsl:value-of select="$gi"/>
        <xsl:text>` element of the master document </xsl:text>
        <xsl:text>that was not flagged as belonging to this </xsl:text>
        <xsl:value-of select="$doctype"/>
        <xsl:text> (\verb`xml:id="</xsl:text>
        <xsl:value-of select="@xml:id"/>
        <xsl:text>"`).&#xa;</xsl:text>
      </xsl:if>
    </xsl:if>
  </xsl:template>
  <xsl:template name="readme">
    <xsl:variable name="file">
      <xsl:text>README.md</xsl:text>
    </xsl:variable>
    <xsl:result-document format="textFormat"
      href="{$file}">
      <xsl:text>&#xa;[//]: # (the </xsl:text>
      <xsl:value-of select="$docname"/>
      <xsl:text> LaTeX </xsl:text>
      <xsl:value-of select="$doctype"/>
      <xsl:text>)&#xa;&#xa;# The </xsl:text>
      <xsl:value-of select="$docname"/>
      <xsl:text> LaTeX2ε package&#xa;&#xa;This is the </xsl:text>
      <xsl:value-of select="$file"/>
      <xsl:text> file for the </xsl:text>
      <xsl:value-of select="$docname"/>
      <xsl:text> LaTeX </xsl:text>
      <xsl:value-of select="$doctype"/>
      <xsl:text> v</xsl:text>
      <xsl:value-of select="$latestrevhist"/>
      <xsl:text> (</xsl:text>
      <xsl:value-of select="$date"/>
      <xsl:text>)&#xa;generated by ClassPack v.</xsl:text>
      <xsl:value-of select="$thisversion"/>
      <xsl:text> (</xsl:text>
      <xsl:value-of select="$thisverdate"/>
      <xsl:text>) on </xsl:text>
      <xsl:value-of
        select="format-dateTime(current-dateTime(),
                '[D] [MNn] [Y] at [H]:[m]:[s] ')"/>
      <xsl:text>&#xa;</xsl:text>
      <xsl:apply-templates select="$readme/*"
        mode="readme"/>
      <xsl:if test="$readme//db:footnote">
        <xsl:text>---&#xa;#### Footnotes&#xa;</xsl:text>
        <xsl:for-each select="$readme//db:footnote">
          <xsl:number format="1." count="db:footnote"
            level="any"/>
          <xsl:apply-templates select="node()"
            mode="inline"/>
          <xsl:text>&#xa;</xsl:text>
        </xsl:for-each>
      </xsl:if>
    </xsl:result-document>
  </xsl:template>
  <xsl:template name="manifest">
    <xsl:variable name="file">
      <xsl:text>MANIFEST</xsl:text>
    </xsl:variable>
    <xsl:result-document format="textFormat"
      href="{$file}">
      <!-- THESE THREE ALWAYS -->
      <xsl:text>README.md
MANIFEST
VERSION&#xa;</xsl:text>
      <!-- always the PDF, DTX, and INS -->
      <xsl:value-of select="$docname"/>
      <xsl:text>.dtx&#xa;</xsl:text>
      <xsl:value-of select="$docname"/>
      <xsl:text>.ins&#xa;</xsl:text>
      <xsl:value-of select="$docname"/>
      <xsl:text>.pdf&#xa;</xsl:text>
      <!-- the style or class file, obv -->
      <xsl:value-of select="$docname"/>
      <xsl:text>.</xsl:text>
      <xsl:value-of select="$filetype"/>
      <xsl:text>&#xa;</xsl:text>
      <!-- annotated appendix files and standalone
           unannotated files (the Makefile
           omits scripts from packaging )-->
      <xsl:for-each
        select="/db:book/db:part[@xml:id='code']
                /db:appendix[@xml:id
                and @xlink:href
                and descendant::db:programlisting
                and not(matches(@xlink:href,'\.cls$'))
                and not(matches(@xlink:href,'\.sty$'))]
                |
                /db:book/db:part[@xml:id='files']
                /db:chapter[@xml:id and @xlink:href
                and db:programlisting]">
        <xsl:value-of select="@xlink:href"/>
        <xsl:text>&#xa;</xsl:text>
      </xsl:for-each>
      <!-- IMAGE FOR DOC, should be in cover/artwork -->
      <xsl:if
        test="/db:book/db:info/db:abstract/@xlink:href
              and
              not(/db:book/db:info/db:abstract/@xlink:show='none')">
        <xsl:value-of
          select="/db:book/db:info/db:abstract/@xlink:href"/>
        <xsl:text>&#xa;</xsl:text>
      </xsl:if>
      <!-- ANY SPECIAL REQUESTS -->
      <xsl:for-each
        select="/db:book/db:info/db:cover
                /db:constraintdef[@xml:id='manifest']
                /db:simplelist/db:member[.!='']">
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>&#xa;</xsl:text>
      </xsl:for-each>
    </xsl:result-document>
  </xsl:template>
  <xsl:template name="makecommentchar">
    <xsl:variable name="closest"
      select="(ancestor::db:book |
               ancestor::db:chapter |
               ancestor::db:appendix)[1]"/>
    <xsl:value-of
      select="$commentchars/*
              [name()=$closest/@arch]
              [@filetype=$closest/@userlevel]"/>
  </xsl:template>
  <xsl:template name="makesentinel">
    <xsl:variable name="closest"
      select="(ancestor::db:book |
               ancestor::db:chapter |
               ancestor::db:appendix)[1]"/>
    <xsl:variable name="filetype"
      select="$commentchars/*
              [name()=$closest/@arch]
              [@filetype=$closest/@userlevel]"/>
    <xsl:value-of select="$filetype"/>
    <!-- repeat for non-XML file types -->
    <xsl:if test="$filetype[not(@term)]">
      <xsl:value-of select="$filetype"/>
    </xsl:if>
    <xsl:text> </xsl:text>
  </xsl:template>
  <xsl:template name="maketermsentinel">
    <xsl:variable name="closest"
      select="(ancestor::db:book |
               ancestor::db:chapter |
               ancestor::db:appendix)[1]"/>
    <xsl:variable name="filetype"
      select="$commentchars/*
              [name()=$closest/@arch]
              [@filetype=$closest/@userlevel]"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="$filetype/@term"/>
    <!-- repeat for non-XML file types -->
    <xsl:if test="$filetype[not(@term)]">
      <xsl:value-of select="$filetype/@term"/>
    </xsl:if>
  </xsl:template>
  <xsl:template name="makefence">
    <xsl:variable name="sentinel">
      <xsl:call-template name="makesentinel"/>
    </xsl:variable>
    <xsl:variable name="termsentinel">
      <xsl:call-template name="maketermsentinel"/>
    </xsl:variable>
    <xsl:value-of select="$sentinel"/>
    <xsl:for-each select="for $i in 1 to 66 return $i">
      <xsl:text>*</xsl:text>
    </xsl:for-each>
    <xsl:value-of select="$termsentinel"/>
    <xsl:text>&#xa;</xsl:text>
  </xsl:template>
  <xsl:template name="start-tag">
    <xsl:param name="identity">
      <xsl:value-of select="@xml:id"/>
    </xsl:param>
    <xsl:text>% \iffalse&#xa;%&lt;*</xsl:text>
    <xsl:value-of select="$identity"/>
    <xsl:text>>&#xa;% \fi&#xa;</xsl:text>
  </xsl:template>
  <xsl:template name="end-tag">
    <xsl:param name="identity">
      <xsl:value-of select="@xml:id"/>
    </xsl:param>
    <xsl:text>% \iffalse&#xa;%&lt;/</xsl:text>
    <xsl:value-of select="$identity"/>
    <xsl:text>>&#xa;% \fi&#xa;</xsl:text>
  </xsl:template>
  <xsl:function name="sil:commentchar">
    <xsl:param name="context"/>
    <xsl:variable name="closest"
      select="($context/ancestor-or-self::db:book[1] |
               $context/ancestor-or-self::db:chapter[1] |
               $context/ancestor-or-self::db:appendix[1])[last()]"/>
    <xsl:variable name="thisdoctype"
      select="tokenize($closest/@xlink:href,'\.')
                  [last()]"/>
    <xsl:value-of
      select="($commentchars/*[name()=$closest/@arch][1] |
               $commentchars/*[@filetype=$closest/@userlevel][1] |
               $commentchars/*[@filetype=$thisdoctype][1])[last()]"/>
  </xsl:function>
  <xsl:function name="sil:termcommentchar">
    <xsl:param name="context"/>
    <!-- no parameters: evaluates using current context -->
    <xsl:variable name="closest"
      select="$context/ancestor-or-self::*
              [name()='book'
               or
               name()='chapter'
               or
               name()='appendix'][1]"/>
    <xsl:variable name="filetype"
      select="$commentchars/*
               [name()=$closest/@arch]
               [@filetype=$closest/@userlevel][1]"/>
    <xsl:value-of select="$filetype/@term"/>
  </xsl:function>
  <xsl:function name="sil:sentinel">
    <xsl:param name="context"/>
    <xsl:value-of select="sil:commentchar($context)"/>
    <xsl:if test="sil:commentchar($context)!='&lt;!--'">
      <xsl:value-of select="sil:commentchar($context)"/>
    </xsl:if>
  </xsl:function>
  <xsl:template name="avoidverb">
    <xsl:variable name="delim">
      <xsl:choose>
        <xsl:when test="not(contains(.,'|'))">
          <xsl:text>|</xsl:text>
        </xsl:when>
        <xsl:when test="not(contains(.,'+'))">
          <xsl:text>+</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>`</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="matches(.,'.*[\\\$\^{}&lt;\|&gt;␣].*')
                      and not(ancestor::db:title)
                      and not(ancestor::db:footnote)
                      and not(ancestor::db:term)">
        <xsl:choose>
          <xsl:when
            test="db:replaceable or @condition='nolit'">
            <xsl:text>\verb</xsl:text>
            <xsl:value-of select="$delim"/>
            <!-- PREFIXING -->
            <xsl:if test="(name()='command'
                           or
                           @role='length')
                          and
                          (@language='TeX'
                           or
                           @language='LaTeXe'
                           or
                           @language='LaTeX'
                           or
                           not(@language))">
              <xsl:text>\</xsl:text>
            </xsl:if>
            <xsl:if test="@language='XSLT'
                          and
                          (name()='varname' and
                            (@role='variable' or
                             @role='parameter'))
                          or
                          name()='parameter'">
              <xsl:text>$</xsl:text>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='Makefile'
                          and
                          @role='variable'">
              <xsl:text>$(</xsl:text>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='BiBTeX'
                          and
                          @role='entrytype'">
              <xsl:text>@</xsl:text>
            </xsl:if>
            <xsl:apply-templates/>
            <!-- SUFFIXING -->
            <xsl:if test="@xlink:type">
              <xsl:text>.</xsl:text>
              <xsl:value-of select="@xlink:type"/>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='Makefile'
                          and
                          @role='variable'">
              <xsl:text>)</xsl:text>
            </xsl:if>
            <xsl:value-of select="$delim"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>\verb</xsl:text>
            <xsl:value-of select="$delim"/>
            <xsl:if test="(name()='command'
                           or
                           @role='length')
                          and
                          (@language='TeX'
                           or
                           @language='LaTeXe'
                           or
                           @language='LaTeX'
                           or
                           not(@language))">
              <xsl:text>\</xsl:text>
            </xsl:if>
            <xsl:if test="@language='XSLT'
                          and
                          (name()='varname' and
                            (@role='variable' or
                             @role='parameter'))
                          or
                          name()='parameter'">
              <xsl:text>$</xsl:text>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='Makefile'
                          and
                          @role='variable'">
              <xsl:text>$(</xsl:text>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='BiBTeX'
                          and
                          @role='entrytype'">
              <xsl:text>@</xsl:text>
            </xsl:if>
            <xsl:value-of select="normalize-space(.)"/>
            <xsl:if test="@xlink:type">
              <xsl:text>.</xsl:text>
              <xsl:value-of select="@xlink:type"/>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='Makefile'
                          and
                          @role='variable'">
              <xsl:text>)</xsl:text>
            </xsl:if>
            <xsl:value-of select="$delim"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="matches(.,'.*[\\\$\^{}&lt;\|&gt;].*')
                      and
                      (ancestor::db:title or
                       ancestor::db:footnote or
                       ancestor::db:term)">
        <xsl:message>
          <xsl:text>Warning: verbatim in </xsl:text>
          <xsl:value-of select="name(ancestor::*[3])"/>
          <xsl:text>/</xsl:text>
          <xsl:value-of select="name(ancestor::*[2])"/>
          <xsl:text>/</xsl:text>
          <xsl:value-of select="name(ancestor::*[1])"/>
          <xsl:text> for "</xsl:text>
          <xsl:value-of select="normalize-space(.)"/>
          <xsl:text>"</xsl:text>
        </xsl:message>
        <xsl:text>\url{</xsl:text>
        <xsl:value-of select="normalize-space(.)"/>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when
            test="matches(.,'.*[&amp;_~#%].*')
                  and
                  not(db:replaceable or @condition='nolit')">
            <xsl:text>\url{</xsl:text>
            <xsl:value-of select="normalize-space(.)"/>
            <xsl:text>}</xsl:text>
          </xsl:when>
          <xsl:when
            test="db:replaceable or @condition='nolit'">
            <xsl:text>{\ttfamily{}</xsl:text>
            <xsl:if
              test="(name()='command'
                     or
                     @role='length')
                    and
                    (@language='TeX'
                     or
                     @language='LaTeXe'
                     or
                     @language='LaTeX'
                     or
                     not(@language))">
              <xsl:text>\textbackslash{}</xsl:text>
            </xsl:if>
            <xsl:if test="@language='XSLT'
                          and
                          (name()='varname' and
                            (@role='variable' or
                             @role='parameter'))
                          or
                          name()='parameter'">
              <xsl:text>\$</xsl:text>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='Makefile'
                          and
                          @role='variable'">
              <xsl:text>\$(</xsl:text>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='BiBTeX'
                          and
                          @role='entrytype'">
              <xsl:text>@</xsl:text>
            </xsl:if>
            <xsl:apply-templates/>
            <xsl:if test="@xlink:type">
              <xsl:text>.</xsl:text>
              <xsl:value-of select="@xlink:type"/>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='Makefile'
                          and
                          @role='variable'">
              <xsl:text>)</xsl:text>
            </xsl:if>
            <xsl:text>}</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>{\ttfamily{}</xsl:text>
            <xsl:if
              test="(name()='command'
                     or
                     @role='length')
                    and
                    (@language='TeX'
                     or
                     @language='LaTeXe'
                     or
                     @language='LaTeX'
                     or
                     not(@language))">
              <xsl:text>\textbackslash{}</xsl:text>
            </xsl:if>
            <xsl:if test="@language='XSLT'
                          and
                          (name()='varname' and
                            (@role='variable' or
                             @role='parameter'))
                          or
                          name()='parameter'">
              <xsl:text>\$</xsl:text>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='bash'
                          and
                          @role='variable'">
              <xsl:text>\$</xsl:text>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='Makefile'
                          and
                          @role='variable'">
              <xsl:text>\$(</xsl:text>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='BiBTeX'
                          and
                          @role='entrytype'">
              <xsl:text>@</xsl:text>
            </xsl:if>
            <xsl:value-of select="normalize-space(.)"/>
            <xsl:if test="@xlink:type">
              <xsl:text>.</xsl:text>
              <xsl:value-of select="@xlink:type"/>
            </xsl:if>
            <xsl:if test="name()='varname'
                          and
                          @language='Makefile'
                          and
                          @role='variable'">
              <xsl:text>)</xsl:text>
            </xsl:if>
            <xsl:text>}</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
    <!-- finally, does it need an index entry?
    <xsl:variable name="gi" select="local-name()"/>
    <xsl:if test="tokenize(/db:book/@xreflabel,':')[.=$gi]
                  and not(ancestor::db:warning)">
      <xsl:text>\index{</xsl:text>
      <xsl:value-of select="replace(.,'_','\\_')"/>
      <xsl:text>@\textbf{</xsl:text>
      <xsl:value-of select="replace(.,'_','\\_')"/>
      <xsl:text>}}</xsl:text>
    </xsl:if>
    -->
  </xsl:template>
  <xsl:template name="htrim">
    <xsl:param name="text"/>
    <xsl:variable name="TEXT"
      select="translate($text,'&#x9;',' ')"/>
    <xsl:choose>
      <xsl:when test="contains($TEXT,'  ')">
        <xsl:call-template name="htrim">
          <xsl:with-param name="text">
            <xsl:value-of
              select="substring-before($TEXT,'  ')"/>
            <xsl:text> </xsl:text>
            <xsl:value-of
              select="substring-after($TEXT,'  ')"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="contains($TEXT,'&#xa; ')">
        <xsl:call-template name="htrim">
          <xsl:with-param name="text">
            <xsl:value-of
              select="substring-before($TEXT,'&#xa; ')"/>
            <xsl:text>&#xa;</xsl:text>
            <xsl:value-of
              select="substring-after($TEXT,'&#xa; ')"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$TEXT"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template name="lrtrim">
    <xsl:param name="text"/>
    <xsl:param name="indent"/>
    <xsl:variable name="extent"
      select="string-length($text)"/>
    <xsl:choose>
      <xsl:when test="substring($text,1,1)='&#xa;'">
        <xsl:call-template name="lrtrim">
          <xsl:with-param name="text"
            select="substring($text,2)"/>
          <xsl:with-param name="indent"
            select="$indent"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="substring($text,$extent)=' ' or
                      substring($text,$extent)='&#xa;' or
                      substring($text,$extent)='&#x9;'">
        <xsl:call-template name="lrtrim">
          <xsl:with-param name="text"
            select="substring($text,1,($extent - 1))"/>
          <xsl:with-param name="indent" select="$indent"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$indent!=''">
        <xsl:call-template name="addindent">
          <xsl:with-param name="text" select="$text"/>
          <xsl:with-param name="indent" select="$indent"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template name="addindent">
    <xsl:param name="text"/>
    <xsl:param name="indent"/>
    <xsl:value-of select="$indent"/>
    <xsl:choose>
      <xsl:when test="contains($text,'&#xa;')">
        <xsl:value-of
          select="substring-before($text,'&#xa;')"/>
        <xsl:text>&#xa;</xsl:text>
        <xsl:call-template name="addindent">
          <xsl:with-param name="text"
            select="substring-after($text,'&#xa;')"/>
          <xsl:with-param name="indent"
            select="$indent"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template name="breakline">
    <xsl:param name="string"/>
    <xsl:param name="max">
      <xsl:value-of select="$maxcodelen"/>
    </xsl:param>
    <xsl:choose>
      <xsl:when test="string-length($string)>$max">
        <!-- see if there is a substring that will fit -->
        <xsl:choose>
          <!-- if no spaces, we're stuck, so use as-is -->
          <xsl:when test="not(contains($string,' '))">
            <xsl:value-of select="$string"/>
          </xsl:when>
          <!-- if first substring fits, output/repeat -->
          <xsl:when
            test="string-length(substring-before($string,
                                ' '))&lt;=$max">
            <xsl:value-of select="$max"/>
            <xsl:text>/</xsl:text>
            <xsl:value-of
              select="string-length(substring-before($string,
                                    ' '))"/>
            <xsl:value-of
              select="substring-before($string,' ')"/>
            <xsl:text> </xsl:text>
            <!-- see how much we have left -->
            <xsl:choose>
              <xsl:when
                test="($max - 1 -
                      string-length(substring-before($string,
                                    ' ')))
                      &lt;
                      string-length(substring-before(
                                    substring-after($string,
                                    ' '), ' '))">
                <xsl:text>&#xa;   </xsl:text>
                <xsl:call-template name="breakline">
                  <xsl:with-param name="string"
                    select="substring-after($string,' ')"/>
                </xsl:call-template>
              </xsl:when>
              <xsl:otherwise>
                <xsl:call-template name="breakline">
                  <xsl:with-param name="string"
                    select="substring-after($string,' ')"/>
                  <xsl:with-param name="max"
                    select="$max - 1 -
                            string-length(substring-before(
                            $string,' '))"/>
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$string"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template name="compensate-space">
    <xsl:if test="preceding-sibling::node() and
                  preceding-sibling::* and
                  count(preceding-sibling::node()[1] |
                        preceding-sibling::*[1])=1">
      <xsl:text> </xsl:text>
    </xsl:if>
  </xsl:template>
  <xsl:template name="repeatarg">
    <xsl:param name="limit"/>
    <xsl:param name="count">
      <xsl:text>0</xsl:text>
    </xsl:param>
    <xsl:param name="prefix"/>
    <xsl:param name="suffix"/>
    <xsl:if test="$count&lt;$limit">
      <xsl:value-of select="$prefix"/>
      <xsl:text>#</xsl:text>
      <xsl:value-of select="$count + 1"/>
      <xsl:value-of select="$suffix"/>
      <xsl:call-template name="repeatarg">
        <xsl:with-param name="limit" select="$limit"/>
        <xsl:with-param name="count" select="$count + 1"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <xsl:template name="copyright-statement">
    <xsl:param name="ftype"/>
    <xsl:text>% Transformed from </xsl:text>
    <xsl:value-of select="$docname"/>
    <xsl:text>.xml by ClassPack db2dtx.xsl
    <xsl:value-of select="$thisversion"/>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="$thisverdate"/>
    <xsl:text>) on </xsl:text>
    <xsl:value-of
      select="format-dateTime(current-dateTime(),
              '[FNn] [D] [MNn] [Y] at [H]:[m]:[s]')"/>
    <xsl:text>&#xa;%&#xa;% </xsl:text>
    <xsl:value-of select="$docname"/>
    <xsl:text>.</xsl:text>
    <xsl:value-of select="$ftype"/>
    <xsl:text> is copyright © </xsl:text>
    <xsl:variable name="copyright-first">
      <xsl:for-each
        select="//db:info/db:revhistory/db:revision">
        <xsl:sort select="@version" order="descending"/>
        <xsl:if test="position()=last()">
          <xsl:value-of
            select="substring(db:date/@YYYY-MM-DD,1,4)"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>
    <xsl:value-of select="$copyright-first"/>
    <xsl:variable name="copyright-latest"
      select="substring($date,1,4)"/>
    <xsl:if test="$copyright-latest > $copyright-first">
      <xsl:text>-</xsl:text>
      <xsl:value-of select="$copyright-latest"/>
    </xsl:if>
    <xsl:text> by </xsl:text>
    <!-- RIGHTS -->
    <xsl:choose>
      <xsl:when test="/db:book/@audience='lppl'">
        <xsl:value-of
          select="normalize-space(//db:info//db:author[1]/
                  db:personname/db:firstname)"/>
        <xsl:text> </xsl:text>
        <xsl:value-of
          select="normalize-space(//db:info//db:author[1]/
                  db:personname/db:surname)"/>
        <xsl:text> &lt;</xsl:text>
        <xsl:value-of
          select="//db:info//db:author[1]/db:email"/>
        <xsl:text>>&#xa;</xsl:text>
        <xsl:text>%
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of
          select="normalize-space(
                  //db:info/db:copyright/db:holder)"/>
        <xsl:text> &lt;</xsl:text>
        <xsl:value-of
          select="substring-after(
                  //db:info/db:copyright/db:holder
                  /@xlink:href,'mailto:')"/>
        <xsl:text>>&#xa;</xsl:text>
        <xsl:text>%
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>% The current maintainer of this work is </xsl:text>
    <xsl:choose>
      <xsl:when
        test="//db:info//db:author[@role='maintainer']">
        <xsl:for-each
          select="//db:info//db:author
                  [@role='maintainer']">
          <xsl:value-of
            select="normalize-space(
                    db:personname/db:firstname)"/>
          <xsl:text> </xsl:text>
          <xsl:value-of
            select="normalize-space(
                    db:personname/db:surname)"/>
          <xsl:text> &lt;</xsl:text>
          <xsl:value-of select="db:email"/>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of
          select="normalize-space(
                  //db:info//db:author/
                  db:personname/db:firstname)"/>
        <xsl:text> </xsl:text>
        <xsl:value-of
          select="normalize-space(
                  //db:info//db:author/
                  db:personname/db:surname)"/>
        <xsl:text> &lt;</xsl:text>
        <xsl:value-of
          select="//db:info//db:author/db:email"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>>&#xa;</xsl:text>
    <xsl:text>%
    <xsl:value-of select="$docname"/>
    <xsl:text>.dtx and </xsl:text>
    <xsl:value-of select="$docname"/>
    <xsl:text>.ins,</xsl:text>
    <xsl:if test="/db:book/db:part
                  [@xml:id='code']
                  /db:appendix">
      <xsl:text>&#xa;% the derived file</xsl:text>
      <xsl:if test="count(/db:book/db:part
                    [@xml:id='code']
                    /db:appendix)>0">
        <xsl:text>s</xsl:text>
      </xsl:if>
      <xsl:text> </xsl:text>
      <xsl:for-each
        select="db:book/db:part
                [@xml:id='code']
                /(db:chapter|db:appendix)[@xlink:href]">
        <xsl:value-of select="@xlink:href"/>
        <xsl:text>, </xsl:text>
      </xsl:for-each>
    </xsl:if>
    <xsl:text>&#xa;% and any other ancillary files </xsl:text>
    <xsl:text>listed in the MANIFEST.&#xa;</xsl:text>
  </xsl:template>
  <xsl:template name="dtxtext">
    <xsl:param name="text"/>
    <xsl:choose>
      <xsl:when test="normalize-space(.)=''">
        <xsl:text></xsl:text>
      </xsl:when>
      <xsl:when test="starts-with($text,'&#x9;')">
        <xsl:call-template name="dtxtext">
          <xsl:with-param name="text"
            select="substring($text,2)"/>
        </xsl:call-template>
      </xsl:when>
      <!-- if it starts with a newline AND
           it’s in a <programlisting> element AND
           this is the first span of its type, prefix
           a percent -->
      <xsl:when test="starts-with($text,'&#xa;') and
                      parent::db:programlisting and
                      count(preceding-sibling::text())=0">
        <xsl:text>% </xsl:text>
        <xsl:call-template name="dtxtext">
          <xsl:with-param name="text"
            select="substring($text,2)"/>
        </xsl:call-template>
      </xsl:when>
      <!-- break strings containing newlines -->
      <xsl:when test="contains($text,'&#xa;')">
        <xsl:value-of
          select="substring-before($text,'&#xa;')"/>
        <xsl:choose>
          <xsl:when
            test="normalize-space(substring-after(
                                  $text,'&#xa;'))=''
                  and
                  parent::db:programlisting
                  and
                  count(following-sibling::text())=0">
            <xsl:text></xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>&#xa;% </xsl:text>
            <xsl:call-template name="dtxtext">
              <xsl:with-param name="text"
                select="substring-after($text,'&#xa;')"/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <!-- convert underscores in filenames -->
      <xsl:when
        test="ancestor::db:filename[db:replaceable] and
              contains($text,'_')">
        <xsl:value-of select="substring-before($text,'_')"/>
        <xsl:text>\_</xsl:text>
        <xsl:call-template name="dtxtext">
          <xsl:with-param name="text"
            select="substring-after($text,'_')"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template name="delogify">
    <xsl:param name="string"/>
    <xsl:choose>
      <xsl:when test="contains($string,'\LaTeXe{}')">
        <xsl:call-template name="delogify">
          <xsl:with-param name="string">
            <xsl:value-of
              select="substring-before($string,'\LaTeXe{}')"/>
            <xsl:text>LaTeX2e</xsl:text>
            <xsl:value-of
              select="substring-after($string,'\LaTeXe{}')"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="contains($string,'\LaTeX{}')">
        <xsl:call-template name="delogify">
          <xsl:with-param name="string">
            <xsl:value-of
              select="substring-before($string,'\LaTeX{}')"/>
            <xsl:text>LaTeX</xsl:text>
            <xsl:value-of
              select="substring-after($string,'\LaTeX{}')"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="contains($string,'\TeX{}')">
        <xsl:call-template name="delogify">
          <xsl:with-param name="string">
            <xsl:value-of
              select="substring-before($string,'\TeX{}')"/>
            <xsl:text>TeX</xsl:text>
            <xsl:value-of
              select="substring-after($string,'\TeX{}')"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$string"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="db:info" mode="ins">
    <xsl:variable name="file">
      <xsl:value-of select="$docname"/>
      <xsl:text>.ins</xsl:text>
    </xsl:variable>
    <xsl:result-document format="textFormat" href="{$file}">
      <xsl:call-template name="copyright-statement">
        <xsl:with-param name="ftype">
          <xsl:text>ins</xsl:text>
        </xsl:with-param>
      </xsl:call-template>
      <xsl:text>%
\input docstrip.tex
\keepsilent
\usedir{</xsl:text>
      <xsl:choose>
        <xsl:when test="/db:book/@xml:base">
          <xsl:value-of select="/db:book/@xml:base"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>tex/latex</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>}&#xa;</xsl:text>
      <xsl:choose>
        <xsl:when test="$doctype='script'">
          <xsl:text>\nopreamble&#xa;</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>\preamble

This is a generated file.

Copyright © </xsl:text>
          <xsl:value-of
            select="format-date(db:copyright
                    /db:year/@YYYY-MM-DD,'[Y]')"/>
          <!-- omit range if copyright is this year -->
          <xsl:if
            test="number(
                  substring(
                  db:copyright/db:year/@YYYY-MM-DD,1,4))
                  &lt; year-from-date(current-date())">
            <xsl:text>-</xsl:text>
            <xsl:value-of
              select="format-date(current-date(),'[Y]')"/>
          </xsl:if>
          <xsl:text> by </xsl:text>
          <xsl:value-of
            select="normalize-space(db:copyright
                    /db:holder)"/>
          <xsl:text>&#xa;&#xa;</xsl:text>
          <xsl:for-each select="db:annotation/*">
            <xsl:call-template name="htrim">
              <xsl:with-param name="text">
                <xsl:call-template name="lrtrim">
                  <xsl:with-param name="text" select="."/>
                </xsl:call-template>
              </xsl:with-param>
            </xsl:call-template>
            <xsl:text>&#xa;&#xa;</xsl:text>
          </xsl:for-each>
          <xsl:text>\endpreamble&#xa;</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
        <!-- inactive due to errors
        <xsl:for-each
          select="/db:book/db:part[@xml:id='files']
                  /db:chapter[db:programlisting]">
          <xsl:variable name="ext"
            select="substring-after(@xlink:href,'.')"/>
          <xsl:text>\declarepreamble\</xsl:text>
          <xsl:value-of select="@xml:id"/>
          <xsl:text>
\DoubleperCent\space This is file </xsl:text>
          <xsl:value-of select="@xlink:href"/>
          <xsl:text>&#xa;\endpreamble&#xa;</xsl:text>
          <xsl:text>\declarepostamble\</xsl:text>
          <xsl:value-of select="@xml:id"/>
          <xsl:text>
\DoubleperCent\space End of file </xsl:text>
          <xsl:value-of select="@xlink:href"/>
          <xsl:text>&#xa;\endpreamble&#xa;</xsl:text>
        </xsl:for-each>
        -->
        <xsl:text>\generate{</xsl:text>
        <!-- 1. Generate the main file (cls or sty) first -->
        <xsl:text>\file{</xsl:text>
        <xsl:value-of select="$name"/>
        <xsl:text>.</xsl:text>
        <xsl:value-of select="$filetype"/>
        <xsl:text>}{\from{</xsl:text>
        <xsl:value-of select="$docname"/>
        <xsl:text>.dtx}{</xsl:text>
        <xsl:value-of select="$doctype"/>
        <xsl:text>}}&#xa;</xsl:text>
        <!-- 2. Then any new files FROM WITHIN THE DOCUMENTATION (!)
             specially designated -->
        <xsl:for-each
          select="//db:programlisting
                  [@xml:id
                   and
                   @xlink:show='new'
                   and
                   @xlink:href]">
          <xsl:variable name="ext"
            select="substring-after(@xlink:href,'.')"/>
          <xsl:choose>
            <xsl:when test="$ext='tex'">
              <xsl:text>          \usepreamble\</xsl:text>
              <xsl:value-of select="@xml:id"/>
              <xsl:text>preamble\usepostamble\</xsl:text>
              <xsl:value-of select="@xml:id"/>
              <xsl:text>postamble</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>          </xsl:text>
              <xsl:text>\usepreamble\empty</xsl:text>
              <xsl:text>\usepostamble\empty</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>&#xa;          \file{</xsl:text>
          <xsl:value-of select="@xlink:href"/>
          <xsl:text>}{\from{</xsl:text>
          <xsl:value-of select="$docname"/>
          <xsl:text>.dtx}{</xsl:text>
          <xsl:value-of select="@xml:id"/>
          <xsl:text>}}&#xa;</xsl:text>
        </xsl:for-each>
        <!-- 3. Then the files with annotations -->
        <xsl:for-each
          select="/db:book/db:part[@xml:id='code']
                  /db:appendix
                    [descendant::db:programlisting
                     and
                     @xml:id
                     and
                     @xlink:href
                     and
                     not(@userlevel='cp')]">
          <!-- check omitted cp files are in manifest -->
          <xsl:if test="@userlevel='cp' and
                        id('manifest')/db:simplelist/db:member
                          [.=current()/@xlink:href]">
            <xsl:message>
              <xsl:text>Warning: file </xsl:text>
              <xsl:value-of select="@xlink:href"/>
              <xsl:text>not being generated but in manifest</xsl:text>
            </xsl:message>
          </xsl:if>
          <xsl:variable name="ext"
            select="substring-after(@xlink:href,'.')"/>
          <xsl:choose>
            <xsl:when test="$ext='tex'">
              <xsl:text>          \usepreamble\</xsl:text>
              <xsl:value-of select="@xml:id"/>
              <xsl:text>preamble\usepostamble\</xsl:text>
              <xsl:value-of select="@xml:id"/>
              <xsl:text>postamble&#xa;</xsl:text>
            </xsl:when>
            <xsl:when test="$ext='cls' or $ext='sty'">
<!--
  <xsl:text>          \usepreamble\usepostamble</xsl:text>
-->
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>          </xsl:text>
              <xsl:text>\usepreamble\empty</xsl:text>
              <xsl:text>\usepostamble\empty&#xa;</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>          \file{</xsl:text>
          <xsl:value-of select="@xlink:href"/>
          <xsl:text>}{\from{</xsl:text>
          <xsl:value-of select="$docname"/>
          <xsl:text>.dtx}{</xsl:text>
          <xsl:value-of select="@xml:id"/>
          <xsl:text>}}&#xa;</xsl:text>
        </xsl:for-each>
        <!-- 4. Then the files extracted whole without annotation -->
        <xsl:for-each
          select="/db:book/db:part[@xml:id='files']
                  /db:chapter
                  [db:programlisting
                   and
                   @xml:id
                   and @xlink:href]">
          <xsl:variable name="ext"
            select="substring-after(@xlink:href,'.')"/>
          <xsl:choose>
            <xsl:when test="$ext='TeX'">
              <xsl:text>          \usepreamble\</xsl:text>
              <xsl:value-of select="@xml:id"/>
              <xsl:text>preamble\usepostamble\</xsl:text>
              <xsl:value-of select="@xml:id"/>
              <xsl:text>postamble</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>          </xsl:text>
              <xsl:text>\usepreamble\empty</xsl:text>
              <xsl:text>\usepostamble\empty</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>&#xa;          \file{</xsl:text>
          <xsl:value-of select="@xlink:href"/>
          <xsl:text>}{\from{</xsl:text>
          <xsl:value-of select="$docname"/>
          <xsl:text>.dtx}{</xsl:text>
          <xsl:value-of select="@xml:id"/>
          <xsl:text>}}&#xa;</xsl:text>
        </xsl:for-each>
        <xsl:text>}
\obeyspaces
\Msg{********************************************************}
\Msg{**                                                    **}
\Msg{** Read the documentation before using </xsl:text>
        <xsl:choose>
          <xsl:when
            test="not(db:part[@xml:id='class']
                  /descendant::db:appendix[@xml:id])">
            <xsl:text>this </xsl:text>
            <xsl:value-of select="$doctype"/>
            <xsl:text>.    </xsl:text>
          </xsl:when>
          <xsl:when
            test="$doctype='cls' and
                  not(db:part[@xml:id='class']
                  /descendant::db:appendix[@xml:id]
                  [substring-after(@xlink:href,'.')!='cls'])">
            <xsl:text>these classes. </xsl:text>
          </xsl:when>
          <xsl:when
            test="$doctype='sty' and
                  not(db:part[@xml:id='class']
                  /descendant::db:appendix[@xml:id]
                  [substring-after(@xlink:href,'.')!='sty'])">
            <xsl:text>these packages.</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>these files.   </xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>**}
\Msg{**                                                    **}
\Msg{** To finish the installation you have to move the    **}
\Msg{** following file</xsl:text>
        <xsl:if
          test="count(/db:book/db:part[@xml:id='files']
                      /db:chapter[db:programlisting and
                                  @xml:id and @xlink:href]) +
                count(/db:book/db:part[@xml:id='code']
                      /db:appendix[db:programlisting and
                                   @xml:id and @xlink:href]) + 1
                &gt; 1">
          <xsl:text>s</xsl:text>
        </xsl:if>
        <xsl:text> into a directory searched by TeX:  </xsl:text>
        <xsl:if
          test="count(/db:book/db:part[@xml:id='files']
                      /db:chapter[db:programlisting and
                                  @xml:id and @xlink:href]) +
                count(/db:book/db:part[@xml:id='code']
                      /db:appendix[db:programlisting and
                                   @xml:id and @xlink:href]) + 1
                = 1">
          <xsl:text> </xsl:text>
        </xsl:if>
        <xsl:text>**}
\Msg{**                                                    **}
</xsl:text>
        <!-- class or package first -->
        <xsl:text>\Msg{**     </xsl:text>
        <xsl:value-of
          select="/db:book/db:part
                  [@xml:id='code']
                  /db:chapter[1]/@xlink:href"/>
        <xsl:variable name="spacetaken"
          select="4 +
                  string-length(/db:book/db:part[@xml:id='code']
                                /db:chapter[1]/@xlink:href)
                  + 4"/>
        <xsl:value-of
          select="for $s in (1 to
                    xs:integer((50 - $spacetaken) div 2))
                  return '&#x20;'"/>
        <xsl:text>      **}</xsl:text>
        <!-- remaining files -->
        <xsl:for-each
          select="/db:book/db:part[@xml:id='files']
                  /db:chapter[descendant::db:programlisting
                              and
                              @xml:id and @xlink:href]
                  |
                  /db:book/db:part[@xml:id='code']
                  /db:appendix[descendant::db:programlisting
                               and
                               @xml:id and @xlink:href]">
          <xsl:text>&#xa;\Msg{**     </xsl:text>
          <xsl:value-of select="@xlink:href"/>
          <xsl:variable name="spacetaken"
            select="string-length(@xlink:href)"/>
          <xsl:value-of
            select="for $s in (1 to
                    xs:integer((46 - $spacetaken) div 2))
                    return '&#x20;'"/>
          <xsl:if test="($spacetaken div 2) !=
                        (xs:integer($spacetaken) div 2)">
            <xsl:text>    </xsl:text>
          </xsl:if>
          <xsl:text>  **}</xsl:text>
        </xsl:for-each>
        <xsl:text>
\Msg{**                                                    **}
\Msg{** To produce the documentation run the file          **}
\Msg{** </xsl:text>
        <xsl:value-of select="$docname"/>
        <xsl:text>.dtx through </xsl:text>
        <!-- space available is 50 chars less the word 'through' (7)
             and the two spaces either side, less the width of the
             class/package name plus 4 for the extension, less the
             width of the program names and their spaces -->
        <xsl:choose>
          <xsl:when test="//db:part[@conformance='lualatex']">
            <xsl:text>XeLaTeX and biber</xsl:text>
            <xsl:variable name="spacetaken"
              select="string-length($docname) + 4 + 9 + 17"/>
            <xsl:value-of
              select="for $s in (1 to
                        xs:integer((50 - $spacetaken) div 2))
                      return '&#x20;'"/>
          </xsl:when>
          <xsl:when test="//db:part[@conformance='xelatex']">
            <xsl:text>XeLaTeX and biber</xsl:text>
            <xsl:variable name="spacetaken"
              select="string-length($docname) + 4 + 9 + 17"/>
            <xsl:value-of
              select="for $s in (1 to
                        xs:integer((50 - $spacetaken) div 2))
                      return '&#x20;'"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>pdfLaTeX and BiBTeX</xsl:text>
            <xsl:variable name="spacetaken"
              select="string-length($docname) + 4 + 9 + 19"/>
            <xsl:value-of
              select="for $s in (1 to
                        xs:integer((50 - $spacetaken) div 2))
                      return ' '"/>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>   **}
\Msg{**                                                    **}
\Msg{** Happy LaTeXing!                                    **}
\Msg{**                                                    **}
\Msg{********************************************************}
\endbatchfile
</xsl:text>
    </xsl:result-document>
  </xsl:template>
  <!--
  <xsl:template
    match="db:part/db:title |
           db:chapter/db:title |
           db:chapter/db:subtitle |
           db:chapter/db:para" mode="files"/>
-->

  <xsl:template match="db:part/db:title" mode="files">
    <xsl:text>% \section{</xsl:text>
    <xsl:apply-templates mode="files"/>
    <xsl:text>}\label{</xsl:text>
    <xsl:value-of select="../@xml:id"/>
    <xsl:text>}&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="db:chapter" mode="files">
    <xsl:apply-templates mode="files"/>
  </xsl:template>

  <xsl:template match="db:chapter/db:title" mode="files">
    <xsl:text>% \subsection{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}\label{</xsl:text>
    <xsl:value-of select="../@xml:id"/>
    <xsl:text>}&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="db:warning" mode="files">
    <xsl:call-template name="sidewarn"/>
  </xsl:template>

  <xsl:template match="db:chapter/db:para
                       |
                       db:part/db:para" mode="files">
    <xsl:text>% </xsl:text>
    <xsl:apply-templates/>
    <xsl:text>&#xa;</xsl:text>
  </xsl:template>

  <!--
emphasis
filename
package
productname
tag
xref
-->

  <xsl:template match="db:programlisting" mode="files">
    <xsl:choose>
      <xsl:when test="ancestor::db:part[@xml:id='files']">
        <xsl:text>% \iffalse&#xa;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>%    </xsl:text>
        <xsl:text>\begin</xsl:text>
        <xsl:text>{macrocode}&#xa;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <!-- create the latexdoc tag -->
    <xsl:text>%&lt;*</xsl:text>
    <xsl:value-of select="parent::db:chapter/@xml:id"/>
    <xsl:text>>&#xa;</xsl:text>
    <xsl:call-template name="lrtrim">
      <xsl:with-param name="text"
        select="unparsed-text(concat($docname,'/',@xlink:href))"/>
    </xsl:call-template>
    <!-- end tag -->
    <xsl:text>&#xa;%&lt;/</xsl:text>
    <xsl:value-of select="parent::db:chapter/@xml:id"/>
    <xsl:text>>&#xa;</xsl:text>
    <xsl:choose>
      <xsl:when test="ancestor::db:part[@xml:id='files']">
        <xsl:text>% \fi&#xa;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>%    </xsl:text>
        <xsl:text>\end</xsl:text>
        <xsl:text>{macrocode}&#xa;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:variable name="TeX">
    <span class="TeX">T<span
    class="E">E</span>X</span>
  </xsl:variable>
  <xsl:variable name="LaTeX">
    <span class="LaTeX">L<span
    class="A">A</span><span
    class="TeX">T<span
    class="E">E</span>X</span></span>
  </xsl:variable>
  <xsl:template name="webindex">
    <xsl:message>
      <xsl:text>Making index.html for web site</xsl:text>
    </xsl:message>
    <xsl:result-document format="htmlFormat" href="index.html">
      <html>
        <head>
          <title>LaTeX Packages and Document Classes
          from Silmaril</title>
          <meta charset="utf-8"/>
          <meta name="viewport"
            content="width=device-width,initial-scale=1,
                     user-scalable=no"/>
          <link rel="stylesheet" href="../assets/css/main.css"/>
          <noscript>
            <link rel="stylesheet"
              href="../assets/css/noscript.css"/>
          </noscript>
        </head>
        <body class="is-preload">
          <section id="top" class="wrapper style2">
            <header class="major">
              <h2>
                <xsl:copy-of select="$LaTeX"/>
                <xsl:text> style packages and document classes
                from Silmaril</xsl:text>
              </h2>
              <ul class="icons labeled">
                <li>
                  <span class="icon solid fa-book">
                    <span class="label">
                      <a href="../#downloads">Index</a>
                    </span>
                  </span>
                </li>
                <li>
                  <span class="icon solid fa-book-open">
                    <span class="label">
                      <a href="../#more">About</a>
                    </span>
                  </span>
                </li>
              </ul>
            </header>
            <div class="inner alt">
              <section class="spotlight" id="{$docname}">
                <div class="image">
                  <img src="{$docname}.png" alt="{$shorttitle}"/>
                </div>
                <div class="content">
                  <h3>
                    <xsl:text>The </xsl:text>
                    <a href="http://ctan.org/pkg/{$docname}">
                      <xsl:value-of select="$docname"/>
                    </a>
                    <xsl:text> package</xsl:text>
                  </h3>
                  <h4 class="subtitle">
                    <xsl:text>v </xsl:text>
                    <xsl:value-of select="/db:book/@version"/>
                    <xsl:text>.</xsl:text>
                    <xsl:value-of select="/db:book/@revision"/>
                    <xsl:text> of </xsl:text>
                    <xsl:value-of
                      select="/db:book/db:info
                              /db:revhistory/db:revision
                              [@version=max(parent::db:revhistory
                              /db:revision/@version)]
                              /db:date/@YYYY-MM-DD"/>
                    <xsl:text> </xsl:text>
                    <span class="reversed">©</span>
                    <xsl:text> </xsl:text>
                    <xsl:value-of
                      select="format-date($firstverdate,'[Y]')"/>
                    <xsl:text>–</xsl:text>
                    <xsl:value-of
                      select="format-date(current-date(),'[Y]')"/>
                    <xsl:text> </xsl:text>
                    <a href="www.latex-project.org/lppl/lppl-1-3c/">LPPL 
                      1.3c</a>
                  </h4>
                  <p>
                    <xsl:value-of select="/db:book/db:info/db:title"/>
                  </p>
                  <div class="table-wrapper">
                    <table>
                      <thead>
                        <tr>
                          <th>File</th>
                          <th>Description</th>
                        </tr>
                      </thead>
                      <tbody>
                        <!-- ZIP -->
                        <tr>
                          <td>
                            <tt>
                              <a href="{$docname}-{/db:book/@version}.{
                                       /db:book/@revision}.zip">
                                <xsl:value-of
                                  select="$docname"/>
                                <xsl:text>-</xsl:text>
                                <xsl:value-of
                                  select="/db:book/@version"/>
                                <xsl:text>.</xsl:text>
                                <xsl:value-of
                                  select="/db:book/@revision"/>
                                <xsl:text>.zip</xsl:text>
                              </a>
                            </tt>
                          </td>
                          <td>Zip file of the CTAN distribution</td>
                        </tr>
                        <!-- TDS ZIP -->
                        <tr>
                          <td>
                            <tt>
                              <a href="{$docname}-{/db:book/@version}.{
                                       /db:book/@revision}.tds.zip">
                                <xsl:value-of
                                  select="$docname"/>
                                <xsl:text>-</xsl:text>
                                <xsl:value-of
                                  select="/db:book/@version"/>
                                <xsl:text>.</xsl:text>
                                <xsl:value-of
                                  select="/db:book/@revision"/>
                                <xsl:text>.tds.zip</xsl:text>
                              </a>
                            </tt>
                          </td>
                          <td>TeX Directory Structure (TDS) zipfile</td>
                        </tr>
                        <!-- DTX -->
                        <tr>
                          <td>
                            <tt>
                              <a href="{$docname}.dtx">
                                <xsl:value-of select="$docname"/>
                                <xsl:text>.dtx</xsl:text>
                              </a>
                            </tt>
                          </td>
                          <td>
                            <xsl:text>Doc</xsl:text>
                            <xsl:copy-of select="$TeX"/>
                            <xsl:text> combined code and
                              documentation</xsl:text>
                          </td>
                        </tr>
                        <!-- INS -->
                        <tr>
                          <td>
                            <tt>
                              <a href="{$docname}.ins">
                                <xsl:value-of select="$docname"/>
                                <xsl:text>.ins</xsl:text>
                              </a>
                            </tt>
                          </td>
                          <td>
                            <xsl:text>Doc</xsl:text>
                            <xsl:copy-of select="$TeX"/>
                            <xsl:text> installer</xsl:text>
                          </td>
                        </tr>
                        <!-- PDF -->
                        <tr>
                          <td>
                            <tt>
                              <a href="{$docname}.pdf">
                                <xsl:value-of select="$docname"/>
                                <xsl:text>.pdf</xsl:text>
                              </a>
                            </tt>
                          </td>
                          <td>
                            <xsl:text>PDF documentation</xsl:text>
                          </td>
                        </tr>
                        <!-- CLS/STY -->
                        <tr>
                          <td>
                            <tt>
                              <a
                                href="{$docname}.{/db:book/@userlevel}">
                                <xsl:value-of select="$docname"/>
                                <xsl:text>.</xsl:text>
                                <xsl:value-of
                                  select="/db:book/@userlevel"/>
                              </a>
                            </tt>
                          </td>
                          <td>
                            <xsl:text>the </xsl:text>
                            <xsl:value-of select="/db:book/@arch"/>
                            <xsl:text> file</xsl:text>
                          </td>
                        </tr>
                        <!-- PNG -->
                        <!-- convert \
                             -density 300 <docname>.pdf[0] \
                             -quality 100 \
                             -flatten \
                             -sharpen 0x1.0 <docname>.png
                        -->
                        <tr>
                          <td>
                            <tt>
                              <a href="{$docname}.png">
                                <xsl:value-of select="$docname"/>
                                <xsl:text>.png</xsl:text>
                              </a>
                            </tt>
                          </td>
                          <td>associated image</td>
                        </tr>
                        <!-- extractables -->
                        <xsl:if
                          test="/db:book/db:part[@xml:id='files']">
                          <tr>
                            <xsl:for-each
                              select="/db:book/db:part
                                      [@xml:id='files']
                                      /db:chapter[@xml:id
                                      and
                                      @xlink:href
                                      and
                                      db:programlisting]">
                              <td>
                                <tt>
                                  <a href="{@xlink:href}">
                                    <xsl:value-of
                                      select="@xlink:href"/>
                                  </a>
                                </tt>
                              </td>
                              <td>
                                <xsl:value-of select="@role"/>
                              </td>
                            </xsl:for-each>
                          </tr>
                        </xsl:if>
                        <xsl:if
                          test="/db:book/db:info/db:cover
                                /db:constraintdef[@xml:id='manifest']">
                          <xsl:for-each
                            select="/db:book/db:info/db:cover/
                                    db:constraintdef
                                    [@xml:id='manifest']/
                                    db:simplelist/db:member">
                            <tr>
                              <td>
                                <tt>
                                  <a href="{.}">
                                    <xsl:value-of select="."/>
                                  </a>
                                </tt>
                              </td>
                              <td>
                                <xsl:value-of select="@role"/>
                              </td>
                            </tr>
                          </xsl:for-each>
                        </xsl:if>
                      </tbody>
                    </table>
                  </div>
                  <p>
                    <xsl:text>Use the CTAN zip file with
                      automated installers like </xsl:text>
                    <em>tlmgr</em>
                    <xsl:text> and MiK</xsl:text>
                    <xsl:copy-of select="$TeX"/>
                    <xsl:text>. Use the TDS zip file when
                      installing to your </xsl:text>
                    <a href="{concat('http://latex.silmaril.ie',
                             '/formattinginformation/personal.html')}">
                      <xsl:text>Personal </xsl:text>
                      <xsl:copy-of select="$TeX"/>
                      <xsl:text> Directory</xsl:text>
                    </a>
                    <xsl:text>. The individual DTX and INS files are
                      for manual installation.</xsl:text>
                  </p>
                  <ul class="icons">
                    <li>
                      <a href="MANIFEST">MANIFEST</a>
                    </li>
                    <li>
                      <a href="VERSION">VERSION</a>
                    </li>
                    <li>
                      <a href="README.md">README.md</a>
                    </li>
                    <li class="generated">
                      <xsl:text>ClassPack v </xsl:text>
                      <xsl:value-of select="$thisversion"/>
                      <br/>
                      <xsl:value-of select="format-date(current-date(),
                                            '[D01]-[M01]-[Y03]')"/>
                    </li>
                  </ul>
                  <ul class="actions">
                    <li>
                      <a class="button small primary scrolly"
                        href="#downloads">Downloads</a>
                    </li>
                    <li>
                      <a class="button small scrolly"
                        href="#more">Learn More</a>
                    </li>
                    <li>
                      <a class="button small scrolly"
                        href="#contact">Contact us</a>
                    </li>
                    <li>
                      <a class="button small scrolly"
                        href="#header">Home</a>
                    </li>
                  </ul>
                </div>
              </section>
            </div>
          </section>
          <footer id="footer">
            <ul class="icons">
              <li>
                <a href="news:comp.text.tex"
                  title="Usenet support"
                  style="font-family:monospace;"
                  class="icon solid fa-newspaper">
                  <span class="label">comp.text.tex</span>
                </a>
              </li>
              <li>
                <a href="https://mastodon.ie/@latex"
                  title="Talk to @latex on Mastodon"
                  class="icon brands fa-deskpro">
                  <span class="label">Mastodon</span>
                </a>
              </li>
              <li>
                <a href="https://twitter.com/silmaril_ie"
                  title="Talk to @silmaril_ie on Twitter"
                  class="icon brands fa-twitter">
                  <span class="label">Twitter</span>
                </a>
              </li>
              <li>
                <a
                  href="mailto:latex@silmaril.ie?subject=LaTeX support"
                  style="font-family:monospace;"
                  title="Email latex@silmaril.ie"
                  class="icon solid fa-envelope-open">
                  <span class="label">latex@silmaril.ie</span>
                </a>
              </li>
            </ul>
            <p class="copyright">
              <xsl:text>© </xsl:text>
              <xsl:value-of select="format-date($firstverdate,'[Y]')"/>
              <xsl:text>–</xsl:text>
              <xsl:value-of select="format-date(current-date(),'[Y]')"/>
              <xsl:text> </xsl:text>
              <a href="http://www.silmaril.ie/">Silmaril Consultants</a>
              <xsl:text>. Design: </xsl:text>
              <a href="http://html5up.net">HTML5 UP</a>
            </p>
          </footer>
          <script src="../assets/js/jquery.min.js"/>
          <script src="../assets/js/jquery.scrolly.min.js"/>
          <script src="../assets/js/browser.min.js"/>
          <script src="../assets/js/breakpoints.min.js"/>
          <script src="../assets/js/util.js"/>
          <script src="../assets/js/main.js"/>
        </body>
      </html>
    </xsl:result-document>
  </xsl:template>
  <xsl:template match="db:keywordset | db:keyword"/>
</xsl:stylesheet>

